@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -116,9 +116,12 @@ typedef struct VMCPU
     /** Which host CPU ID is this EMT running on.
      * Only valid when in RC or HMR0 with scheduling disabled. */
     RTCPUID volatile        idHostCpu;                              /* 56 / 36 */
+    /** The CPU set index corresponding to idHostCpu, UINT32_MAX if not valid.
+     * @remarks Best to make sure iHostCpuSet shares cache line with idHostCpu! */
+    uint32_t volatile       iHostCpuSet;                            /* 60 / 40 */
 
     /** Trace groups enable flags.  */
-    uint32_t                fTraceGroups;                           /* 60 / 40 */
+    uint32_t                fTraceGroups;                           /* 64 / 44 */
     /** Align the structures below bit on a 64-byte boundary and make sure it starts
      * at the same offset in both 64-bit and 32-bit builds.
      *
@@ -127,28 +130,19 @@ typedef struct VMCPU
      *          data could be lumped together at the end with a < 64 byte padding
      *          following it (to grow into and align the struct size).
      *   */
-    uint8_t                 abAlignment1[HC_ARCH_BITS == 64 ? 60 : 16+64];
+    uint8_t                 abAlignment1[HC_ARCH_BITS == 64 ? 56 : 12+64];
     /** State data for use by ad hoc profiling. */
     uint32_t                uAdHoc;
     /** Profiling samples for use by ad hoc profiling. */
     STAMPROFILEADV          aStatAdHoc[8];                          /* size: 40*8 = 320 */
 
-    /** CPUM part. */
-    union
-    {
-#ifdef ___CPUMInternal_h
-        struct CPUMCPU      s;
-#endif
-        uint8_t             padding[3584];      /* multiple of 64 */
-    } cpum;
-
     /** HM part. */
     union
     {
 #ifdef ___HMInternal_h
         struct HMCPU    s;
 #endif
-        uint8_t             padding[5568];      /* multiple of 64 */
+        uint8_t             padding[5760];      /* multiple of 64 */
     } hm;
 
     /** EM part. */
@@ -157,7 +151,7 @@ typedef struct VMCPU
 #ifdef ___EMInternal_h
         struct EMCPU        s;
 #endif
-        uint8_t             padding[1472];      /* multiple of 64 */
+        uint8_t             padding[1408];      /* multiple of 64 */
     } em;
 
     /** IEM part. */
@@ -224,8 +218,17 @@ typedef struct VMCPU
         uint8_t             padding[64];        /* multiple of 64 */
     } dbgf;
 
+    /** GIM part. */
+    union
+    {
+#ifdef ___GIMInternal_h
+        struct GIMCPU s;
+#endif
+        uint8_t             padding[64];      /* multiple of 64 */
+    } gim;
+
     /** Align the following members on page boundary. */
-    uint8_t                 abAlignment2[192];
+    uint8_t                 abAlignment2[3584];
 
     /** PGM part. */
     union
@@ -236,6 +239,15 @@ typedef struct VMCPU
         uint8_t             padding[4096];      /* multiple of 4096 */
     } pgm;
 
+    /** CPUM part. */
+    union
+    {
+#ifdef ___CPUMInternal_h
+        struct CPUMCPU      s;
+#endif
+        uint8_t             padding[4096];      /* multiple of 4096 */
+    } cpum;
+
 } VMCPU;
 
 
@@ -270,9 +282,9 @@ typedef struct VMCPU
 /** @} */
 
 
-/** The name of the Guest Context VMM Core module. */
-#define VMMGC_MAIN_MODULE_NAME          "VMMGC.gc"
-/** The name of the Ring 0 Context VMM Core module. */
+/** The name of the raw-mode context VMM Core module. */
+#define VMMRC_MAIN_MODULE_NAME          "VMMRC.rc"
+/** The name of the ring-0 context VMM Core module. */
 #define VMMR0_MAIN_MODULE_NAME          "VMMR0.r0"
 
 /**
@@ -358,6 +370,12 @@ typedef struct VMCPU
 #define VMCPU_FF_INTERRUPT_SMI              RT_BIT_32(VMCPU_FF_INTERRUPT_SMI_BIT)
 /** PDM critical section unlocking is pending, process promptly upon return to R3. */
 #define VMCPU_FF_PDM_CRITSECT               RT_BIT_32(5)
+/** This action forces the VCPU out of the halted state. */
+#define VMCPU_FF_UNHALT                     RT_BIT_32(6)
+/** Pending IEM action (bit number). */
+#define VMCPU_FF_IEM_BIT                    7
+/** Pending IEM action (mask). */
+#define VMCPU_FF_IEM                        RT_BIT_32(VMCPU_FF_IEM_BIT)
 /** This action forces the VM to service pending requests from other
  * thread or requests which must be executed in another context. */
 #define VMCPU_FF_REQUEST                    RT_BIT_32(9)
@@ -373,10 +391,10 @@ typedef struct VMCPU
 /** Same as VM_FF_PGM_SYNC_CR3 except that global pages can be skipped.
  * (NON-GLOBAL FLUSH) */
 #define VMCPU_FF_PGM_SYNC_CR3_NON_GLOBAL    RT_BIT_32(17)
-/** Check for pending TLB shootdown actions.
- * Consumer: HM
- * @todo rename to VMCPU_FF_HM_TLB_SHOOTDOWN  */
-#define VMCPU_FF_TLB_SHOOTDOWN              RT_BIT_32(18)
+/** Check for pending TLB shootdown actions (deprecated)
+ * Reserved for furture HM re-use if necessary / safe.
+ * Consumer: HM */
+#define VMCPU_FF_TLB_SHOOTDOWN_UNUSED       RT_BIT_32(18)
 /** Check for pending TLB flush action.
  * Consumer: HM
  * @todo rename to VMCPU_FF_HM_TLB_FLUSH  */
@@ -395,6 +413,8 @@ typedef struct VMCPU
 #endif /* VBOX_WITH_RAW_MODE */
 /** Inhibit interrupts pending. See EMGetInhibitInterruptsPC(). */
 #define VMCPU_FF_INHIBIT_INTERRUPTS         RT_BIT_32(24)
+/** Block injection of non-maskable interrupts to the guest. */
+#define VMCPU_FF_BLOCK_NMIS                 RT_BIT_32(25)
 #ifdef VBOX_WITH_RAW_MODE
 /** CSAM needs to scan the page that's being executed */
 # define VMCPU_FF_CSAM_SCAN_PAGE            RT_BIT_32(26)
@@ -414,6 +434,7 @@ typedef struct VMCPU
                                                  | VM_FF_PDM_QUEUES | VM_FF_PDM_DMA | VM_FF_EMT_RENDEZVOUS)
 /** Externally forced VMCPU actions. Used to quit the idle/wait loop. */
 #define VMCPU_FF_EXTERNAL_HALTED_MASK           (  VMCPU_FF_INTERRUPT_APIC | VMCPU_FF_INTERRUPT_PIC | VMCPU_FF_REQUEST \
+                                                 | VMCPU_FF_INTERRUPT_NMI  | VMCPU_FF_INTERRUPT_SMI | VMCPU_FF_UNHALT \
                                                  | VMCPU_FF_TIMER)
 
 /** High priority VM pre-execution actions. */
@@ -439,7 +460,7 @@ typedef struct VMCPU
 #define VM_FF_HIGH_PRIORITY_POST_MASK           (VM_FF_PGM_NO_MEMORY)
 /** High priority post-execution actions. */
 #define VMCPU_FF_HIGH_PRIORITY_POST_MASK        (  VMCPU_FF_PDM_CRITSECT | VM_WHEN_RAW_MODE(VMCPU_FF_CSAM_PENDING_ACTION, 0) \
-                                                 | VMCPU_FF_HM_UPDATE_CR3 | VMCPU_FF_HM_UPDATE_PAE_PDPES)
+                                                 | VMCPU_FF_HM_UPDATE_CR3 | VMCPU_FF_HM_UPDATE_PAE_PDPES | VMCPU_FF_IEM)
 
 /** Normal priority VM post-execution actions. */
 #define VM_FF_NORMAL_PRIORITY_POST_MASK         (  VM_FF_CHECK_VM_STATE | VM_FF_DBGF | VM_FF_RESET \
@@ -451,7 +472,7 @@ typedef struct VMCPU
 #define VM_FF_NORMAL_PRIORITY_MASK              (  VM_FF_REQUEST | VM_FF_PDM_QUEUES | VM_FF_PDM_DMA | VM_FF_REM_HANDLER_NOTIFY \
                                                  | VM_FF_EMT_RENDEZVOUS)
 /** Normal priority VMCPU actions. */
-#define VMCPU_FF_NORMAL_PRIORITY_MASK           (VMCPU_FF_REQUEST)
+#define VMCPU_FF_NORMAL_PRIORITY_MASK           (VMCPU_FF_REQUEST | VMCPU_FF_UNHALT)
 
 /** Flags to clear before resuming guest execution. */
 #define VMCPU_FF_RESUME_GUEST_MASK              (VMCPU_FF_TO_R3)
@@ -460,7 +481,7 @@ typedef struct VMCPU
 #define VM_FF_HM_TO_R3_MASK                     (  VM_FF_TM_VIRTUAL_SYNC | VM_FF_PGM_NEED_HANDY_PAGES | VM_FF_PGM_NO_MEMORY \
                                                  | VM_FF_PDM_QUEUES | VM_FF_EMT_RENDEZVOUS)
 /** VMCPU Flags that cause the HM loops to go back to ring-3. */
-#define VMCPU_FF_HM_TO_R3_MASK                  (VMCPU_FF_TO_R3 | VMCPU_FF_TIMER | VMCPU_FF_PDM_CRITSECT)
+#define VMCPU_FF_HM_TO_R3_MASK                  (VMCPU_FF_TO_R3 | VMCPU_FF_TIMER | VMCPU_FF_PDM_CRITSECT | VMCPU_FF_IEM)
 
 /** High priority ring-0 VM pre HM-mode execution mask. */
 #define VM_FF_HP_R0_PRE_HM_MASK                 (VM_FF_HM_TO_R3_MASK | VM_FF_REQUEST | VM_FF_PGM_POOL_FLUSH_PENDING | VM_FF_PDM_DMA)
@@ -486,8 +507,7 @@ typedef struct VMCPU
 /** All the forced VMCPU flags except those related to raw-mode and hardware
  * assisted execution. */
 #define VMCPU_FF_ALL_REM_MASK                   (~(  VMCPU_FF_HIGH_PRIORITY_PRE_RAW_MASK | VMCPU_FF_PDM_CRITSECT \
-                                                   | VMCPU_FF_TLB_FLUSH | VMCPU_FF_TLB_SHOOTDOWN \
-                                                   | VM_WHEN_RAW_MODE(VMCPU_FF_CSAM_PENDING_ACTION, 0) ))
+                                                   | VMCPU_FF_TLB_FLUSH | VM_WHEN_RAW_MODE(VMCPU_FF_CSAM_PENDING_ACTION, 0) ))
 /** @} */
 
 /** @def VM_FF_SET
@@ -655,10 +675,9 @@ typedef struct VMCPU
                                                       RTThreadNativeSelf(), (pVCpu) ? (pVCpu)->hNativeThreadR0 : 0, \
                                                       (pVCpu) ? (pVCpu)->idCpu : 0))
 #else
-# define VMCPU_ASSERT_EMT(pVCpu) \
-    AssertMsg(VMCPU_IS_EMT(pVCpu), \
-              ("Not emulation thread! Thread=%RTnthrd ThreadEMT=%RTnthrd idCpu=%#x\n", \
-              RTThreadNativeSelf(), (pVCpu)->hNativeThread, (pVCpu)->idCpu))
+# define VMCPU_ASSERT_EMT(pVCpu)            AssertMsg(VMCPU_IS_EMT(pVCpu), \
+                                                      ("Not emulation thread! Thread=%RTnthrd ThreadEMT=%RTnthrd idCpu=%#x\n", \
+                                                      RTThreadNativeSelf(), (pVCpu)->hNativeThread, (pVCpu)->idCpu))
 #endif
 
 /** @def VM_ASSERT_EMT_RETURN
@@ -684,10 +703,10 @@ typedef struct VMCPU
 # define VMCPU_ASSERT_EMT_RETURN(pVCpu, rc) AssertReturn(VMCPU_IS_EMT(pVCpu), (rc))
 #else
 # define VMCPU_ASSERT_EMT_RETURN(pVCpu, rc) \
-    AssertMsg(VMCPU_IS_EMT(pVCpu), \
-              ("Not emulation thread! Thread=%RTnthrd ThreadEMT=%RTnthrd idCpu=%#x\n", \
-               RTThreadNativeSelf(), (pVCpu)->hNativeThread, (pVCpu)->idCpu), \
-              (rc))
+    AssertMsgReturn(VMCPU_IS_EMT(pVCpu), \
+                    ("Not emulation thread! Thread=%RTnthrd ThreadEMT=%RTnthrd idCpu=%#x\n", \
+                     RTThreadNativeSelf(), (pVCpu)->hNativeThread, (pVCpu)->idCpu), \
+                    (rc))
 #endif
 
 /** @def VMCPU_ASSERT_EMT_OR_GURU
@@ -960,6 +979,18 @@ typedef struct VM
     {
 #ifdef ___CPUMInternal_h
         struct CPUM s;
+#endif
+#ifdef ___VBox_vmm_cpum_h
+        /** Read only info exposed about the host and guest CPUs.   */
+        struct
+        {
+            /** Padding for hidden fields. */
+            uint8_t                 abHidden0[64];
+            /** Host CPU feature information. */
+            CPUMFEATURES            HostFeatures;
+            /** Guest CPU feature information. */
+            CPUMFEATURES            GuestFeatures;
+        } const ro;
 #endif
         uint8_t     padding[1536];      /* multiple of 64 */
     } cpum;
@@ -1069,7 +1100,7 @@ typedef struct VM
 #ifdef ___TMInternal_h
         struct TM   s;
 #endif
-        uint8_t     padding[2432];      /* multiple of 64 */
+        uint8_t     padding[2496];      /* multiple of 64 */
     } tm;
 
     /** DBGF part. */
@@ -1108,6 +1139,14 @@ typedef struct VM
         uint8_t     padding[0x11100];   /* multiple of 64 */
     } rem;
 
+    union
+    {
+#ifdef ___GIMInternal_h
+        struct GIM s;
+#endif
+        uint8_t     padding[320];        /* multiple of 64 */
+    } gim;
+
     /* ---- begin small stuff ---- */
 
     /** VM part. */
@@ -1130,7 +1169,7 @@ typedef struct VM
 
 
     /** Padding for aligning the cpu array on a page boundary. */
-    uint8_t         abAlignment2[414];
+    uint8_t         abAlignment2[30];
 
     /* ---- end small stuff ---- */
 
@@ -1145,8 +1184,8 @@ typedef struct VM
 RT_C_DECLS_BEGIN
 
 /** The VM structure.
- * This is imported from the VMMGCBuiltin module, i.e. it's a one
- * of those magic globals which we should avoid using.
+ * This is imported from the VMMRCBuiltin module, i.e. it's a one of those magic
+ * globals which we should avoid using.
  */
 extern DECLIMPORT(VM)   g_VM;
 
