@@ -299,6 +299,25 @@ BOOL vbvaFindChangedRect(SURFOBJ *psoDest, SURFOBJ *psoSrc, RECTL *prclDest, POI
 }
 #endif /* VBOX_VBVA_ADJUST_RECT */
 
+static DECLCALLBACK(void *) hgsmiEnvAlloc(void *pvEnv, HGSMISIZE cb)
+{
+    NOREF(pvEnv);
+    return EngAllocMem(0, cb, MEM_ALLOC_TAG);
+}
+
+static DECLCALLBACK(void) hgsmiEnvFree(void *pvEnv, void *pv)
+{
+    NOREF(pvEnv);
+    EngFreeMem(pv);
+}
+
+static HGSMIENV g_hgsmiEnvDisp =
+{
+    NULL,
+    hgsmiEnvAlloc,
+    hgsmiEnvFree
+};
+
 int VBoxDispVBVAInit(PVBOXDISPDEV pDev)
 {
     int rc;
@@ -408,7 +427,7 @@ int VBoxDispVBVAInit(PVBOXDISPDEV pDev)
                             (uint8_t *)pDev->memInfo.VideoRamBase+pDev->layout.offDisplayInfo+sizeof(HGSMIHOSTFLAGS),
                             pDev->layout.cbDisplayInfo-sizeof(HGSMIHOSTFLAGS),
                             info.areaDisplay.offBase+pDev->layout.offDisplayInfo+sizeof(HGSMIHOSTFLAGS),
-                            false /*fOffsetBased*/);
+                            &g_hgsmiEnvDisp);
 
         if (RT_SUCCESS(rc))
         {
