@@ -406,8 +406,8 @@ NTSTATUS vboxWddmSwapchainCtxEscape(PVBOXMP_DEVEXT pDevExt, PVBOXWDDM_CONTEXT pC
 
     if (cbSize < RT_OFFSETOF(VBOXDISPIFESCAPE_SWAPCHAININFO, SwapchainInfo.ahAllocs[pSwapchainInfo->SwapchainInfo.cAllocs]))
     {
-        return STATUS_INVALID_PARAMETER;
         WARN(("invalid cbSize2 %d", cbSize));
+        return STATUS_INVALID_PARAMETER;
     }
 
     if (!pSwapchainInfo->SwapchainInfo.winHostID)
@@ -1806,8 +1806,6 @@ static int vboxWddmWdProgram(PVBOXMP_DEVEXT pDevExt, uint32_t cMillis)
                 if (offCmd != HGSMIOFFSET_VOID)
                 {
                     VBoxVideoCmnPortWriteUlong(VBoxCommonFromDeviceExt(pDevExt)->guestCtx.port, offCmd);
-                    /* Make the compiler aware that the host has changed memory. */
-                    ASMCompilerBarrier();
                     rc = VBoxSHGSMICommandDoneSynch(&VBoxCommonFromDeviceExt(pDevExt)->guestCtx.heapCtx, pHdr);
                     AssertRC(rc);
                     if (RT_SUCCESS(rc))
