@@ -1514,8 +1514,6 @@ static int vboxWddmVdmaSubmitVbva(PVBOXMP_DEVEXT pDevExt, PVBOXVDMAINFO pInfo, H
 static int vboxWddmVdmaSubmitHgsmi(PVBOXMP_DEVEXT pDevExt, PVBOXVDMAINFO pInfo, HGSMIOFFSET offDr)
 {
     VBoxVideoCmnPortWriteUlong(VBoxCommonFromDeviceExt(pDevExt)->guestCtx.port, offDr);
-    /* Make the compiler aware that the host has changed memory. */
-    ASMCompilerBarrier();
     return VINF_SUCCESS;
 }
 #define vboxWddmVdmaSubmit vboxWddmVdmaSubmitHgsmi
@@ -1575,6 +1573,25 @@ static int vboxVdmaInformHost(PVBOXMP_DEVEXT pDevExt, PVBOXVDMAINFO pInfo, VBOXV
 }
 #endif
 
+static DECLCALLBACK(void *) hgsmiEnvAlloc(void *pvEnv, HGSMISIZE cb)
+{
+    NOREF(pvEnv);
+    return RTMemAlloc(cb);
+}
+
+static DECLCALLBACK(void) hgsmiEnvFree(void *pvEnv, void *pv)
+{
+    NOREF(pvEnv);
+    RTMemFree(pv);
+}
+
+static HGSMIENV g_hgsmiEnvVdma =
+{
+    NULL,
+    hgsmiEnvAlloc,
+    hgsmiEnvFree
+};
+
 /* create a DMACommand buffer */
 int vboxVdmaCreate(PVBOXMP_DEVEXT pDevExt, VBOXVDMAINFO *pInfo
 #ifdef VBOX_WITH_VDMA
@@ -1613,7 +1630,7 @@ int vboxVdmaCreate(PVBOXMP_DEVEXT pDevExt, VBOXVDMAINFO *pInfo
                              pvBuffer,
                              cbBuffer,
                              offBuffer,
-                             false /*fOffsetBased*/);
+                             &g_hgsmiEnvVdma);
         Assert(RT_SUCCESS(rc));
         if(RT_SUCCESS(rc))
 #endif
