@@ -24,6 +24,7 @@
 #include "common/VBoxMPCommon.h"
 #include "VBoxDisplay.h"
 #include <iprt/initterm.h>
+#include <VBox/version.h>
 
 /* Resource list */
 VIDEO_ACCESS_RANGE  VBoxLegacyVGAResourceList[] =
@@ -725,6 +726,10 @@ ULONG DriverEntry(IN PVOID Context1, IN PVOID Context2)
 
     LOGF_ENTER();
 
+    LOGREL(("VBox XPDM Driver for Windows version %d.%d.%dr%d, %d bit; Built %s %s",
+            VBOX_VERSION_MAJOR, VBOX_VERSION_MINOR, VBOX_VERSION_BUILD, VBOX_SVN_REV,
+            (sizeof (void*) << 3), __DATE__, __TIME__));
+
     VIDEO_HW_INITIALIZATION_DATA vhwData;
 
     /*Zero the structure*/
