@@ -15,9 +15,10 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #ifndef WIN32_NO_STATUS
 # include <ntstatus.h>
 # define WIN32_NO_STATUS
@@ -637,7 +638,7 @@ VBoxCredProvCredential::ExtractAccoutData(PWSTR pwszAccountData, PWSTR *ppwszAcc
         if (SUCCEEDED(hr))
         {
             *ppwszAccoutName = pwszName;
-            *pPos++; /* Skip @, point to domain name (if any). */
+            pPos++; /* Skip @, point to domain name (if any). */
             if (    pPos != NULL
                 && *pPos != L'\0')
             {
