@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2014 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,12 +21,20 @@
 #include "VBoxTray.h"
 #include "VBoxHelpers.h"
 #include "VBoxVRDP.h"
+
 #include <VBox/VMMDev.h>
+#ifdef DEBUG
+# define LOG_ENABLED
+# define LOG_GROUP LOG_GROUP_DEFAULT
+#endif
+#include <VBox/log.h>
+
 #include <VBoxGuestInternal.h>
 #include <iprt/assert.h>
 #include <iprt/ldr.h>
 
 
+
 /* The guest receives VRDP_ACTIVE/VRDP_INACTIVE notifications.
  *
  * When VRDP_ACTIVE is received, the guest asks host about the experience level.
@@ -93,7 +101,7 @@ static void vboxExperienceSet (uint32_t level)
             /*
              * The parameter has to be disabled.
              */
-            Log(("VBoxTray: vboxExperienceSet: Saving %s\n", parameters[i].name));
+            LogFlowFunc(("Saving %s\n", parameters[i].name));
 
             /* Save the current value. */
             switch (parameters[i].type)
@@ -127,7 +135,7 @@ static void vboxExperienceSet (uint32_t level)
                      */
                     if (parameters[i].cbSavedValue > sizeof (parameters[i].achSavedValue))
                     {
-                        Log(("VBoxTray: vboxExperienceSet: Not enough space %d > %d\n", parameters[i].cbSavedValue, sizeof (parameters[i].achSavedValue)));
+                        LogFlowFunc(("Not enough space %d > %d\n", parameters[i].cbSavedValue, sizeof (parameters[i].achSavedValue)));
                         break;
                     }
 
@@ -143,7 +151,7 @@ static void vboxExperienceSet (uint32_t level)
                     break;
             }
 
-            Log(("VBoxTray: vboxExperienceSet: Disabling %s\n", parameters[i].name));
+            LogFlowFunc(("Disabling %s\n", parameters[i].name));
 
             /* Disable the feature. */
             switch (parameters[i].type)
@@ -198,7 +206,7 @@ static void vboxExperienceRestore (uint32_t level)
     {
         if (parameters[i].level > level)
         {
-            Log(("VBoxTray: vboxExperienceRestore: Restoring %s\n", parameters[i].name));
+            LogFlowFunc(("Restoring %s\n", parameters[i].name));
 
             /* Restore the feature. */
             switch (parameters[i].type)
@@ -268,7 +276,7 @@ static VBOXVRDPCONTEXT gCtx = {0};
 
 int VBoxVRDPInit(const VBOXSERVICEENV *pEnv, void **ppInstance, bool *pfStartThread)
 {
-    Log(("VBoxTray: VBoxVRDPInit\n"));
+    LogFlowFunc(("VBoxVRDPInit\n"));
 
     gCtx.pEnv      = pEnv;
     gCtx.level     = VRDP_EXPERIENCE_LEVEL_FULL;
@@ -295,7 +303,7 @@ int VBoxVRDPInit(const VBOXSERVICEENV *pEnv, void **ppInstance, bool *pfStartThr
 
 void VBoxVRDPDestroy(const VBOXSERVICEENV *pEnv, void *pInstance)
 {
-    Log(("VBoxTray: VBoxVRDPDestroy\n"));
+    LogFlowFunc(("VBoxVRDPDestroy\n"));
     VBOXVRDPCONTEXT *pCtx = (VBOXVRDPCONTEXT *)pInstance;
     vboxExperienceRestore (pCtx->level);
     if (gCtx.hModUxTheme != NIL_RTLDRMOD)
@@ -321,11 +329,11 @@ unsigned __stdcall VBoxVRDPThread(void *pInstance)
     maskInfo.u32NotMask = 0;
     if (DeviceIoControl (gVBoxDriver, VBOXGUEST_IOCTL_CTL_FILTER_MASK, &maskInfo, sizeof (maskInfo), NULL, 0, &cbReturned, NULL))
     {
-        Log(("VBoxTray: VBoxVRDPThread: DeviceIOControl(CtlMask - or) succeeded\n"));
+        LogFlowFunc(("DeviceIOControl(CtlMask - or) succeeded\n"));
     }
     else
     {
-        Log(("VBoxTray: VBoxVRDPThread: DeviceIOControl(CtlMask) failed\n"));
+        LogFlowFunc(("DeviceIOControl(CtlMask) failed\n"));
         return 0;
     }
 
@@ -337,13 +345,13 @@ unsigned __stdcall VBoxVRDPThread(void *pInstance)
         waitEvent.u32EventMaskIn = VMMDEV_EVENT_VRDP;
         if (DeviceIoControl(gVBoxDriver, VBOXGUEST_IOCTL_WAITEVENT, &waitEvent, sizeof(waitEvent), &waitEvent, sizeof(waitEvent), &cbReturned, NULL))
         {
-            Log(("VBoxTray: VBoxVRDPThread: DeviceIOControl succeeded\n"));
+            LogFlowFunc(("DeviceIOControl succeeded\n"));
 
             /* are we supposed to stop? */
             if (WaitForSingleObject(pCtx->pEnv->hStopEvent, 0) == WAIT_OBJECT_0)
                 break;
 
-            Log(("VBoxTray: VBoxVRDPThread: checking event\n"));
+            LogFlowFunc(("checking event\n"));
 
             /* did we get the right event? */
             if (waitEvent.u32EventFlagsOut & VMMDEV_EVENT_VRDP)
@@ -365,7 +373,7 @@ unsigned __stdcall VBoxVRDPThread(void *pInstance)
                                      sizeof(VMMDevVRDPChangeRequest),
                                      &cbReturned, NULL))
                 {
-                    Log(("VBoxTray: VBoxVRDPThread: u8VRDPActive = %d, level %d\n", vrdpChangeRequest.u8VRDPActive, vrdpChangeRequest.u32VRDPExperienceLevel));
+                    LogFlowFunc(("u8VRDPActive = %d, level %d\n", vrdpChangeRequest.u8VRDPActive, vrdpChangeRequest.u32VRDPExperienceLevel));
 
                     if (vrdpChangeRequest.u8VRDPActive)
                     {
@@ -378,7 +386,7 @@ unsigned __stdcall VBoxVRDPThread(void *pInstance)
                         {
                             pCtx->fSavedThemeEnabled = pCtx->pfnIsThemeActive ();
 
-                            Log(("VBoxTray: VBoxVRDPThread: pCtx->fSavedThemeEnabled = %d\n", pCtx->fSavedThemeEnabled));
+                            LogFlowFunc(("pCtx->fSavedThemeEnabled = %d\n", pCtx->fSavedThemeEnabled));
 
                             if (pCtx->fSavedThemeEnabled)
                             {
@@ -396,7 +404,7 @@ unsigned __stdcall VBoxVRDPThread(void *pInstance)
                             {
                                 /* @todo the call returns S_OK but theming remains disabled. */
                                 HRESULT hrc = pCtx->pfnEnableTheming (TRUE);
-                                Log(("VBoxTray: VBoxVRDPThread: enabling theme rc = 0x%08X\n", hrc));
+                                LogFlowFunc(("enabling theme rc = 0x%08X\n", hrc));
                                 pCtx->fSavedThemeEnabled = FALSE;
                             }
                         }
@@ -408,8 +416,9 @@ unsigned __stdcall VBoxVRDPThread(void *pInstance)
                 }
                 else
                 {
-                    Log(("VBoxTray: VBoxVRDPThread: Error from DeviceIoControl VBOXGUEST_IOCTL_VMMREQUEST\n"));
-
+#ifndef DEBUG_andy /* Too noisy for me. */
+                    LogFlowFunc(("Error from DeviceIoControl VBOXGUEST_IOCTL_VMMREQUEST\n"));
+#endif
                     /* sleep a bit to not eat too much CPU in case the above call always fails */
                     if (WaitForSingleObject(pCtx->pEnv->hStopEvent, 10) == WAIT_OBJECT_0)
                     {
@@ -421,8 +430,9 @@ unsigned __stdcall VBoxVRDPThread(void *pInstance)
         }
         else
         {
-            Log(("VBoxTray: VBoxVRDPThread: Error from DeviceIoControl VBOXGUEST_IOCTL_WAITEVENT\n"));
-
+#ifndef DEBUG_andy
+            LogFlowFunc(("Error from DeviceIoControl VBOXGUEST_IOCTL_WAITEVENT\n"));
+#endif
             /* sleep a bit to not eat too much CPU in case the above call always fails */
             if (WaitForSingleObject(pCtx->pEnv->hStopEvent, 10) == WAIT_OBJECT_0)
             {
@@ -436,14 +446,14 @@ unsigned __stdcall VBoxVRDPThread(void *pInstance)
     maskInfo.u32NotMask = VMMDEV_EVENT_VRDP;
     if (DeviceIoControl (gVBoxDriver, VBOXGUEST_IOCTL_CTL_FILTER_MASK, &maskInfo, sizeof (maskInfo), NULL, 0, &cbReturned, NULL))
     {
-        Log(("VBoxTray: VBoxVRDPThread: DeviceIOControl(CtlMask - not) succeeded\n"));
+        LogFlowFunc(("DeviceIOControl(CtlMask - not) succeeded\n"));
     }
     else
     {
-        Log(("VBoxTray: VBoxVRDPThread: DeviceIOControl(CtlMask) failed\n"));
+        LogFlowFunc(("DeviceIOControl(CtlMask) failed\n"));
     }
 
-    Log(("VBoxTray: VBoxVRDPThread: Finished VRDP change request thread\n"));
+    LogFlowFunc(("Finished VRDP change request thread\n"));
     return 0;
 }
 
