@@ -1,4 +1,4 @@
-/* $Rev: 83687 $ */
+/* $Rev: 102121 $ */
 /** @file
  * VBoxGuest - Linux specifics.
  *
@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,9 +18,10 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
   */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_SUP_DRV
 
 #include "the-linux-kernel.h"
@@ -49,9 +50,9 @@
 #include <VBox/log.h>
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /** The device name. */
 #define DEVICE_NAME             "vboxguest"
 /** The device name for the device node open to everyone.. */
@@ -67,9 +68,9 @@
 #endif
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static void vboxguestLinuxTermPci(struct pci_dev *pPciDev);
 static int  vboxguestLinuxModInit(void);
 static void vboxguestLinuxModExit(void);
@@ -85,9 +86,9 @@ static unsigned int vboxguestPoll(struct file *pFile, poll_table *pPt);
 static ssize_t vboxguestRead(struct file *pFile, char *pbBuf, size_t cbRead, loff_t *poff);
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /**
  * Device extention & session data association structure.
  */
@@ -130,17 +131,6 @@ static char                     g_szDbgLogDst[128];
 # endif
 #endif
 
-/** Our file node major id.
- * Either set dynamically at run time or statically at compile time. */
-#ifdef CONFIG_VBOXGUEST_MAJOR
-static unsigned int             g_iModuleMajor = CONFIG_VBOXGUEST_MAJOR;
-#else
-static unsigned int             g_iModuleMajor = 0;
-#endif
-#ifdef CONFIG_VBOXADD_MAJOR
-# error "CONFIG_VBOXADD_MAJOR -> CONFIG_VBOXGUEST_MAJOR"
-#endif
-
 /** The input device handle */
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
 static struct input_dev        *g_pInputDevice = NULL;
@@ -351,7 +341,7 @@ static irqreturn_t vboxguestLinuxISR(int iIrrq, void *pvDevId)
 static irqreturn_t vboxguestLinuxISR(int iIrrq, void *pvDevId, struct pt_regs *pRegs)
 #endif
 {
-    bool fTaken = VBoxGuestCommonISR(&g_DevExt);
+    bool fTaken = VbgdCommonISR(&g_DevExt);
     return IRQ_RETVAL(fTaken);
 }
 
@@ -396,9 +386,9 @@ static void vboxguestLinuxTermISR(void)
  * our kernel session. */
 static int vboxguestLinuxSetMouseStatus(uint32_t fStatus)
 {
-    return VBoxGuestCommonIOCtl(VBOXGUEST_IOCTL_SET_MOUSE_STATUS, &g_DevExt,
-                                g_pKernelSession, &fStatus, sizeof(fStatus),
-                                NULL);
+    return VbgdCommonIoCtl(VBOXGUEST_IOCTL_SET_MOUSE_STATUS, &g_DevExt,
+                           g_pKernelSession, &fStatus, sizeof(fStatus),
+                           NULL);
 }
 
 
@@ -485,10 +475,11 @@ static int __init vboxguestLinuxCreateInputDevice(void)
 static void vboxguestLinuxTermInputDevice(void)
 {
     VbglGRFree(&g_pMouseStatusReq->header);
+    /* See documentation of input_register_device(): input_free_device()
+     * should not be called after a device has been registered. */
     input_unregister_device(g_pInputDevice);
-    input_free_device(g_pInputDevice);
 }
-#endif
+#endif /* VBOXGUEST_WITH_INPUT_DRIVER */
 
 
 /**
@@ -503,23 +494,11 @@ static int __init vboxguestLinuxInitDeviceNodes(void)
     /*
      * The full feature device node.
      */
-    if (g_iModuleMajor > 0)
+    rc = misc_register(&g_MiscDevice);
+    if (rc)
     {
-        rc = register_chrdev(g_iModuleMajor, DEVICE_NAME, &g_FileOps);
-        if (rc < 0)
-        {
-            LogRel((DEVICE_NAME ": register_chrdev failed: g_iModuleMajor: %d, rc: %d\n", g_iModuleMajor, rc));
-            return rc;
-        }
-    }
-    else
-    {
-        rc = misc_register(&g_MiscDevice);
-        if (rc)
-        {
-            LogRel((DEVICE_NAME ": misc_register failed for %s (rc=%d)\n", DEVICE_NAME, rc));
-            return rc;
-        }
+        LogRel((DEVICE_NAME ": misc_register failed for %s (rc=%d)\n", DEVICE_NAME, rc));
+        return rc;
     }
 
     /*
@@ -529,10 +508,7 @@ static int __init vboxguestLinuxInitDeviceNodes(void)
     if (rc)
     {
         LogRel((DEVICE_NAME ": misc_register failed for %s (rc=%d)\n", DEVICE_NAME_USER, rc));
-        if (g_iModuleMajor > 0)
-            unregister_chrdev(g_iModuleMajor, DEVICE_NAME);
-        else
-            misc_deregister(&g_MiscDevice);
+        misc_deregister(&g_MiscDevice);
         return rc;
     }
 
@@ -545,10 +521,7 @@ static int __init vboxguestLinuxInitDeviceNodes(void)
  */
 static void vboxguestLinuxTermDeviceNodes(void)
 {
-    if (g_iModuleMajor > 0)
-        unregister_chrdev(g_iModuleMajor, DEVICE_NAME);
-    else
-        misc_deregister(&g_MiscDevice);
+    misc_deregister(&g_MiscDevice);
     misc_deregister(&g_MiscDeviceUser);
 }
 
@@ -622,18 +595,18 @@ static int __init vboxguestLinuxModInit(void)
 # warning "huh? which arch + version is this?"
             VBOXOSTYPE enmOsType = VBOXOSTYPE_Linux;
 #endif
-            rc = VBoxGuestInitDevExt(&g_DevExt,
-                                     g_IOPortBase,
-                                     g_pvMMIOBase,
-                                     g_cbMMIO,
-                                     enmOSType,
-                                     VMMDEV_EVENT_MOUSE_POSITION_CHANGED);
+            rc = VbgdCommonInitDevExt(&g_DevExt,
+                                      g_IOPortBase,
+                                      g_pvMMIOBase,
+                                      g_cbMMIO,
+                                      enmOSType,
+                                      VMMDEV_EVENT_MOUSE_POSITION_CHANGED);
             if (RT_SUCCESS(rc))
             {
                 /*
                  * Create the kernel session for this driver.
                  */
-                rc = VBoxGuestCreateKernelSession(&g_DevExt,
+                rc = VbgdCommonCreateKernelSession(&g_DevExt,
                                                   &g_pKernelSession);
                 if (RT_SUCCESS(rc))
                 {
@@ -652,8 +625,8 @@ static int __init vboxguestLinuxModInit(void)
                         if (rc >= 0)
                         {
                             /* some useful information for the user but don't show this on the console */
-                            LogRel((DEVICE_NAME ": major %d, IRQ %d, I/O port %RTiop, MMIO at %RHp (size 0x%x)\n",
-                                    g_iModuleMajor, g_pPciDev->irq, g_IOPortBase, g_MMIOPhysAddr, g_cbMMIO));
+                            LogRel((DEVICE_NAME ": misc device minor %d, IRQ %d, I/O port %RTiop, MMIO at %RHp (size 0x%x)\n",
+                                    g_MiscDevice.minor, g_pPciDev->irq, g_IOPortBase, g_MMIOPhysAddr, g_cbMMIO));
                             printk(KERN_DEBUG DEVICE_NAME ": Successfully loaded version "
                                    VBOX_VERSION_STRING " (interface " RT_XSTR(VMMDEV_VERSION) ")\n");
                             return rc;
@@ -669,13 +642,13 @@ static int __init vboxguestLinuxModInit(void)
                         rc = RTErrConvertFromErrno(rc);
                     }
 #endif
-                    VBoxGuestCloseSession(&g_DevExt, g_pKernelSession);
+                    VbgdCommonCloseSession(&g_DevExt, g_pKernelSession);
                 }
-                VBoxGuestDeleteDevExt(&g_DevExt);
+                VbgdCommonDeleteDevExt(&g_DevExt);
             }
             else
             {
-                LogRel((DEVICE_NAME ": VBoxGuestInitDevExt failed with rc=%Rrc\n", rc));
+                LogRel((DEVICE_NAME ": VbgdCommonInitDevExt failed with rc=%Rrc\n", rc));
                 rc = RTErrConvertFromErrno(rc);
             }
             vboxguestLinuxTermISR();
@@ -706,8 +679,8 @@ static void __exit vboxguestLinuxModExit(void)
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
     vboxguestLinuxTermInputDevice();
 #endif
-    VBoxGuestCloseSession(&g_DevExt, g_pKernelSession);
-    VBoxGuestDeleteDevExt(&g_DevExt);
+    VbgdCommonCloseSession(&g_DevExt, g_pKernelSession);
+    VbgdCommonDeleteDevExt(&g_DevExt);
     vboxguestLinuxTermISR();
     pci_unregister_driver(&g_PciDriver);
     RTLogDestroy(RTLogRelSetDefaultInstance(NULL));
@@ -732,9 +705,13 @@ static int vboxguestLinuxOpen(struct inode *pInode, struct file *pFilp)
      * Call common code to create the user session. Associate it with
      * the file so we can access it in the other methods.
      */
-    rc = VBoxGuestCreateUserSession(&g_DevExt, &pSession);
+    rc = VbgdCommonCreateUserSession(&g_DevExt, &pSession);
     if (RT_SUCCESS(rc))
+    {
         pFilp->private_data = pSession;
+        if (MINOR(pInode->i_rdev) == g_MiscDeviceUser.minor)
+            pSession->fUserSession = true;
+    }
 
     Log(("vboxguestLinuxOpen: g_DevExt=%p pSession=%p rc=%d/%d (pid=%d/%d %s)\n",
          &g_DevExt, pSession, rc, vboxguestLinuxConvertToNegErrno(rc),
@@ -759,7 +736,7 @@ static int vboxguestLinuxRelease(struct inode *pInode, struct file *pFilp)
      * the file pointer didn't get left on the polling queue. */
     vboxguestFAsync(-1, pFilp, 0);
 #endif
-    VBoxGuestCloseSession(&g_DevExt, (PVBOXGUESTSESSION)pFilp->private_data);
+    VbgdCommonCloseSession(&g_DevExt, (PVBOXGUESTSESSION)pFilp->private_data);
     pFilp->private_data = NULL;
     return 0;
 }
@@ -810,7 +787,7 @@ static int vboxguestLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigne
          * Process the IOCtl.
          */
         size_t cbDataReturned;
-        rc = VBoxGuestCommonIOCtl(uCmd, &g_DevExt, pSession, pvBuf, cbData, &cbDataReturned);
+        rc = VbgdCommonIoCtl(uCmd, &g_DevExt, pSession, pvBuf, cbData, &cbDataReturned);
 
         /*
          * Copy ioctl data and output buffer back to user space.
@@ -905,7 +882,7 @@ static unsigned int vboxguestPoll(struct file *pFile, poll_table *pPt)
  *
  * @remarks This is probably not really used as X11 lets the driver do its own
  *          event reading. The poll condition is therefore also cleared when we
- *          see VMMDevReq_GetMouseStatus in VBoxGuestCommonIOCtl_VMMRequest.
+ *          see VMMDevReq_GetMouseStatus in VbgdCommonIoCtl_VMMRequest.
  */
 static ssize_t vboxguestRead(struct file *pFile, char *pbBuf, size_t cbRead, loff_t *poff)
 {
@@ -929,7 +906,7 @@ static ssize_t vboxguestRead(struct file *pFile, char *pbBuf, size_t cbRead, lof
 }
 
 
-void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
+void VbgdNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
 {
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
     int rc;
@@ -940,9 +917,9 @@ void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
      * Wake up everyone that's in a poll() and post anyone that has
      * subscribed to async notifications.
      */
-    Log(("VBoxGuestNativeISRMousePollEvent: wake_up_all\n"));
+    Log3(("VbgdNativeISRMousePollEvent: wake_up_all\n"));
     wake_up_all(&g_PollEventQueue);
-    Log(("VBoxGuestNativeISRMousePollEvent: kill_fasync\n"));
+    Log3(("VbgdNativeISRMousePollEvent: kill_fasync\n"));
     kill_fasync(&g_pFAsyncQueue, SIGIO, POLL_IN);
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
     /* Report events to the kernel input device */
@@ -961,7 +938,7 @@ void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
 # endif
     }
 #endif
-    Log(("VBoxGuestNativeISRMousePollEvent: done\n"));
+    Log3(("VbgdNativeISRMousePollEvent: done\n"));
 }
 
 
@@ -980,7 +957,7 @@ static int vboxguestLinuxParamLogGrpSet(const char *pszValue, struct kernel_para
 {
     if (g_fLoggerCreated)
     {
-        PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelDefaultInstance();
+        PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
         if (pLogger)
             RTLogGroupSettings(pLogger, pszValue);
     }
@@ -990,11 +967,10 @@ static int vboxguestLinuxParamLogGrpSet(const char *pszValue, struct kernel_para
     return 0;
 }
 
-
 /** log and dbg_log parameter getter. */
 static int vboxguestLinuxParamLogGrpGet(char *pszBuf, struct kernel_param *pParam)
 {
-    PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelDefaultInstance();
+    PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
     *pszBuf = '\0';
     if (pLogger)
         RTLogGetGroupSettings(pLogger, pszBuf, _4K);
@@ -1007,7 +983,7 @@ static int vboxguestLinuxParamLogFlagsSet(const char *pszValue, struct kernel_pa
 {
     if (g_fLoggerCreated)
     {
-        PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelDefaultInstance();
+        PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
         if (pLogger)
             RTLogFlags(pLogger, pszValue);
     }
@@ -1016,11 +992,10 @@ static int vboxguestLinuxParamLogFlagsSet(const char *pszValue, struct kernel_pa
     return 0;
 }
 
-
 /** log and dbg_log_flags parameter getter. */
 static int vboxguestLinuxParamLogFlagsGet(char *pszBuf, struct kernel_param *pParam)
 {
-    PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelDefaultInstance();
+    PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
     *pszBuf = '\0';
     if (pLogger)
         RTLogGetFlags(pLogger, pszBuf, _4K);
@@ -1033,7 +1008,7 @@ static int vboxguestLinuxParamLogDstSet(const char *pszValue, struct kernel_para
 {
     if (g_fLoggerCreated)
     {
-        PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelDefaultInstance();
+        PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
         if (pLogger)
             RTLogDestinations(pLogger, pszValue);
     }
@@ -1042,17 +1017,43 @@ static int vboxguestLinuxParamLogDstSet(const char *pszValue, struct kernel_para
     return 0;
 }
 
-
 /** log and dbg_log_dest parameter getter. */
 static int vboxguestLinuxParamLogDstGet(char *pszBuf, struct kernel_param *pParam)
 {
-    PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelDefaultInstance();
+    PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
     *pszBuf = '\0';
     if (pLogger)
         RTLogGetDestinations(pLogger, pszBuf, _4K);
     return strlen(pszBuf);
 }
 
+
+/** r3_log_to_host parameter setter. */
+static int vboxguestLinuxParamR3LogToHostSet(const char *pszValue, struct kernel_param *pParam)
+{
+    if (    pszValue == NULL
+        || *pszValue == '\0'
+        || *pszValue == 'n'
+        || *pszValue == 'N'
+        || *pszValue == 'd'
+        || *pszValue == 'D'
+        || (   (*pszValue == 'o' || *pszValue == 'O')
+            && (*pszValue == 'f' || *pszValue == 'F') )
+       )
+        g_DevExt.fLoggingEnabled = false;
+    else
+        g_DevExt.fLoggingEnabled = true;
+    return 0;
+}
+
+/** r3_log_to_host parameter getter. */
+static int vboxguestLinuxParamR3LogToHostGet(char *pszBuf, struct kernel_param *pParam)
+{
+    strcpy(pszBuf, g_DevExt.fLoggingEnabled ? "enabled" : "disabled");
+    return strlen(pszBuf);
+}
+
+
 /*
  * Define module parameters.
  */
@@ -1064,6 +1065,7 @@ module_param_call(dbg_log,        vboxguestLinuxParamLogGrpSet,   vboxguestLinux
 module_param_call(dbg_log_flags,  vboxguestLinuxParamLogFlagsSet, vboxguestLinuxParamLogFlagsGet, NULL, 0664);
 module_param_call(dbg_log_dest,   vboxguestLinuxParamLogDstSet,   vboxguestLinuxParamLogDstGet,   NULL, 0664);
 # endif
+module_param_call(r3_log_to_host, vboxguestLinuxParamR3LogToHostSet, vboxguestLinuxParamR3LogToHostGet, NULL, 0664);
 
 #endif /* 2.6.0 and later */
 
