@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007-2011 Oracle Corporation
+ * Copyright (C) 2007-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,13 +24,17 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
+#if defined(VBOX_VBGLR3_XFREE86) || defined(VBOX_VBGLR3_XORG)
+# define VBOX_VBGLR3_XSERVER
+#endif
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/string.h>
-#ifndef VBOX_VBGLR3_XFREE86
-# include <iprt/cpp/mem.h>
+#ifndef VBOX_VBGLR3_XSERVER
+# include <iprt/mem.h>
 #endif
 #include <iprt/assert.h>
 #include <iprt/stdarg.h>
@@ -52,7 +56,11 @@ extern "C" void* xf86memset(const void*,int,xf86size_t);
 # undef memset
 # define memset xf86memset
 
-# undef RTSTrEnd
+#endif /* VBOX_VBGLR3_XFREE86 */
+
+#ifdef VBOX_VBGLR3_XSERVER
+
+# undef RTStrEnd
 # define RTStrEnd xf86RTStrEnd
 
 DECLINLINE(char const *) RTStrEnd(char const *pszString, size_t cchMax)
@@ -85,11 +93,12 @@ DECLINLINE(char *) RTStrEnd(char *pszString, size_t cchMax)
     return (char *)memchr(pszString, '\0', cchMax);
 }
 
-#endif /* VBOX_VBGLR3_XFREE86 */
+#endif /* VBOX_VBGLR3_XSERVER */
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * Structure containing information needed to enumerate through guest
  * properties.
@@ -113,6 +122,7 @@ using namespace guestProp;
  * Connects to the guest property service.
  *
  * @returns VBox status code
+ * @returns VERR_NOT_SUPPORTED if guest properties are not available on the host.
  * @param   pu32ClientId    Where to put the client id on success. The client id
  *                          must be passed to all the other calls to the service.
  */
@@ -131,6 +141,8 @@ VBGLR3DECL(int) VbglR3GuestPropConnect(uint32_t *pu32ClientId)
         rc = Info.result;
         if (RT_SUCCESS(rc))
             *pu32ClientId = Info.u32ClientID;
+        if (rc == VERR_NOT_IMPLEMENTED || rc == VERR_HGCM_SERVICE_NOT_FOUND)
+            rc = VERR_NOT_SUPPORTED;
     }
     return rc;
 }
@@ -248,7 +260,7 @@ VBGLR3DECL(int) VbglR3GuestPropWriteValue(uint32_t u32ClientId, const char *pszN
     return rc;
 }
 
-#ifndef VBOX_VBGLR3_XFREE86
+#ifndef VBOX_VBGLR3_XSERVER
 /**
  * Write a property value where the value is formatted in RTStrPrintfV fashion.
  *
@@ -293,7 +305,7 @@ VBGLR3DECL(int) VbglR3GuestPropWriteValueF(uint32_t u32ClientId, const char *psz
     va_end(va);
     return rc;
 }
-#endif /* VBOX_VBGLR3_XFREE86 */
+#endif /* VBOX_VBGLR3_XSERVER */
 
 /**
  * Retrieve a property.
@@ -390,7 +402,7 @@ VBGLR3DECL(int) VbglR3GuestPropRead(uint32_t u32ClientId, const char *pszName,
     return VINF_SUCCESS;
 }
 
-#ifndef VBOX_VBGLR3_XFREE86
+#ifndef VBOX_VBGLR3_XSERVER
 /**
  * Retrieve a property value, allocating space for it.
  *
@@ -470,7 +482,7 @@ VBGLR3DECL(void) VbglR3GuestPropReadValueFree(char *pszValue)
 {
     RTMemFree(pszValue);
 }
-#endif /* VBOX_VBGLR3_XFREE86 */
+#endif /* VBOX_VBGLR3_XSERVER */
 
 /**
  * Retrieve a property value, using a user-provided buffer to store it.
@@ -505,7 +517,7 @@ VBGLR3DECL(int) VbglR3GuestPropReadValue(uint32_t u32ClientId, const char *pszNa
 }
 
 
-#ifndef VBOX_VBGLR3_XFREE86
+#ifndef VBOX_VBGLR3_XSERVER
 /**
  * Raw API for enumerating guest properties which match a given pattern.
  *
@@ -558,7 +570,7 @@ VBGLR3DECL(int) VbglR3GuestPropEnumRaw(uint32_t u32ClientId,
             ||  rc == VERR_BUFFER_OVERFLOW))
     {
         int rc2 = VbglHGCMParmUInt32Get(&Msg.size, pcbBufActual);
-        if (!RT_SUCCESS(rc2))
+        if (RT_FAILURE(rc2))
             rc = rc2;
     }
     return rc;
@@ -567,14 +579,13 @@ VBGLR3DECL(int) VbglR3GuestPropEnumRaw(uint32_t u32ClientId,
 
 /**
  * Start enumerating guest properties which match a given pattern.
+ *
  * This function creates a handle which can be used to continue enumerating.
  *
  * @returns VBox status code.
  * @retval  VINF_SUCCESS on success, *ppHandle points to a handle for continuing
  *          the enumeration and *ppszName, *ppszValue, *pu64Timestamp and
  *          *ppszFlags are set.
- * @retval  VERR_NOT_FOUND if no matching properties were found.  In this case
- *          the return parameters are not initialised.
  * @retval  VERR_TOO_MUCH_DATA if it was not possible to determine the amount
  *          of local space needed to store all the enumeration data.  This is
  *          due to a race between allocating space and the host adding new
@@ -589,14 +600,21 @@ VBGLR3DECL(int) VbglR3GuestPropEnumRaw(uint32_t u32ClientId,
  * @param   ppHandle        where the handle for continued enumeration is stored
  *                          on success.  This must be freed with
  *                          VbglR3GuestPropEnumFree when it is no longer needed.
- * @param   ppszName        Where to store the first property name on success.
- *                          Should not be freed. Optional.
- * @param   ppszValue       Where to store the first property value on success.
- *                          Should not be freed. Optional.
- * @param   ppszValue       Where to store the first timestamp value on success.
- *                          Optional.
- * @param   ppszFlags       Where to store the first flags value on success.
- *                          Should not be freed. Optional.
+ * @param   ppszName        Where to store the next property name.  This will be
+ *                          set to NULL if there are no more properties to
+ *                          enumerate.  This pointer should not be freed. Optional.
+ * @param   ppszValue       Where to store the next property value.  This will be
+ *                          set to NULL if there are no more properties to
+ *                          enumerate.  This pointer should not be freed. Optional.
+ * @param   pu64Timestamp   Where to store the next property timestamp.  This
+ *                          will be set to zero if there are no more properties
+ *                          to enumerate. Optional.
+ * @param   ppszFlags       Where to store the next property flags.  This will be
+ *                          set to NULL if there are no more properties to
+ *                          enumerate.  This pointer should not be freed. Optional.
+ *
+ * @remarks While all output parameters are optional, you need at least one to
+ *          figure out when to stop.
  */
 VBGLR3DECL(int) VbglR3GuestPropEnum(uint32_t u32ClientId,
                                     char const * const *papszPatterns,
@@ -608,70 +626,72 @@ VBGLR3DECL(int) VbglR3GuestPropEnum(uint32_t u32ClientId,
                                     char const **ppszFlags)
 {
     /* Create the handle. */
-    RTCMemAutoPtr<VBGLR3GUESTPROPENUM, VbglR3GuestPropEnumFree> Handle;
-    Handle = (PVBGLR3GUESTPROPENUM)RTMemAllocZ(sizeof(VBGLR3GUESTPROPENUM));
-    if (!Handle)
+    PVBGLR3GUESTPROPENUM pHandle = (PVBGLR3GUESTPROPENUM)RTMemAllocZ(sizeof(VBGLR3GUESTPROPENUM));
+    if (RT_LIKELY(pHandle))
+    {/* likely */}
+    else
         return VERR_NO_MEMORY;
 
     /* Get the length of the pattern string, including the final terminator. */
-    size_t cchPatterns = 1;
+    size_t cbPatterns = 1;
     for (uint32_t i = 0; i < cPatterns; ++i)
-        cchPatterns += strlen(papszPatterns[i]) + 1;
+        cbPatterns += strlen(papszPatterns[i]) + 1;
 
-    /* Pack the pattern array */
-    RTCMemAutoPtr<char> Patterns;
-    Patterns = (char *)RTMemAlloc(cchPatterns);
+    /* Pack the pattern array. */
+    char *pszzPatterns = (char *)RTMemAlloc(cbPatterns);
     size_t off = 0;
     for (uint32_t i = 0; i < cPatterns; ++i)
     {
         size_t cb = strlen(papszPatterns[i]) + 1;
-        memcpy(&Patterns[off], papszPatterns[i], cb);
+        memcpy(&pszzPatterns[off], papszPatterns[i], cb);
         off += cb;
     }
-    Patterns[off] = '\0';
-
-    /* Randomly chosen initial size for the buffer to hold the enumeration
-     * information. */
-    uint32_t cchBuf = 4096;
-    RTCMemAutoPtr<char> Buf;
+    pszzPatterns[off] = '\0';
 
     /* In reading the guest property data we are racing against the host
      * adding more of it, so loop a few times and retry on overflow. */
-    int rc = VINF_SUCCESS;
+    uint32_t cbBuf  = 4096; /* picked out of thin air */
+    char    *pchBuf = NULL;
+    int      rc     = VINF_SUCCESS;
     for (int i = 0; i < 10; ++i)
     {
-        if (!Buf.realloc(cchBuf))
+        void *pvNew = RTMemRealloc(pchBuf, cbBuf);
+        if (pvNew)
+            pchBuf = (char *)pvNew;
+        else
         {
             rc = VERR_NO_MEMORY;
             break;
         }
-        rc = VbglR3GuestPropEnumRaw(u32ClientId, Patterns.get(),
-                                    Buf.get(), cchBuf, &cchBuf);
+        rc = VbglR3GuestPropEnumRaw(u32ClientId, pszzPatterns, pchBuf, cbBuf, &cbBuf);
         if (rc != VERR_BUFFER_OVERFLOW)
             break;
-        cchBuf += 4096;  /* Just to increase our chances */
+        cbBuf += 4096;  /* Just to increase our chances */
     }
+    RTMemFree(pszzPatterns);
     if (RT_SUCCESS(rc))
     {
         /*
-         * Transfer ownership of the buffer to the handle structure and
-         * call VbglR3GuestPropEnumNext to retrieve the first entry.
+         * Complete the handle and call VbglR3GuestPropEnumNext to retrieve the first entry.
          */
-        Handle->pchNext = Handle->pchBuf = Buf.release();
-        Handle->pchBufEnd = Handle->pchBuf + cchBuf;
+        pHandle->pchNext   = pchBuf;
+        pHandle->pchBuf    = pchBuf;
+        pHandle->pchBufEnd = pchBuf + cbBuf;
 
         const char *pszNameTmp;
         if (!ppszName)
             ppszName = &pszNameTmp;
-        rc = VbglR3GuestPropEnumNext(Handle.get(), ppszName, ppszValue,
-                                     pu64Timestamp, ppszFlags);
-        if (RT_SUCCESS(rc) && *ppszName != NULL)
-            *ppHandle = Handle.release();
-        else if (RT_SUCCESS(rc))
-            rc = VERR_NOT_FOUND; /* No matching properties found. */
+        rc = VbglR3GuestPropEnumNext(pHandle, ppszName, ppszValue, pu64Timestamp, ppszFlags);
+        if (RT_SUCCESS(rc))
+        {
+            *ppHandle = pHandle;
+            return rc;
+        }
     }
     else if (rc == VERR_BUFFER_OVERFLOW)
         rc = VERR_TOO_MUCH_DATA;
+    RTMemFree(pchBuf);
+    RTMemFree(pHandle);
     return rc;
 }
 
@@ -777,11 +797,39 @@ VBGLR3DECL(int) VbglR3GuestPropEnumNext(PVBGLR3GUESTPROPENUM pHandle,
  */
 VBGLR3DECL(void) VbglR3GuestPropEnumFree(PVBGLR3GUESTPROPENUM pHandle)
 {
+    if (!pHandle)
+        return;
     RTMemFree(pHandle->pchBuf);
     RTMemFree(pHandle);
 }
 
 
+/**
+ * Deletes a guest property.
+ *
+ * @returns VBox status code.
+ * @param   u32ClientId     The client id returned by VbglR3InvsSvcConnect().
+ * @param   pszName         The property to delete.  Utf8
+ */
+VBGLR3DECL(int) VbglR3GuestPropDelete(uint32_t u32ClientId, const char *pszName)
+{
+    AssertPtrReturn(pszName,  VERR_INVALID_POINTER);
+
+    DelProperty Msg;
+
+    Msg.hdr.result = VERR_WRONG_ORDER;
+    Msg.hdr.u32ClientID = u32ClientId;
+    Msg.hdr.u32Function = DEL_PROP;
+    Msg.hdr.cParms = 1;
+    VbglHGCMParmPtrSetString(&Msg.name, pszName);
+    int rc = vbglR3DoIOCtl(VBOXGUEST_IOCTL_HGCM_CALL(sizeof(Msg)), &Msg, sizeof(Msg));
+    if (RT_SUCCESS(rc))
+        rc = Msg.hdr.result;
+
+    return rc;
+}
+
+
 /**
  * Deletes a set of keys.
  *
@@ -815,7 +863,7 @@ VBGLR3DECL(int) VbglR3GuestPropDelSet(uint32_t u32ClientId,
     while (RT_SUCCESS(rc) && pszName)
     {
         rc = VbglR3GuestPropWriteValue(u32ClientId, pszName, NULL);
-        if (!RT_SUCCESS(rc))
+        if (RT_FAILURE(rc))
             break;
 
         rc = VbglR3GuestPropEnumNext(pHandle,
@@ -946,4 +994,4 @@ VBGLR3DECL(int) VbglR3GuestPropWait(uint32_t u32ClientId,
 
     return VINF_SUCCESS;
 }
-#endif /* VBOX_VBGLR3_XFREE86 */
+#endif /* VBOX_VBGLR3_XSERVER */
