@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# Linux Additions X11 setup init script ($Revision: 91647 $)
+# Linux Additions X11 setup init script ($Revision: 101523 $)
 #
 
 #
@@ -38,17 +38,19 @@ cpu=`uname -m`;
 case "$cpu" in
   i[3456789]86|x86)
     cpu="x86"
-    LIB="/usr/lib"
+    lib_candidates="/usr/lib/i386-linux-gnu /usr/lib /lib"
     ;;
   x86_64|amd64)
     cpu="amd64"
-    if test -d "/usr/lib64"; then
-      LIB="/usr/lib64"
-    else
-      LIB="/usr/lib"
-    fi
+    lib_candidates="/usr/lib/x86_64-linux-gnu /usr/lib64 /usr/lib /lib64 /lib"
     ;;
 esac
+for i in $lib_candidates; do
+  if test -d "$i/VBoxGuestAdditions"; then
+    LIB=$i
+    break
+  fi
+done
 
 # Find the version of X installed
 # The last of the three is for the X.org 6.7 included in Fedora Core 2
@@ -66,9 +68,7 @@ x11conf_files="/etc/X11/xorg.conf /etc/X11/xorg.conf-4 /etc/X11/.xorg.conf \
     /usr/X11R6/etc/X11/XF86Config /usr/X11R6/lib/X11/XF86Config-4 \
     /usr/X11R6/lib/X11/XF86Config"
 
-if [ -f /etc/arch-release ]; then
-    system=arch
-elif [ -f /etc/redhat-release ]; then
+if [ -f /etc/redhat-release ]; then
     system=redhat
 elif [ -f /etc/debian_version ]; then
     system=debian
@@ -82,22 +82,6 @@ else
     system=other
 fi
 
-if [ "$system" = "arch" ]; then
-    USECOLOR=yes
-    . /etc/rc.d/functions
-    fail_msg() {
-        stat_fail
-    }
-
-    succ_msg() {
-        stat_done
-    }
-
-    begin() {
-        stat_busy "$1"
-    }
-fi
-
 if [ "$system" = "redhat" ]; then
     . /etc/init.d/functions
     fail_msg() {
@@ -284,9 +268,9 @@ setup()
     dox11config="true"
     # By default, we want to run our xorg.conf setup script
     setupxorgconf="true"
-    # On all but the oldest X servers we want to use our new mouse
-    # driver.
-    newmouse="--newMouse"
+    # All but the oldest supported X servers can automatically set up the
+    # keyboard driver.
+    autokeyboard="--autoKeyboard"
     # On more recent servers our kernel mouse driver will be used
     # automatically
     automouse="--autoMouse"
@@ -295,12 +279,23 @@ setup()
     case "`uname -r`" in 2.4.*)
         test -c /dev/psaux && nopsaux="";;
     esac
+    # Should we use the VMSVGA driver instead of VBoxVideo?
+    if grep 80eebeef /proc/bus/pci/devices > /dev/null; then
+    vmsvga=""
+    elif grep 15ad0405 /proc/bus/pci/devices > /dev/null; then
+        vmsvga="--vmsvga"
+    else
+        dox11config=""
+    fi
     # The video driver to install for X.Org 6.9+
     vboxvideo_src=
     # The mouse driver to install for X.Org 6.9+
     vboxmouse_src=
     # The driver extension
     driver_ext=".so"
+    # The configuration file we generate if no original was found but we need
+    # one.
+    main_cfg="/etc/X11/xorg.conf"
 
     modules_dir=`X -showDefaultModulePath 2>&1` || modules_dir=
     if [ -z "$modules_dir" ]; then
@@ -336,24 +331,24 @@ setup()
         1.11.* )
             xserver_version="X.Org Server 1.11"
             vboxvideo_src=vboxvideo_drv_111.so
-            test "$system" = "redhat" || setupxorgconf=""
+            test "$system" = "redhat" && test -z "${vmsvga}" || setupxorgconf=""
             ;;
         1.10.* )
             xserver_version="X.Org Server 1.10"
             vboxvideo_src=vboxvideo_drv_110.so
-            test "$system" = "redhat" || setupxorgconf=""
+            test "$system" = "redhat" && test -z "${vmsvga}" || setupxorgconf=""
             ;;
         1.9.* )
             xserver_version="X.Org Server 1.9"
             vboxvideo_src=vboxvideo_drv_19.so
             # Fedora 14 to 16 patched out vboxvideo detection
-            test "$system" = "redhat" || setupxorgconf=""
+            test "$system" = "redhat" && test -z "${vmsvga}" || setupxorgconf=""
             ;;
         1.8.* )
             xserver_version="X.Org Server 1.8"
             vboxvideo_src=vboxvideo_drv_18.so
             # Fedora 13 shipped without vboxvideo detection
-            test "$system" = "redhat" || setupxorgconf=""
+            test "$system" = "redhat" && test -z "${vmsvga}" || setupxorgconf=""
             ;;
         1.7.* )
             xserver_version="X.Org Server 1.7"
@@ -376,9 +371,12 @@ setup()
             xserver_version="X.Org Server 1.5"
             vboxvideo_src=vboxvideo_drv_15.so
             vboxmouse_src=vboxmouse_drv_15.so
-            # SUSE with X.Org 1.5 is another special case, and is also
-            # handled specially
-            test "$system" = "suse" && automouse=""
+            # Historical note: SUSE with X.Org Server 1.5 disabled automatic
+            # mouse configuration and was handled specially.  However since our
+            # kernel driver seems to have problems with X.Org Server 1.5 anyway
+            # we just create an X.Org configuration file and use the user space
+            # one generally, no longer just for SUSE.
+            automouse=""
             ;;
         1.4.* )
             xserver_version="X.Org Server 1.4"
@@ -407,12 +405,22 @@ setup()
             automouse=""
             ;;
         6.7* | 6.8.* | 4.2.* | 4.3.* )
-            # Assume X.Org post-fork or XFree86
+            # As the module binaries are the same we use one text for these
+            # four server versions.
             xserver_version="XFree86 4.2/4.3 and X.Org 6.7/6.8"
             driver_ext=.o
             vboxvideo_src=vboxvideo_drv.o
             vboxmouse_src=vboxmouse_drv.o
             automouse=""
+            autokeyboard=""
+            case $x_version in
+                6.8.* )
+                    autokeyboard="true"
+                    ;;
+                4.2.* | 4.3.* )
+                    main_cfg="/etc/X11/XF86Config"
+                    ;;
+            esac
             ;;
         * )
             # Anything else, including all X server versions as of 1.12.
@@ -462,7 +470,7 @@ setup()
                     if grep -q "VirtualBox generated" "$i"; then
                         generated="$generated  `printf "$i\n"`"
                     else
-                        "$lib_dir/x11config.sh" $newmouse $automouse $nopsaux "$i"
+                        "$lib_dir/x11config.sh" $autokeyboard $automouse $nopsaux $vmsvga "$i"
                     fi
                     configured="true"
                 fi
@@ -472,12 +480,11 @@ setup()
             done
             # X.Org Server 1.5 and 1.6 can detect hardware they know, but they
             # need a configuration file for VBoxVideo.
-            main_cfg="/etc/X11/xorg.conf"
-            nobak="/etc/X11/xorg.vbox.nobak"
+            nobak_cfg="`expr "${main_cfg}" : '\([^.]*\)'`.vbox.nobak"
             if test -z "$configured"; then
                 touch "$main_cfg"
-                "$lib_dir/x11config.sh" $newmouse $automouse $nopsaux --noBak "$main_cfg"
-                touch "$nobak"
+                "$lib_dir/x11config.sh" $autokeyboard $automouse $nopsaux $vmsvga --noBak "$main_cfg"
+                touch "${nobak_cfg}"
             fi
         fi
         succ_msg
@@ -490,8 +497,8 @@ $generated
 
 EOF
         cat << EOF
-You may need to restart the hal service and the Window System (or just restart
-the guest system) to enable the Guest Additions.
+You may need to restart the the Window System (or just restart the guest system)
+to enable the Guest Additions.
 
 EOF
     fi
@@ -537,6 +544,7 @@ EOF
     # And set up VBoxClient to start when the X session does
     install_x11_startup_app "$lib_dir/98vboxadd-xclient" "$share_dir/vboxclient.desktop" VBoxClient VBoxClient-all ||
         fail "See the log file $LOG for more information."
+    ln -s "$lib_dir/98vboxadd-xclient" /usr/bin/VBoxClient-all 2>/dev/null
     succ_msg
 }
 
@@ -544,24 +552,30 @@ EOF
 cleanup()
 {
     # Restore xorg.conf files as far as possible
-    ## List of generated files which have been changed since we generated them
+    # List of generated files which have been changed since we generated them
     newer=""
-    ## Are we dealing with a legacy information which didn't support
+    # Are we dealing with a legacy information which didn't support
     # uninstallation?
     legacy=""
-    ## Do any of the restored configuration files still reference our drivers?
+    # Do any of the restored configuration files still reference our drivers?
     failed=""
+    # Have we encountered a "nobak" configuration file which means that there
+    # is no original file to restore?
+    nobak=""
     test -r "$CONFIG_DIR/$CONFIG" || legacy="true"
-    main_cfg="/etc/X11/xorg.conf"
-    nobak="/etc/X11/xorg.vbox.nobak"
-    if test -r "$nobak"; then
-        test -r "$main_cfg" &&
-            if test -n "$legacy" -o ! "$nobak" -ot "$main_cfg"; then
-                rm -f "$nobak" "$main_cfg"
+    for main_cfg in "/etc/X11/xorg.conf" "/etc/X11/XF86Config"; do
+        nobak_cfg="`expr "${main_cfg}" : '\([^.]*\)'`.vbox.nobak"
+        if test -r "${nobak_cfg}"; then
+            test -r "${main_cfg}" &&
+            if test -n "${legacy}" -o ! "${nobak_cfg}" -ot "${main_cfg}"; then
+                rm -f "${nobak_cfg}" "${main_cfg}"
             else
-                newer="$newer`printf "  $main_cfg (no original)\n"`"
+                newer="${newer}`printf "  ${main_cfg} (no original)\n"`"
             fi
-    else
+            nobak="true"
+        fi
+    done
+    if test -z "${nobak}"; then
         for i in $x11conf_files; do
             if test -r "$i.vbox"; then
                 if test ! "$i" -nt "$i.vbox" -o -n "$legacy"; then
@@ -596,14 +610,27 @@ $failed
 EOF
 
     # Remove X.Org drivers
-    find "$x11_modules_dir" /usr/lib64/xorg/modules /usr/lib/xorg/modules \
-        /usr/X11R6/lib64/modules /usr/X11R6/lib/modules \
-        /usr/X11R6/lib/X11/modules \
-        '(' -name 'vboxvideo_drv*' -o -name 'vboxmouse_drv*' ')' \
-        -exec rm -f '{}' ';' 2>/dev/null
+    modules_dir=`X -showDefaultModulePath 2>&1` || modules_dir=
+    if [ -z "$modules_dir" ]; then
+        for dir in /usr/lib64/xorg/modules /usr/lib/xorg/modules /usr/X11R6/lib64/modules /usr/X11R6/lib/modules /usr/X11R6/lib/X11/modules; do
+            if [ -d $dir ]; then
+                modules_dir=$dir
+                break
+            fi
+        done
+    fi
+    rm -f "$modules_dir/drivers/vboxvideo_drv"* 2>/dev/null
+    rm -f "$modules_dir/input/vboxmouse_drv"* 2>/dev/null
 
     # Remove the link to vboxvideo_dri.so
-    rm -f /usr/lib/dri/vboxvideo_dri.so /usr/lib64/dri/vboxvideo_dri.so 2>/dev/null
+    for dir in /usr/lib/dri /usr/lib32/dri /usr/lib64/dri \
+        /usr/lib/xorg/modules/dri /usr/lib32/xorg/modules/dri \
+        /usr/lib64/xorg/modules/dri /usr/lib/i386-linux-gnu/dri \
+        /usr/lib/x86_64-linux-gnu/dri; do
+        if [ -d $dir ]; then
+            rm -f "$dir/vboxvideo_dri.so" 2>/dev/null
+        fi
+    done
 
     # Remove VBoxClient autostart files
     rm /etc/X11/Xsession.d/98vboxadd-xclient 2>/dev/null
@@ -611,6 +638,7 @@ EOF
     rm /etc/X11/xinit/xinitrc.d/98vboxadd-xclient.sh 2>/dev/null
     rm /etc/xdg/autostart/vboxclient.desktop 2>/dev/null
     rm /usr/share/autostart/vboxclient.desktop 2>/dev/null
+    rm /usr/bin/VBoxClient-all 2>/dev/null
 
     # Remove other files
     rm /usr/share/xserver-xorg/pci/vboxvideo.ids 2>/dev/null
