@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# Linux Additions kernel module init script ($Revision: 99217 $)
+# Linux Additions kernel module init script ($Revision: 102425 $)
 #
 
 #
@@ -16,7 +16,7 @@
 #
 
 
-# chkconfig: 357 30 70
+# chkconfig: 345 30 70
 # description: VirtualBox Linux Additions kernel modules
 #
 ### BEGIN INIT INFO
@@ -57,9 +57,7 @@ for i in $lib_candidates; do
   fi
 done
 
-if [ -f /etc/arch-release ]; then
-    system=arch
-elif [ -f /etc/redhat-release ]; then
+if [ -f /etc/redhat-release ]; then
     system=redhat
 elif [ -f /etc/SuSE-release ]; then
     system=suse
@@ -71,22 +69,6 @@ else
     system=other
 fi
 
-if [ "$system" = "arch" ]; then
-    USECOLOR=yes
-    . /etc/rc.d/functions
-    fail_msg() {
-        stat_fail
-    }
-
-    succ_msg() {
-        stat_done
-    }
-
-    begin() {
-        stat_busy "$1"
-    }
-fi
-
 if [ "$system" = "redhat" ]; then
     . /etc/init.d/functions
     fail_msg() {
@@ -418,10 +400,10 @@ setup_modules()
             --use-module-symvers /tmp/vboxguest-Module.symvers \
             --module-source $MODULE_SRC/vboxvideo \
             --no-print-directory install >> $LOG 2>&1; then
-            show_error "Look at $LOG to find out what went wrong"
-            return 1
+            show_error "Look at $LOG to find out what went wrong. The module is not built but the others are."
+        else
+            succ_msg
         fi
-        succ_msg
     fi
     depmod
     return 0
@@ -472,6 +454,9 @@ extra_setup()
 
     # Put mount.vboxsf in the right place
     ln -sf "$lib_path/$PACKAGE/mount.vboxsf" /sbin
+    # And an rc file to re-build the kernel modules and re-set-up the X server.
+    ln -sf "$lib_path/$PACKAGE/vboxadd" /sbin/rcvboxadd
+    ln -sf "$lib_path/$PACKAGE/vboxadd-x11" /sbin/rcvboxadd-x11
     # At least Fedora 11 and Fedora 12 require the correct security context when
     # executing this command from service scripts. Shouldn't hurt for other
     # distributions.
@@ -546,6 +531,8 @@ cleanup()
 
     # Remove other files
     rm /sbin/mount.vboxsf 2>/dev/null
+    rm /sbin/rcvboxadd 2>/dev/null
+    rm /sbin/rcvboxadd-x11 2>/dev/null
     rm /etc/udev/rules.d/60-vboxadd.rules 2>/dev/null
 }
 
