@@ -1,3 +1,4 @@
+/* $Id: vboxvideo.h $ */
 /** @file
  *
  * VirtualBox X11 Additions graphics driver
@@ -64,17 +65,17 @@
 
 #define TRACE_ENTRY() \
 do { \
-    vbvxMsg(__PRETTY_FUNCTION__); \
+    vbvxMsg(RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
     vbvxMsg(": entering\n"); \
 } while(0)
 #define TRACE_EXIT() \
 do { \
-    vbvxMsg(__PRETTY_FUNCTION__); \
+    vbvxMsg(RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
     vbvxMsg(": leaving\n"); \
 } while(0)
 #define TRACE_LOG(...) \
 do { \
-    vbvxMsg("%s: ", __PRETTY_FUNCTION__); \
+    vbvxMsg("%s: ", RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
     vbvxMsg(__VA_ARGS__); \
 } while(0)
 # define TRACE_LINE() do \
@@ -95,7 +96,7 @@ if (!(expr)) \
 { \
     vbvxMsg("\nAssertion failed!\n\n"); \
     vbvxMsg("%s\n", #expr); \
-    vbvxMsg("at %s (%s:%d)\n", __PRETTY_FUNCTION__, __FILE__, __LINE__); \
+    vbvxMsg("at %s (%s:%d)\n", RT_GCC_EXTENSION __PRETTY_FUNCTION__, __FILE__, __LINE__); \
     vbvxMsg out; \
     vbvxAbortServer(); \
 }
