@@ -80,9 +80,9 @@ typedef struct _ExtensionEntry {
  */
 typedef void (*EventSwapPtr) (xEvent *, xEvent *);
 
-extern _X_EXPORT EventSwapPtr EventSwapVector[128];
+extern EventSwapPtr EventSwapVector[128];
 
-extern _X_EXPORT void NotImplemented (	/* FIXME: this may move to another file... */
+extern void NotImplemented (	/* FIXME: this may move to another file... */
 	xEvent *,
 	xEvent *);
 
@@ -91,7 +91,7 @@ extern _X_EXPORT void NotImplemented (	/* FIXME: this may move to another file..
 
 #define    GetGCValue(pGC, GCElement)    (pGC->GCElement)
 
-extern _X_EXPORT ExtensionEntry *AddExtension(
+extern ExtensionEntry *AddExtension(
     char* /*name*/,
     int /*NumEvents*/,
     int /*NumErrors*/,
@@ -101,12 +101,12 @@ extern _X_EXPORT ExtensionEntry *AddExtension(
     unsigned short (* /*MinorOpcodeProc*/)(ClientPtr /*client*/)
 );
 
-extern _X_EXPORT Bool AddExtensionAlias(
+extern Bool AddExtensionAlias(
     char* /*alias*/,
     ExtensionEntry * /*extension*/);
 
-extern _X_EXPORT ExtensionEntry *CheckExtension(const char *extname);
-extern _X_EXPORT ExtensionEntry *GetExtensionEntry(int major);
+extern ExtensionEntry *CheckExtension(const char *extname);
+extern ExtensionEntry *GetExtensionEntry(int major);
 
 #endif /* EXTENSIONSTRUCT_H */
 
