@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -84,17 +84,6 @@ ip_init(PNATState pData)
     tcp_init(pData);
 }
 
-static struct libalias *select_alias(PNATState pData, struct mbuf* m)
-{
-    struct libalias *la = pData->proxy_alias;
-
-    struct m_tag *t;
-    if ((t = m_tag_find(m, PACKET_TAG_ALIAS, NULL)) != 0)
-        return (struct libalias *)&t[1];
-
-    return la;
-}
-
 /*
  * Ip input routine.  Checksum and byte swap header.  If fragmented
  * try to reassemble.  Process options.  Pass to next level.
@@ -118,7 +107,7 @@ ip_input(PNATState pData, struct mbuf *m)
         if (!(m->m_flags & M_SKIP_FIREWALL))
         {
             STAM_PROFILE_START(&pData->StatALIAS_input, b);
-            rc = LibAliasIn(select_alias(pData, m), mtod(m, char *), m_length(m, NULL));
+            rc = LibAliasIn(pData->proxy_alias, mtod(m, char *), m_length(m, NULL));
             STAM_PROFILE_STOP(&pData->StatALIAS_input, b);
             Log2(("NAT: LibAlias return %d\n", rc));
         }
@@ -198,11 +187,16 @@ ip_input(PNATState pData, struct mbuf *m)
     /* check ip_ttl for a correct ICMP reply */
     if (ip->ip_ttl==0 || ip->ip_ttl == 1)
     {
+        /* XXX: if we're in destination so perhaps we need to send ICMP_TIMXCEED_REASS */
         icmp_error(pData, m, ICMP_TIMXCEED, ICMP_TIMXCEED_INTRANS, 0, "ttl");
         goto no_free_m;
     }
 
     ip->ip_ttl--;
+    if (ip->ip_sum > RT_H2N_U16_C(0xffffU - (1 << 8)))
+        ip->ip_sum += RT_H2N_U16_C(1 << 8) + 1;
+    else
+        ip->ip_sum += RT_H2N_U16_C(1 << 8);
 
     /*
      * Drop multicast (class d) and reserved (class e) here.  The rest
