@@ -1,12 +1,10 @@
 /* $Id: UIVMLogViewer.cpp $ */
 /** @file
- *
- * VBox frontends: Qt4 GUI ("VirtualBox"):
- * UIVMLogViewer class implementation
+ * VBox Qt GUI - UIVMLogViewer class implementation.
  */
 
 /*
- * Copyright (C) 2008-2011 Oracle Corporation
+ * Copyright (C) 2008-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,34 +16,37 @@
  */
 
 #ifdef VBOX_WITH_PRECOMPILED_HEADERS
-# include "precomp.h"
+# include <precomp.h>
 #else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
 /* Qt includes: */
-#include <QCheckBox>
-#include <QDateTime>
-#include <QDir>
-#include <QFileDialog>
-#include <QKeyEvent>
-#include <QLabel>
-#include <QScrollBar>
-#include <QTextEdit>
+# include <QCheckBox>
+# include <QDateTime>
+# include <QDir>
+# include <QFileDialog>
+# include <QKeyEvent>
+# include <QLabel>
+# include <QScrollBar>
+# include <QTextEdit>
+# include <QDesktopWidget>
 
 /* GUI includes: */
-#include "UIVMLogViewer.h"
-#include "QITabWidget.h"
-#include "UIIconPool.h"
-#include "UISpecialControls.h"
-#include "VBoxGlobal.h"
-#include "UIMessageCenter.h"
-#include "VBoxUtils.h"
+# include "UIVMLogViewer.h"
+# include "QITabWidget.h"
+# include "UIIconPool.h"
+# include "UISpecialControls.h"
+# include "VBoxGlobal.h"
+# include "UIMessageCenter.h"
+# include "VBoxUtils.h"
+# include "UIExtraDataManager.h"
 
 /* COM includes: */
-#include "COMEnums.h"
-#include "CSystemProperties.h"
+# include "COMEnums.h"
+# include "CSystemProperties.h"
 
 #endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
+
 /* VM Log Viewer search panel: */
 class UIVMLogViewerSearchPanel : public QIWithRetranslateUI<QWidget>
 {
@@ -73,13 +74,13 @@ public:
         m_pSearchLabel->setBuddy(m_pSearchEditor);
 
         /* Next/Previous buttons: */
-        m_pNextPrevButtons = new UIRoundRectSegmentedButton(2, this);
+        m_pNextPrevButtons = new UIRoundRectSegmentedButton(this, 2);
         m_pNextPrevButtons->setEnabled(0, false);
         m_pNextPrevButtons->setEnabled(1, false);
 #ifndef Q_WS_MAC
         /* No icons on the Mac: */
-        m_pNextPrevButtons->setIcon(0, UIIconPool::defaultIcon(UIIconPool::ArrowBackIcon, this));
-        m_pNextPrevButtons->setIcon(1, UIIconPool::defaultIcon(UIIconPool::ArrowForwardIcon, this));
+        m_pNextPrevButtons->setIcon(0, UIIconPool::defaultIcon(UIIconPool::UIDefaultIconType_ArrowBack, this));
+        m_pNextPrevButtons->setIcon(1, UIIconPool::defaultIcon(UIIconPool::UIDefaultIconType_ArrowForward, this));
 #endif /* !Q_WS_MAC */
 
         /* Case sensitive check-box: */
@@ -89,7 +90,7 @@ public:
         m_pWarningSpacer = new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Minimum);
         m_pWarningIcon = new QLabel(this);
         m_pWarningIcon->hide();
-        QIcon icon = UIIconPool::defaultIcon(UIIconPool::MessageBoxWarningIcon, this);
+        QIcon icon = UIIconPool::defaultIcon(UIIconPool::UIDefaultIconType_MessageBoxWarning, this);
         if (!icon.isNull())
             m_pWarningIcon->setPixmap(icon.pixmap(16, 16));
         m_pWarningLabel = new QLabel(this);
@@ -301,9 +302,17 @@ private:
 
         int iResult = -1;
         if (fForward && (fStartCurrent || iPos < strText.size() - 1))
+        {
             iResult = strText.indexOf(m_pSearchEditor->text(), iAnc + iDiff,
                                       m_pCaseSensitiveCheckBox->isChecked() ?
                                       Qt::CaseSensitive : Qt::CaseInsensitive);
+
+            /* When searchstring is changed, search from beginning of log again: */
+            if (iResult == -1 && fStartCurrent)
+                iResult = strText.indexOf(m_pSearchEditor->text(), 0,
+                                          m_pCaseSensitiveCheckBox->isChecked() ?
+                                          Qt::CaseSensitive : Qt::CaseInsensitive);
+        }
         else if (!fForward && iAnc > 0)
             iResult = strText.lastIndexOf(m_pSearchEditor->text(), iAnc - 1,
                                           m_pCaseSensitiveCheckBox->isChecked() ?
@@ -383,9 +392,7 @@ UIVMLogViewer::UIVMLogViewer(QWidget *pParent, Qt::WindowFlags flags, const CMac
     Ui::UIVMLogViewer::setupUi(this);
 
     /* Apply window icons: */
-    setWindowIcon(UIIconPool::iconSetFull(QSize(32, 32), QSize(16, 16),
-                                          ":/vm_show_logs_32px.png",
-                                          ":/show_logs_16px.png"));
+    setWindowIcon(UIIconPool::iconSetFull(":/vm_show_logs_32px.png", ":/vm_show_logs_16px.png"));
 
     /* Create VM Log Vewer container: */
     m_pViewerContainer = new QITabWidget(centralWidget());
@@ -414,12 +421,18 @@ UIVMLogViewer::UIVMLogViewer(QWidget *pParent, Qt::WindowFlags flags, const CMac
     /* Reading log files: */
     refresh();
 
+    /* Load settings: */
+    loadSettings();
+
     /* Loading language constants */
     retranslateUi();
 }
 
 UIVMLogViewer::~UIVMLogViewer()
 {
+    /* Save settings: */
+    saveSettings();
+
     if (!m_machine.isNull())
         m_viewers.remove(m_machine.GetName());
 }
@@ -481,6 +494,10 @@ void UIVMLogViewer::refresh()
                 /* Create a log viewer page and append the read text to it: */
                 QTextEdit *pLogViewer = createLogPage(QFileInfo(strFileName).fileName());
                 pLogViewer->setPlainText(strText);
+                /* Move the cursor position to end: */
+                QTextCursor cursor = pLogViewer->textCursor();
+                cursor.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);
+                pLogViewer->setTextCursor(cursor);
                 /* Add the actual file name and the QTextEdit containing the content to a list: */
                 m_book << qMakePair(strFileName, pLogViewer);
                 isAnyLogPresent = true;
@@ -562,28 +579,12 @@ void UIVMLogViewer::showEvent(QShowEvent *pEvent)
     if (m_fIsPolished)
         return;
 
-    m_fIsPolished = true;
-
-    /* Resize the whole log-viewer to fit 80 symbols in
-     * text-browser for the first time started: */
-    QTextEdit *pFirstPage = currentLogPage();
-    if (pFirstPage)
-    {
-        int fullWidth = pFirstPage->fontMetrics().width(QChar('x')) * 80 +
-                        pFirstPage->verticalScrollBar()->width() +
-                        pFirstPage->frameWidth() * 2 +
-                        /* m_pViewerContainer margin */ 10 * 2 +
-                        /* CentralWidget margin */ 10 * 2;
-        resize(fullWidth, height());
-    }
+    m_fIsPolished = true;   
 
     /* Make sure the log view widget has the focus */
     QWidget *pCurrentLogPage = currentLogPage();
     if (pCurrentLogPage)
-        pCurrentLogPage->setFocus();
-
-    /* Explicit widget centering relatively to it's parent: */
-    VBoxGlobal::centerWidget(this, parentWidget(), false);
+        pCurrentLogPage->setFocus(); 
 }
 
 void UIVMLogViewer::keyPressEvent(QKeyEvent *pEvent)
@@ -597,7 +598,7 @@ void UIVMLogViewer::keyPressEvent(QKeyEvent *pEvent)
             mBtnClose->animateClick();
             return;
         }
-        /* Precess Back key as switch to previous tab: */
+        /* Process Back key as switch to previous tab: */
         case Qt::Key_Back:
         {
             if (m_pViewerContainer->currentIndex() > 0)
@@ -607,7 +608,7 @@ void UIVMLogViewer::keyPressEvent(QKeyEvent *pEvent)
             }
             break;
         }
-        /* Precess Forward key as switch to next tab: */
+        /* Process Forward key as switch to next tab: */
         case Qt::Key_Forward:
         {
             if (m_pViewerContainer->currentIndex() < m_pViewerContainer->count())
@@ -642,5 +643,60 @@ QTextEdit* UIVMLogViewer::createLogPage(const QString &strName)
     return pLogViewer;
 }
 
+void UIVMLogViewer::loadSettings()
+{
+    /* Restore window geometry: */
+    {
+        /* Getting available geometry to calculate default geometry: */
+        const QRect desktopRect = QApplication::desktop()->availableGeometry(this);
+        int iDefaultWidth = desktopRect.width() * 0.5;
+        int iDefaultHeight = desktopRect.height() * 0.75;
+
+        /* Calculate default width to fit 132 characters: */
+        QTextEdit *pCurrentLogPage = currentLogPage();
+        if (pCurrentLogPage)
+        {
+            iDefaultWidth = pCurrentLogPage->fontMetrics().width(QChar('x')) * 132 +
+                pCurrentLogPage->verticalScrollBar()->width() +
+                pCurrentLogPage->frameWidth() * 2 +
+                /* m_pViewerContainer margin */ 10 * 2 +
+                /* CentralWidget margin */ 10 * 2;
+        }
+        QRect defaultGeometry(0, 0, iDefaultWidth, iDefaultHeight);
+        defaultGeometry.moveCenter(parentWidget()->geometry().center());
+
+        /* Load geometry: */
+        m_geometry = gEDataManager->logWindowGeometry(this, defaultGeometry);
+#ifdef Q_WS_MAC
+        move(m_geometry.topLeft());
+        resize(m_geometry.size());
+#else /* Q_WS_MAC */
+        setGeometry(m_geometry);
+#endif /* !Q_WS_MAC */
+        LogRel(("GUI: UIVMLogViewer: Geometry loaded to: Origin=%dx%d, Size=%dx%d\n",
+                m_geometry.x(), m_geometry.y(), m_geometry.width(), m_geometry.height()));
+
+        /* Maximize (if necessary): */
+        if (gEDataManager->logWindowShouldBeMaximized())
+            showMaximized();
+    }
+}
+
+void UIVMLogViewer::saveSettings()
+{
+    /* Save window geometry: */
+    {
+        /* Save geometry: */
+        const QRect saveGeometry = geometry();
+#ifdef Q_WS_MAC
+        gEDataManager->setLogWindowGeometry(saveGeometry, ::darwinIsWindowMaximized(this));
+#else /* Q_WS_MAC */
+        gEDataManager->setLogWindowGeometry(saveGeometry, isMaximized());
+#endif /* !Q_WS_MAC */
+        LogRel(("GUI: UIVMLogViewer: Geometry saved as: Origin=%dx%d, Size=%dx%d\n",
+                saveGeometry.x(), saveGeometry.y(), saveGeometry.width(), saveGeometry.height()));
+    }
+}
+
 #include "UIVMLogViewer.moc"
 
