@@ -1,11 +1,10 @@
+/* $Id: UISession.h $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * UISession class declaration
+ * VBox Qt GUI - UISession class declaration.
  */
 
 /*
- * Copyright (C) 2010-2012 Oracle Corporation
+ * Copyright (C) 2010-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,40 +15,44 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___UIConsole_h___
-#define ___UIConsole_h___
+#ifndef ___UISession_h___
+#define ___UISession_h___
 
 /* Qt includes: */
 #include <QObject>
 #include <QCursor>
 #include <QEvent>
+#include <QMap>
 
 /* GUI includes: */
-#include "UIDefs.h"
+#include "UIExtraDataDefs.h"
+#include "UIMediumDefs.h"
 
 /* COM includes: */
 #include "COMEnums.h"
+#include "CSession.h"
+#include "CMachine.h"
+#include "CConsole.h"
+#include "CDisplay.h"
+#include "CGuest.h"
+#include "CMouse.h"
+#include "CKeyboard.h"
+#include "CMachineDebugger.h"
 
 /* Forward declarations: */
 class QMenu;
-class QMenuBar;
-#ifdef VBOX_GUI_WITH_KEYS_RESET_HANDLER
-# ifdef Q_WS_MAC
-struct __siginfo;
-typedef struct __siginfo siginfo_t;
-# else /* Q_WS_MAC */
-struct siginfo;
-typedef struct siginfo siginfo_t;
-# endif /* !Q_WS_MAC */
-#endif /* VBOX_GUI_WITH_KEYS_RESET_HANDLER */
 class UIFrameBuffer;
 class UIMachine;
 class UIMachineLogic;
-class UIMachineMenuBar;
-class CSession;
+class UIActionPool;
 class CUSBDevice;
 class CNetworkAdapter;
 class CMediumAttachment;
+#ifdef Q_WS_MAC
+class QMenuBar;
+#else /* !Q_WS_MAC */
+class QIcon;
+#endif /* !Q_WS_MAC */
 
 /* CConsole callback event types: */
 enum UIConsoleEventType
@@ -82,44 +85,60 @@ class UISession : public QObject
 
 public:
 
-    /* Machine uisession constructor/destructor: */
-    UISession(UIMachine *pMachine, CSession &session);
-    virtual ~UISession();
+    /** Factory constructor. */
+    static bool create(UISession *&pSession, UIMachine *pMachine);
+    /** Factory destructor. */
+    static void destroy(UISession *&pSession);
 
-    /* Common members: */
-    void powerUp();
-    bool save();
+    /* API: Runtime UI stuff: */
+    bool initialize();
+    bool powerUp();
+    bool saveState();
     bool shutdown();
-    bool powerOff(bool fDiscardState, bool &fServerCrashed);
+    bool powerOff(bool fIncludingDiscard, bool &fServerCrashed);
+    bool restoreCurrentSnapshot();
 
-    /* Common getters: */
+    /** Returns the session instance. */
     CSession& session() { return m_session; }
+    /** Returns the session's machine instance. */
+    CMachine& machine() { return m_machine; }
+    /** Returns the session's console instance. */
+    CConsole& console() { return m_console; }
+    /** Returns the console's display instance. */
+    CDisplay& display() { return m_display; }
+    /** Returns the console's guest instance. */
+    CGuest& guest() { return m_guest; }
+    /** Returns the console's mouse instance. */
+    CMouse& mouse() { return m_mouse; }
+    /** Returns the console's keyboard instance. */
+    CKeyboard& keyboard() { return m_keyboard; }
+    /** Returns the console's debugger instance. */
+    CMachineDebugger& debugger() { return m_debugger; }
+
+    /** Returns the machine name. */
+    const QString& machineName() const { return m_strMachineName; }
+
+    UIActionPool* actionPool() const { return m_pActionPool; }
     KMachineState machineStatePrevious() const { return m_machineStatePrevious; }
     KMachineState machineState() const { return m_machineState; }
     UIMachineLogic* machineLogic() const;
     QWidget* mainMachineWindow() const;
-    QMenu* newMenu(RuntimeMenuType fOptions = RuntimeMenuType_All);
-    QMenuBar* newMenuBar(RuntimeMenuType fOptions = RuntimeMenuType_All);
     QCursor cursor() const { return m_cursor; }
 
-    /** @name Runtime menus configuration stuff.
+#ifndef Q_WS_MAC
+    /** @name Branding stuff.
      ** @{ */
-#ifdef Q_WS_MAC
-    /** Determines Application menu allowed actions. */
-    RuntimeMenuApplicationActionType allowedActionsMenuApplication() const { return m_allowedActionsMenuApplication; }
-#endif /* Q_WS_MAC */
-    /** Determines Machine menu allowed actions. */
-    RuntimeMenuMachineActionType allowedActionsMenuMachine() const { return m_allowedActionsMenuMachine; }
-    /** Determines View menu allowed actions. */
-    RuntimeMenuViewActionType allowedActionsMenuView() const { return m_allowedActionsMenuView; }
-    /** Determines Devices menu allowed actions. */
-    RuntimeMenuDevicesActionType allowedActionsMenuDevices() const { return m_allowedActionsMenuDevices; }
-#ifdef VBOX_WITH_DEBUGGER_GUI
-    /** Determines Debugger menu allowed actions. */
-    RuntimeMenuDebuggerActionType allowedActionsMenuDebugger() const { return m_allowedActionsMenuDebugger; }
-#endif /* VBOX_WITH_DEBUGGER_GUI */
-    /** Determines Help menu allowed actions. */
-    RuntimeMenuHelpActionType allowedActionsMenuHelp() const { return m_allowedActionsMenuHelp; }
+    /** Returns redefined machine-window icon. */
+    QIcon* machineWindowIcon() const { return m_pMachineWindowIcon; }
+    /** Returns redefined machine-window name postfix. */
+    QString machineWindowNamePostfix() const { return m_strMachineWindowNamePostfix; }
+    /** @} */
+#endif /* !Q_WS_MAC */
+
+    /** @name Host-screen configuration variables.
+     ** @{ */
+    /** Returns the list of host-screen geometries we currently have. */
+    QList<QRect> hostScreens() const { return m_hostScreens; }
     /** @} */
 
     /** @name Application Close configuration stuff.
@@ -132,10 +151,14 @@ public:
     bool isAllCloseActionsRestricted() const { return m_fAllCloseActionsRestricted; }
     /** @} */
 
-    /* API: Visual-state stuff: */
-    bool isVisualStateAllowedFullscreen() const;
-    bool isVisualStateAllowedSeamless() const;
-    bool isVisualStateAllowedScale() const;
+    /** Returns whether visual @a state is allowed. */
+    bool isVisualStateAllowed(UIVisualStateType state) const;
+    /** Requests visual-state change. */
+    void changeVisualState(UIVisualStateType visualStateType);
+    /** Requests visual-state to be entered when possible. */
+    void setRequestedVisualState(UIVisualStateType visualStateType) { m_requestedVisualStateType = visualStateType; }
+    /** Returns requested visual-state to be entered when possible. */
+    UIVisualStateType requestedVisualState() const { return m_requestedVisualStateType; }
 
     bool isSaved() const { return machineState() == KMachineState_Saved; }
     bool isTurnedOff() const { return machineState() == KMachineState_PoweredOff ||
@@ -150,33 +173,33 @@ public:
     bool isStuck() const { return machineState() == KMachineState_Stuck; }
     bool wasPaused() const { return machineStatePrevious() == KMachineState_Paused ||
                                     machineStatePrevious() == KMachineState_TeleportingPausedVM; }
+    bool isInitialized() const { return m_fInitialized; }
     bool isFirstTimeStarted() const { return m_fIsFirstTimeStarted; }
-    bool isIgnoreRuntimeMediumsChanging() const { return m_fIsIgnoreRuntimeMediumsChanging; }
     bool isGuestResizeIgnored() const { return m_fIsGuestResizeIgnored; }
-    bool isSeamlessModeRequested() const { return m_fIsSeamlessModeRequested; }
     bool isAutoCaptureDisabled() const { return m_fIsAutoCaptureDisabled; }
 
     /* Guest additions state getters: */
-    bool isGuestAdditionsActive() const { return (m_ulGuestAdditionsRunLevel > AdditionsRunLevelType_None); }
-    bool isGuestSupportsGraphics() const { return isGuestAdditionsActive() && m_fIsGuestSupportsGraphics; }
+    bool isGuestAdditionsActive() const { return (m_ulGuestAdditionsRunLevel > KAdditionsRunLevelType_None); }
+    bool isGuestSupportsGraphics() const { return m_fIsGuestSupportsGraphics; }
+    /* The double check below is correct, even though it is an implementation
+     * detail of the Additions which the GUI should not ideally have to know. */
     bool isGuestSupportsSeamless() const { return isGuestSupportsGraphics() && m_fIsGuestSupportsSeamless; }
 
     /* Keyboard getters: */
+    /** Returns keyboard-state. */
+    int keyboardState() const { return m_iKeyboardState; }
     bool isNumLock() const { return m_fNumLock; }
     bool isCapsLock() const { return m_fCapsLock; }
     bool isScrollLock() const { return m_fScrollLock; }
-    bool isHostNumLock() const { return m_fHostNumLock; }
-    bool isHostCapsLock() const { return m_fHostCapsLock; }
-    bool isHostScrollLock() const { return m_fHostScrollLock; }
-    void setHostNumLock(bool fHostNumLock) { m_fHostNumLock = fHostNumLock; }
-    void setHostCapsLock(bool fHostCapsLock) { m_fHostCapsLock = fHostCapsLock; }
-    void setHostScrollLock(bool fHostScrollLock) { m_fHostScrollLock = fHostScrollLock; }
     uint numLockAdaptionCnt() const { return m_uNumLockAdaptionCnt; }
     uint capsLockAdaptionCnt() const { return m_uCapsLockAdaptionCnt; }
 
     /* Mouse getters: */
+    /** Returns mouse-state. */
+    int mouseState() const { return m_iMouseState; }
     bool isMouseSupportsAbsolute() const { return m_fIsMouseSupportsAbsolute; }
     bool isMouseSupportsRelative() const { return m_fIsMouseSupportsRelative; }
+    bool isMouseSupportsMultiTouch() const { return m_fIsMouseSupportsMultiTouch; }
     bool isMouseHostCursorNeeded() const { return m_fIsMouseHostCursorNeeded; }
     bool isMouseCaptured() const { return m_fIsMouseCaptured; }
     bool isMouseIntegrated() const { return m_fIsMouseIntegrated; }
@@ -188,7 +211,6 @@ public:
     bool unpause() { return setPause(false); }
     bool setPause(bool fOn);
     void setGuestResizeIgnored(bool fIsGuestResizeIgnored) { m_fIsGuestResizeIgnored = fIsGuestResizeIgnored; }
-    void setSeamlessModeRequested(bool fIsSeamlessModeRequested) { m_fIsSeamlessModeRequested = fIsSeamlessModeRequested; }
     void setAutoCaptureDisabled(bool fIsAutoCaptureDisabled) { m_fIsAutoCaptureDisabled = fIsAutoCaptureDisabled; }
     void forgetPreviousMachineState() { m_machineStatePrevious = m_machineState; }
 
@@ -204,24 +226,44 @@ public:
     bool isScreenVisible(ulong uScreenId) const;
     void setScreenVisible(ulong uScreenId, bool fIsMonitorVisible);
 
+    /* Last screen full-screen size: */
+    QSize lastFullScreenSize(ulong uScreenId) const;
+    void setLastFullScreenSize(ulong uScreenId, QSize size);
+
     /* Returns existing framebuffer for the given screen-number;
      * Returns 0 (asserts) if screen-number attribute is out of bounds: */
     UIFrameBuffer* frameBuffer(ulong uScreenId) const;
     /* Sets framebuffer for the given screen-number;
      * Ignores (asserts) if screen-number attribute is out of bounds: */
     void setFrameBuffer(ulong uScreenId, UIFrameBuffer* pFrameBuffer);
+    /** Returns existing frame-buffer vector. */
+    const QVector<UIFrameBuffer*>& frameBuffers() const { return m_frameBufferVector; }
+
+    /** Updates VRDE Server action state. */
+    void updateStatusVRDE() { sltVRDEChange(); }
+    /** Updates Video Capture action state. */
+    void updateStatusVideoCapture() { sltVideoCaptureChange(); }
 
 signals:
 
+    /** Notifies about frame-buffer resize. */
+    void sigFrameBufferResize();
+
     /* Console callback signals: */
+    /** Notifies listeners about keyboard state-change. */
+    void sigKeyboardStateChange(int iState);
+    /** Notifies listeners about mouse state-change. */
+    void sigMouseStateChange(int iState);
     void sigMousePointerShapeChange();
     void sigMouseCapabilityChange();
     void sigKeyboardLedsChange();
     void sigMachineStateChange();
     void sigAdditionsStateChange();
+    void sigAdditionsStateActualChange();
     void sigNetworkAdapterChange(const CNetworkAdapter &networkAdapter);
     void sigMediumChange(const CMediumAttachment &mediumAttachment);
     void sigVRDEChange();
+    void sigVideoCaptureChange();
     void sigUSBControllerChange();
     void sigUSBDeviceStateChange(const CUSBDevice &device, bool bIsAttached, const CVirtualBoxErrorInfo &error);
     void sigSharedFolderChange();
@@ -232,29 +274,47 @@ signals:
     void sigCPUExecutionCapChange();
     void sigGuestMonitorChange(KGuestMonitorChangedEventType changeType, ulong uScreenId, QRect screenGeo);
 
-    /* Notifiers: Qt callback stuff: */
-    void sigHostScreenCountChanged();
-    void sigHostScreenGeometryChanged();
+    /** Notifies about host-screen count change. */
+    void sigHostScreenCountChange();
+    /** Notifies about host-screen geometry change. */
+    void sigHostScreenGeometryChange();
+    /** Notifies about host-screen available-area change. */
+    void sigHostScreenAvailableAreaChange();
 
     /* Session signals: */
-    void sigMachineStarted();
+    void sigInitialized();
 
 public slots:
 
     void sltInstallGuestAdditionsFrom(const QString &strSource);
 
+    /** Defines @a iKeyboardState. */
+    void setKeyboardState(int iKeyboardState) { m_iKeyboardState = iKeyboardState; emit sigKeyboardStateChange(m_iKeyboardState); }
+
+    /** Defines @a iMouseState. */
+    void setMouseState(int iMouseState) { m_iMouseState = iMouseState; emit sigMouseStateChange(m_iMouseState); }
+
 private slots:
 
-    /* Close uisession handler: */
-    void sltCloseVirtualSession();
+    /** Marks machine started. */
+    void sltMarkInitialized() { m_fInitialized = true; }
+
+    /** Close Runtime UI. */
+    void sltCloseRuntimeUI();
+
+#ifdef RT_OS_DARWIN
+    /** Mac OS X: Handles menu-bar configuration-change. */
+    void sltHandleMenuBarConfigurationChange(const QString &strMachineID);
+#endif /* RT_OS_DARWIN */
 
     /* Console events slots */
     void sltMousePointerShapeChange(bool fVisible, bool fAlpha, QPoint hotCorner, QSize size, QVector<uint8_t> shape);
-    void sltMouseCapabilityChange(bool fSupportsAbsolute, bool fSupportsRelative, bool fNeedsHostCursor);
+    void sltMouseCapabilityChange(bool fSupportsAbsolute, bool fSupportsRelative, bool fSupportsMultiTouch, bool fNeedsHostCursor);
     void sltKeyboardLedsChangeEvent(bool fNumLock, bool fCapsLock, bool fScrollLock);
     void sltStateChange(KMachineState state);
     void sltAdditionsChange();
     void sltVRDEChange();
+    void sltVideoCaptureChange();
     void sltGuestMonitorChange(KGuestMonitorChangedEventType changeType, ulong uScreenId, QRect screenGeo);
 
     /* Handlers: Display reconfiguration stuff: */
@@ -262,58 +322,97 @@ private slots:
     void sltHandleHostDisplayAboutToChange();
     void sltCheckIfHostDisplayChanged();
 #endif /* RT_OS_DARWIN */
+
+    /** Handles host-screen count change. */
     void sltHandleHostScreenCountChange();
+    /** Handles host-screen geometry change. */
     void sltHandleHostScreenGeometryChange();
+    /** Handles host-screen available-area change. */
+    void sltHandleHostScreenAvailableAreaChange();
 
 private:
 
+    /** Constructor. */
+    UISession(UIMachine *pMachine);
+    /** Destructor. */
+    ~UISession();
+
     /* Private getters: */
     UIMachine* uimachine() const { return m_pMachine; }
 
     /* Prepare helpers: */
+    bool prepare();
+    bool prepareSession();
+    void prepareActions();
     void prepareConnections();
     void prepareConsoleEventHandlers();
     void prepareScreens();
     void prepareFramebuffers();
-    void prepareMenuPool();
     void loadSessionSettings();
 
     /* Cleanup helpers: */
     void saveSessionSettings();
-    void cleanupMenuPool();
     void cleanupFramebuffers();
     //void cleanupScreens() {}
     void cleanupConsoleEventHandlers();
     void cleanupConnections();
+    void cleanupActions();
+    void cleanupSession();
+    void cleanup();
 
-    /* Update helpers: */
-    void updateSessionSettings();
+#ifdef Q_WS_MAC
+    /** Mac OS X: Updates menu-bar content. */
+    void updateMenu();
+#endif /* Q_WS_MAC */
 
     /* Common helpers: */
     WId winId() const;
     void setPointerShape(const uchar *pShapeData, bool fHasAlpha, uint uXHot, uint uYHot, uint uWidth, uint uHeight);
-    void reinitMenuPool();
-    bool preparePowerUp();
+    bool preprocessInitialization();
+    bool mountAdHocImage(KDeviceType enmDeviceType, UIMediumType enmMediumType, const QString &strImage);
+    bool postprocessInitialization();
     int countOfVisibleWindows();
 
-#ifdef Q_WS_MAC
-    /* Helper: Display reconfiguration stuff: */
-    void recacheDisplayData();
-#endif /* Q_WS_MAC */
-
-#ifdef VBOX_GUI_WITH_KEYS_RESET_HANDLER
-    static void signalHandlerSIGUSR1(int sig, siginfo_t *pInfo, void *pSecret);
-#endif /* VBOX_GUI_WITH_KEYS_RESET_HANDLER */
+    /** Update host-screen data. */
+    void updateHostScreenData();
 
     /* Private variables: */
     UIMachine *m_pMachine;
-    CSession &m_session;
 
-    UIMachineMenuBar *m_pMenuPool;
+    /** Holds the session instance. */
+    CSession m_session;
+    /** Holds the session's machine instance. */
+    CMachine m_machine;
+    /** Holds the session's console instance. */
+    CConsole m_console;
+    /** Holds the console's display instance. */
+    CDisplay m_display;
+    /** Holds the console's guest instance. */
+    CGuest m_guest;
+    /** Holds the console's mouse instance. */
+    CMouse m_mouse;
+    /** Holds the console's keyboard instance. */
+    CKeyboard m_keyboard;
+    /** Holds the console's debugger instance. */
+    CMachineDebugger m_debugger;
+
+    /** Holds the machine name. */
+    QString m_strMachineName;
+
+    /** Holds the action-pool instance. */
+    UIActionPool *m_pActionPool;
+
+#ifdef Q_WS_MAC
+    /** Holds the menu-bar instance. */
+    QMenuBar *m_pMenuBar;
+#endif /* Q_WS_MAC */
 
     /* Screen visibility vector: */
     QVector<bool> m_monitorVisibilityVector;
 
+    /* Screen last full-screen size vector: */
+    QVector<QSize> m_monitorLastFullScreenSizeVector;
+
     /* Frame-buffers vector: */
     QVector<UIFrameBuffer*> m_frameBufferVector;
 
@@ -322,38 +421,35 @@ private:
     KMachineState m_machineState;
     QCursor m_cursor;
 
-    /** @name Runtime menus configuration variables.
+#ifndef Q_WS_MAC
+    /** @name Branding variables.
      ** @{ */
-#ifdef Q_WS_MAC
-    /** Determines Application menu allowed actions. */
-    RuntimeMenuApplicationActionType m_allowedActionsMenuApplication;
-#endif /* Q_WS_MAC */
-    /** Determines Machine menu allowed actions. */
-    RuntimeMenuMachineActionType m_allowedActionsMenuMachine;
-    /** Determines View menu allowed actions. */
-    RuntimeMenuViewActionType m_allowedActionsMenuView;
-    /** Determines Devices menu allowed actions. */
-    RuntimeMenuDevicesActionType m_allowedActionsMenuDevices;
-#ifdef VBOX_WITH_DEBUGGER_GUI
-    /** Determines Debugger menu allowed actions. */
-    RuntimeMenuDebuggerActionType m_allowedActionsMenuDebugger;
-#endif /* VBOX_WITH_DEBUGGER_GUI */
-    /** Determines Help menu allowed actions. */
-    RuntimeMenuHelpActionType m_allowedActionsMenuHelp;
+    /** Holds redefined machine-window icon. */
+    QIcon *m_pMachineWindowIcon;
+    /** Holds redefined machine-window name postfix. */
+    QString m_strMachineWindowNamePostfix;
+    /** @} */
+#endif /* !Q_WS_MAC */
+
+    /** @name Visual-state configuration variables.
+     ** @{ */
+    /** Determines which visual-state should be entered when possible. */
+    UIVisualStateType m_requestedVisualStateType;
     /** @} */
 
 #if defined(Q_WS_WIN)
     HCURSOR m_alphaCursor;
 #endif
-#ifdef Q_WS_MAC
-    /** @name MacOS X: Display reconfiguration variables.
+
+    /** @name Host-screen configuration variables.
      * @{ */
-    /** MacOS X: Watchdog timer looking for display reconfiguration. */
+    /** Holds the list of host-screen geometries we currently have. */
+    QList<QRect> m_hostScreens;
+#ifdef Q_WS_MAC
+    /** Mac OS X: Watchdog timer looking for display reconfiguration. */
     QTimer *m_pWatchdogDisplayChange;
-    /** MacOS X: A list of display geometries we currently have. */
-    QList<QRect> m_screens;
-    /** @} */
 #endif /* Q_WS_MAC */
+    /** @} */
 
     /** @name Application Close configuration variables.
      * @{ */
@@ -366,12 +462,10 @@ private:
     /** @} */
 
     /* Common flags: */
+    bool m_fInitialized : 1;
     bool m_fIsFirstTimeStarted : 1;
-    bool m_fIsIgnoreRuntimeMediumsChanging : 1;
     bool m_fIsGuestResizeIgnored : 1;
-    bool m_fIsSeamlessModeRequested : 1;
     bool m_fIsAutoCaptureDisabled : 1;
-    bool m_fReconfigurable : 1;
 
     /* Guest additions flags: */
     ULONG m_ulGuestAdditionsRunLevel;
@@ -379,18 +473,20 @@ private:
     bool  m_fIsGuestSupportsSeamless : 1;
 
     /* Keyboard flags: */
+    /** Holds the keyboard-state. */
+    int m_iKeyboardState;
     bool m_fNumLock : 1;
     bool m_fCapsLock : 1;
     bool m_fScrollLock : 1;
-    bool m_fHostNumLock : 1;
-    bool m_fHostCapsLock : 1;
-    bool m_fHostScrollLock : 1;
     uint m_uNumLockAdaptionCnt;
     uint m_uCapsLockAdaptionCnt;
 
     /* Mouse flags: */
+    /** Holds the mouse-state. */
+    int m_iMouseState;
     bool m_fIsMouseSupportsAbsolute : 1;
     bool m_fIsMouseSupportsRelative : 1;
+    bool m_fIsMouseSupportsMultiTouch: 1;
     bool m_fIsMouseHostCursorNeeded : 1;
     bool m_fIsMouseCaptured : 1;
     bool m_fIsMouseIntegrated : 1;
@@ -401,4 +497,5 @@ private:
     friend class UIConsoleEventHandler;
 };
 
-#endif // !___UIConsole_h___
+#endif /* !___UISession_h___ */
+
