@@ -1,12 +1,10 @@
 /* $Id: VBoxSnapshotsWgt.cpp $ */
 /** @file
- *
- * VBox frontends: Qt4 GUI ("VirtualBox"):
- * VBoxSnapshotsWgt class implementation
+ * VBox Qt GUI - VBoxSnapshotsWgt class implementation.
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,34 +16,37 @@
  */
 
 #ifdef VBOX_WITH_PRECOMPILED_HEADERS
-# include "precomp.h"
+# include <precomp.h>
 #else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
 /* Qt includes: */
-#include <QDateTime>
-#include <QHeaderView>
-#include <QMenu>
-#include <QScrollBar>
-#include <QWindowsStyle>
-#include <QPointer>
+# include <QDateTime>
+# include <QHeaderView>
+# include <QMenu>
+# include <QScrollBar>
+# include <QWindowsStyle>
+# include <QPointer>
+# include <QApplication>
 
 /* GUI includes: */
-#include "UIIconPool.h"
-#include "UIMessageCenter.h"
-#include "VBoxSnapshotDetailsDlg.h"
-#include "VBoxSnapshotsWgt.h"
-#include "VBoxTakeSnapshotDlg.h"
-#include "UIWizardCloneVM.h"
-#include "UIToolBar.h"
-#include "UIVirtualBoxEventHandler.h"
-#include "UISelectorShortcuts.h"
-#include "UIConverter.h"
+# include "UIIconPool.h"
+# include "UIMessageCenter.h"
+# include "VBoxSnapshotDetailsDlg.h"
+# include "VBoxSnapshotsWgt.h"
+# include "VBoxTakeSnapshotDlg.h"
+# include "UIWizardCloneVM.h"
+# include "UIToolBar.h"
+# include "UIVirtualBoxEventHandler.h"
+# include "UIConverter.h"
+# include "UIModalWindowManager.h"
+# include "UIExtraDataManager.h"
 
 /* COM includes: */
-#include "CConsole.h"
+# include "CConsole.h"
 
 #endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
+
 /**
  *  QTreeWidgetItem subclass for snapshots items
  */
@@ -56,24 +57,27 @@ public:
     enum { ItemType = QTreeWidgetItem::UserType + 1 };
 
     /* Normal snapshot item (child of tree-widget) */
-    SnapshotWgtItem (QTreeWidget *aTreeWidget, const CSnapshot &aSnapshot)
+    SnapshotWgtItem (VBoxSnapshotsWgt *pSnapshotWidget, QTreeWidget *aTreeWidget, const CSnapshot &aSnapshot)
         : QTreeWidgetItem (aTreeWidget, ItemType)
+        , m_pSnapshotWidget(pSnapshotWidget)
         , mIsCurrentState (false)
         , mSnapshot (aSnapshot)
     {
     }
 
     /* Normal snapshot item (child of tree-widget-item) */
-    SnapshotWgtItem (QTreeWidgetItem *aRootItem, const CSnapshot &aSnapshot)
+    SnapshotWgtItem (VBoxSnapshotsWgt *pSnapshotWidget, QTreeWidgetItem *aRootItem, const CSnapshot &aSnapshot)
         : QTreeWidgetItem (aRootItem, ItemType)
+        , m_pSnapshotWidget(pSnapshotWidget)
         , mIsCurrentState (false)
         , mSnapshot (aSnapshot)
     {
     }
 
     /* Current state item (child of tree-widget) */
-    SnapshotWgtItem (QTreeWidget *aTreeWidget, const CMachine &aMachine)
+    SnapshotWgtItem (VBoxSnapshotsWgt *pSnapshotWidget, QTreeWidget *aTreeWidget, const CMachine &aMachine)
         : QTreeWidgetItem (aTreeWidget, ItemType)
+        , m_pSnapshotWidget(pSnapshotWidget)
         , mIsCurrentState (true)
         , mMachine (aMachine)
     {
@@ -81,8 +85,9 @@ public:
     }
 
     /* Current state item (child of tree-widget-item) */
-    SnapshotWgtItem (QTreeWidgetItem *aRootItem, const CMachine &aMachine)
+    SnapshotWgtItem (VBoxSnapshotsWgt *pSnapshotWidget, QTreeWidgetItem *aRootItem, const CMachine &aMachine)
         : QTreeWidgetItem (aRootItem, ItemType)
+        , m_pSnapshotWidget(pSnapshotWidget)
         , mIsCurrentState (true)
         , mMachine (aMachine)
     {
@@ -97,6 +102,19 @@ public:
                 return mIsCurrentState ? QTreeWidgetItem::data (aColumn, aRole) : QVariant (QString ("%1%2")
                                          .arg (QTreeWidgetItem::data (aColumn, Qt::DisplayRole).toString())
                                          .arg (QTreeWidgetItem::data (aColumn, Qt::UserRole).toString()));
+            case Qt::SizeHintRole:
+            {
+                /* Determine the icon metric: */
+                const QStyle *pStyle = QApplication::style();
+                const int iIconMetric = pStyle->pixelMetric(QStyle::PM_SmallIconSize);
+                /* Determine the minimum size-hint for this tree-widget-item: */
+                const QSize baseSizeHint = QTreeWidgetItem::data(aColumn, aRole).toSize();
+                /* Determine the effective height-hint for this tree-widget-item: */
+                const int iEffectiveHeightHint = qMax(baseSizeHint.height(),
+                                                      iIconMetric + 2 * 2 /* margins */);
+                /* Return size-hint for this tree-widget-item: */
+                return QSize(baseSizeHint.width(), iEffectiveHeightHint);
+            }
             default:
                 break;
         }
@@ -168,7 +186,7 @@ public:
             mId = mSnapshot.GetId();
             setText (0, mSnapshot.GetName());
             mOnline = mSnapshot.GetOnline();
-            setIcon (0, vboxGlobal().snapshotIcon (mOnline));
+            setIcon(0, m_pSnapshotWidget->snapshotItemIcon(mOnline));
             mDesc = mSnapshot.GetDescription();
             mTimestamp.setTime_t (mSnapshot.GetTimeStamp() / 1000);
             mCurStateModified = false;
@@ -190,7 +208,7 @@ public:
         if (mMachine.isNull())
             return;
 
-        setIcon (0, gpConverter->toPixmap (aState));
+        setIcon(0, gpConverter->toIcon(aState));
         mMachineState = aState;
         mTimestamp.setTime_t (mMachine.GetLastStateChange() / 1000);
     }
@@ -201,29 +219,33 @@ public:
 
         /* Age: [date time|%1d ago|%1h ago|%1min ago|%1sec ago] */
         SnapshotAgeFormat ageFormat;
-        if (mTimestamp.daysTo (QDateTime::currentDateTime()) > 30)
+        QDateTime now = QDateTime::currentDateTime();
+        QDateTime then = mTimestamp;
+        if (then > now)
+            then = now; /* can happen if the host time is wrong */
+        if (then.daysTo (now) > 30)
         {
-            age = VBoxSnapshotsWgt::tr (" (%1)").arg (mTimestamp.toString (Qt::LocalDate));
+            age = VBoxSnapshotsWgt::tr (" (%1)").arg (then.toString (Qt::LocalDate));
             ageFormat = AgeMax;
         }
-        else if (mTimestamp.secsTo (QDateTime::currentDateTime()) > 60 * 60 * 24)
+        else if (then.secsTo (now) > 60 * 60 * 24)
         {
-            age = VBoxSnapshotsWgt::tr (" (%1 ago)").arg(VBoxGlobal::daysToString(mTimestamp.secsTo (QDateTime::currentDateTime()) / 60 / 60 / 24));
+            age = VBoxSnapshotsWgt::tr (" (%1 ago)").arg(VBoxGlobal::daysToString(then.secsTo (now) / 60 / 60 / 24));
             ageFormat = AgeInDays;
         }
-        else if (mTimestamp.secsTo (QDateTime::currentDateTime()) > 60 * 60)
+        else if (then.secsTo (now) > 60 * 60)
         {
-            age = VBoxSnapshotsWgt::tr (" (%1 ago)").arg(VBoxGlobal::hoursToString(mTimestamp.secsTo (QDateTime::currentDateTime()) / 60 / 60));
+            age = VBoxSnapshotsWgt::tr (" (%1 ago)").arg(VBoxGlobal::hoursToString(then.secsTo (now) / 60 / 60));
             ageFormat = AgeInHours;
         }
-        else if (mTimestamp.secsTo (QDateTime::currentDateTime()) > 60)
+        else if (then.secsTo (now) > 60)
         {
-            age = VBoxSnapshotsWgt::tr (" (%1 ago)").arg(VBoxGlobal::minutesToString(mTimestamp.secsTo (QDateTime::currentDateTime()) / 60));
+            age = VBoxSnapshotsWgt::tr (" (%1 ago)").arg(VBoxGlobal::minutesToString(then.secsTo (now) / 60));
             ageFormat = AgeInMinutes;
         }
         else
         {
-            age = VBoxSnapshotsWgt::tr (" (%1 ago)").arg(VBoxGlobal::secondsToString(mTimestamp.secsTo (QDateTime::currentDateTime())));
+            age = VBoxSnapshotsWgt::tr (" (%1 ago)").arg(VBoxGlobal::secondsToString(then.secsTo (now)));
             ageFormat = AgeInSeconds;
         }
 
@@ -289,6 +311,9 @@ private:
         setToolTip (0, toolTip);
     }
 
+    /** Holds pointer to snapshot-widget 'this' item belongs to. */
+    QPointer<VBoxSnapshotsWgt> m_pSnapshotWidget;
+
     bool mIsCurrentState;
 
     CSnapshot mSnapshot;
@@ -359,42 +384,45 @@ VBoxSnapshotsWgt::VBoxSnapshotsWgt (QWidget *aParent)
     connect (mTreeWidget, SIGNAL (destroyed (QObject *)), treeWidgetStyle, SLOT (deleteLater()));
 // #endif
 
+    /* Cache pixmaps: */
+    m_offlineSnapshotIcon = UIIconPool::iconSet(":/snapshot_offline_16px.png");
+    m_onlineSnapshotIcon = UIIconPool::iconSet(":/snapshot_online_16px.png");
+
+    /* Determine icon metric: */
+    const QStyle *pStyle = QApplication::style();
+    const int iIconMetric = (int)(pStyle->pixelMetric(QStyle::PM_SmallIconSize) * 1.375);
+
     /* ToolBar creation */
     UIToolBar *toolBar = new UIToolBar (this);
-    toolBar->setUsesTextLabel (false);
-    toolBar->setIconSize (QSize (22, 22));
+    toolBar->setIconSize (QSize (iIconMetric, iIconMetric));
     toolBar->setSizePolicy (QSizePolicy::Fixed, QSizePolicy::Fixed);
 
     toolBar->addAction (mTakeSnapshotAction);
     toolBar->addSeparator();
     toolBar->addAction (mRestoreSnapshotAction);
     toolBar->addAction (mDeleteSnapshotAction);
+    toolBar->addAction (mShowSnapshotDetailsAction);
     toolBar->addSeparator();
     toolBar->addAction(mCloneSnapshotAction);
-    toolBar->addSeparator();
-    toolBar->addAction (mShowSnapshotDetailsAction);
 
     ((QVBoxLayout*)layout())->insertWidget (0, toolBar);
 
     /* Setup actions */
     mRestoreSnapshotAction->setIcon(UIIconPool::iconSetFull(
-        QSize (22, 22), QSize (16, 16),
-        ":/discard_cur_state_22px.png", ":/discard_cur_state_16px.png", // TODO: Update Icons!
-        ":/discard_cur_state_dis_22px.png", ":/discard_cur_state_dis_16px.png")); // TODO: Update Icons!
+        ":/snapshot_restore_22px.png", ":/snapshot_restore_16px.png",
+        ":/snapshot_restore_disabled_22px.png", ":/snapshot_restore_disabled_16px.png"));
     mDeleteSnapshotAction->setIcon(UIIconPool::iconSetFull(
-        QSize (22, 22), QSize (16, 16),
-        ":/delete_snapshot_22px.png", ":/delete_snapshot_16px.png",
-        ":/delete_snapshot_dis_22px.png", ":/delete_snapshot_dis_16px.png"));
+        ":/snapshot_delete_22px.png", ":/snapshot_delete_16px.png",
+        ":/snapshot_delete_disabled_22px.png", ":/snapshot_delete_disabled_16px.png"));
     mShowSnapshotDetailsAction->setIcon(UIIconPool::iconSetFull(
-        QSize (22, 22), QSize (16, 16),
-        ":/show_snapshot_details_22px.png", ":/show_snapshot_details_16px.png",
-        ":/show_snapshot_details_dis_22px.png", ":/show_snapshot_details_dis_16px.png"));
+        ":/snapshot_show_details_22px.png", ":/snapshot_show_details_16px.png",
+        ":/snapshot_show_details_disabled_22px.png", ":/snapshot_details_show_disabled_16px.png"));
     mTakeSnapshotAction->setIcon(UIIconPool::iconSetFull(
-        QSize (22, 22), QSize (16, 16),
-        ":/take_snapshot_22px.png", ":/take_snapshot_16px.png",
-        ":/take_snapshot_dis_22px.png", ":/take_snapshot_dis_16px.png"));
-    mCloneSnapshotAction->setIcon(UIIconPool::iconSet(
-        ":/vm_clone_16px.png", ":/vm_clone_disabled_16px.png"));
+        ":/snapshot_take_22px.png", ":/snapshot_take_16px.png",
+        ":/snapshot_take_disabled_22px.png", ":/snapshot_take_disabled_16px.png"));
+    mCloneSnapshotAction->setIcon(UIIconPool::iconSetFull(
+        ":/vm_clone_22px.png", ":/vm_clone_16px.png",
+        ":/vm_clone_disabled_22px.png", ":/vm_clone_disabled_16px.png"));
 
     mRestoreSnapshotAction->setShortcut (QString ("Ctrl+Shift+R"));
     mDeleteSnapshotAction->setShortcut (QString ("Ctrl+Shift+D"));
@@ -411,6 +439,8 @@ VBoxSnapshotsWgt::VBoxSnapshotsWgt (QWidget *aParent)
              this, SLOT (onContextMenuRequested (const QPoint&)));
     connect (mTreeWidget, SIGNAL (itemChanged (QTreeWidgetItem*, int)),
              this, SLOT (onItemChanged (QTreeWidgetItem*)));
+    connect(mTreeWidget, SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)),
+            this, SLOT (sltItemDoubleClicked(QTreeWidgetItem*)));
 
     connect (mTakeSnapshotAction, SIGNAL (triggered()), this, SLOT (sltTakeSnapshot()));
     connect (mRestoreSnapshotAction, SIGNAL (triggered()), this, SLOT (sltRestoreSnapshot()));
@@ -444,7 +474,7 @@ void VBoxSnapshotsWgt::setMachine (const CMachine &aMachine)
     {
         mMachineId = aMachine.GetId();
         mSessionState = aMachine.GetSessionState();
-        m_fShapshotOperationsAllowed = vboxGlobal().shouldWeAllowSnapshotOperations(mMachine);
+        m_fShapshotOperationsAllowed = gEDataManager->machineSnapshotOperationsEnabled(mMachineId);
     }
 
     refreshAll();
@@ -452,31 +482,30 @@ void VBoxSnapshotsWgt::setMachine (const CMachine &aMachine)
 
 void VBoxSnapshotsWgt::retranslateUi()
 {
-    /* Translate uic generated strings */
-    Ui::VBoxSnapshotsWgt::retranslateUi (this);
+    /* Translate uic generated strings: */
+    Ui::VBoxSnapshotsWgt::retranslateUi(this);
 
-    mRestoreSnapshotAction->setText (tr ("&Restore Snapshot"));
-    mDeleteSnapshotAction->setText (tr ("&Delete Snapshot"));
-    mShowSnapshotDetailsAction->setText (tr ("S&how Details"));
-    mTakeSnapshotAction->setText (tr ("Take &Snapshot"));
+    mRestoreSnapshotAction->setText(tr("&Restore Snapshot"));
+    mDeleteSnapshotAction->setText(tr("&Delete Snapshot"));
+    mShowSnapshotDetailsAction->setText(tr("S&how Details"));
+    mTakeSnapshotAction->setText(tr("Take &Snapshot"));
     mCloneSnapshotAction->setText(tr("&Clone..."));
 
+    mRestoreSnapshotAction->setStatusTip(tr("Restore selected snapshot of the virtual machine"));
+    mDeleteSnapshotAction->setStatusTip(tr("Delete selected snapshot of the virtual machine"));
+    mShowSnapshotDetailsAction->setStatusTip(tr("Display a window with selected snapshot details"));
+    mTakeSnapshotAction->setStatusTip(tr("Take a snapshot of the current virtual machine state"));
+    mCloneSnapshotAction->setStatusTip(tr("Clone selected virtual machine"));
 
-    mRestoreSnapshotAction->setStatusTip (tr ("Restore the selected snapshot of the virtual machine"));
-    mDeleteSnapshotAction->setStatusTip (tr ("Delete the selected snapshot of the virtual machine"));
-    mShowSnapshotDetailsAction->setStatusTip (tr ("Show the details of the selected snapshot"));
-    mTakeSnapshotAction->setStatusTip (tr ("Take a snapshot of the current virtual machine state"));
-    mCloneSnapshotAction->setStatusTip(tr("Clone the selected virtual machine"));
-
-    mRestoreSnapshotAction->setToolTip (mRestoreSnapshotAction->text().remove ('&').remove ('.') +
-        QString (" (%1)").arg (mRestoreSnapshotAction->shortcut().toString()));
-    mDeleteSnapshotAction->setToolTip (mDeleteSnapshotAction->text().remove ('&').remove ('.') +
-        QString (" (%1)").arg (mDeleteSnapshotAction->shortcut().toString()));
-    mShowSnapshotDetailsAction->setToolTip (mShowSnapshotDetailsAction->text().remove ('&').remove ('.') +
-        QString (" (%1)").arg (mShowSnapshotDetailsAction->shortcut().toString()));
-    mTakeSnapshotAction->setToolTip (mTakeSnapshotAction->text().remove ('&').remove ('.') +
-        QString (" (%1)").arg (mTakeSnapshotAction->shortcut().toString()));
-    mCloneSnapshotAction->setToolTip(mCloneSnapshotAction->text().remove('&').remove('.') +
+    mRestoreSnapshotAction->setToolTip(mRestoreSnapshotAction->statusTip() +
+        QString(" (%1)").arg(mRestoreSnapshotAction->shortcut().toString()));
+    mDeleteSnapshotAction->setToolTip(mDeleteSnapshotAction->statusTip() +
+        QString(" (%1)").arg(mDeleteSnapshotAction->shortcut().toString()));
+    mShowSnapshotDetailsAction->setToolTip(mShowSnapshotDetailsAction->statusTip() +
+        QString(" (%1)").arg(mShowSnapshotDetailsAction->shortcut().toString()));
+    mTakeSnapshotAction->setToolTip(mTakeSnapshotAction->statusTip() +
+        QString(" (%1)").arg(mTakeSnapshotAction->shortcut().toString()));
+    mCloneSnapshotAction->setToolTip(mCloneSnapshotAction->statusTip() +
         QString(" (%1)").arg(mCloneSnapshotAction->shortcut().toString()));
 }
 
@@ -539,10 +568,9 @@ void VBoxSnapshotsWgt::onContextMenuRequested (const QPoint &aPoint)
     {
         menu.addAction (mRestoreSnapshotAction);
         menu.addAction (mDeleteSnapshotAction);
+        menu.addAction (mShowSnapshotDetailsAction);
         menu.addSeparator();
         menu.addAction(mCloneSnapshotAction);
-        menu.addSeparator();
-        menu.addAction (mShowSnapshotDetailsAction);
     }
     else
     {
@@ -569,12 +597,31 @@ void VBoxSnapshotsWgt::onItemChanged (QTreeWidgetItem *aItem)
     }
 }
 
+void VBoxSnapshotsWgt::sltItemDoubleClicked(QTreeWidgetItem *pItem)
+{
+    /* Make sure *nothing* is being edited currently: */
+    if (mEditProtector)
+        return;
+
+    /* Make sure some *valid* item was *really* double-clicked: */
+    SnapshotWgtItem *pValidItem = pItem ? static_cast<SnapshotWgtItem*>(pItem) : 0;
+    if (!pValidItem)
+        return;
+
+    /* Handle Ctrl+DoubleClick: */
+    if (QApplication::keyboardModifiers() == Qt::ControlModifier)
+    {
+        /* As call for snapshot-restore procedure: */
+        sltRestoreSnapshot(true /* suppress non-critical warnings */);
+    }
+}
+
 void VBoxSnapshotsWgt::sltTakeSnapshot()
 {
     takeSnapshot();
 }
 
-void VBoxSnapshotsWgt::sltRestoreSnapshot()
+void VBoxSnapshotsWgt::sltRestoreSnapshot(bool fSuppressNonCriticalWarnings /* = false*/)
 {
     /* Get currently chosen item: */
     SnapshotWgtItem *pItem = mTreeWidget->currentItem() ? static_cast<SnapshotWgtItem*>(mTreeWidget->currentItem()) : 0;
@@ -586,12 +633,16 @@ void VBoxSnapshotsWgt::sltRestoreSnapshot()
     CSnapshot snapshot = mMachine.FindSnapshot(strSnapshotId);
 
     /* Ask the user if he really wants to restore the snapshot: */
-    int iResultCode = msgCenter().askAboutSnapshotRestoring(snapshot.GetName(), mMachine.GetCurrentStateModified());
-    if (iResultCode & QIMessageBox::Cancel)
-        return;
+    int iResultCode = AlertButton_Ok;
+    if (!fSuppressNonCriticalWarnings || mMachine.GetCurrentStateModified())
+    {
+        iResultCode = msgCenter().confirmSnapshotRestoring(snapshot.GetName(), mMachine.GetCurrentStateModified());
+        if (iResultCode & AlertButton_Cancel)
+            return;
+    }
 
     /* If user also confirmed new snapshot creation: */
-    if (iResultCode & QIMessageBox::OptionChosen)
+    if (iResultCode & AlertOption_CheckBox)
     {
         /* Take snapshot of changed current state: */
         mTreeWidget->setCurrentItem(curStateItem());
@@ -605,17 +656,16 @@ void VBoxSnapshotsWgt::sltRestoreSnapshot()
         return;
 
     /* Restore chosen snapshot: */
-    CConsole console = session.GetConsole();
-    CProgress progress = console.RestoreSnapshot(snapshot);
-    if (console.isOk())
+    CMachine machine = session.GetMachine();
+    CProgress progress = machine.RestoreSnapshot(snapshot);
+    if (machine.isOk())
     {
-        msgCenter().showModalProgressDialog(progress, mMachine.GetName(), ":/progress_snapshot_restore_90px.png",
-                                              msgCenter().mainWindowShown(), true);
+        msgCenter().showModalProgressDialog(progress, mMachine.GetName(), ":/progress_snapshot_restore_90px.png");
         if (progress.GetResultCode() != 0)
-            msgCenter().cannotRestoreSnapshot(progress, snapshot.GetName());
+            msgCenter().cannotRestoreSnapshot(progress, snapshot.GetName(), mMachine.GetName());
     }
     else
-        msgCenter().cannotRestoreSnapshot(progress, snapshot.GetName());
+        msgCenter().cannotRestoreSnapshot(machine, snapshot.GetName(), mMachine.GetName());
 
     /* Unlock machine finally: */
     session.UnlockMachine();
@@ -631,19 +681,18 @@ void VBoxSnapshotsWgt::sltDeleteSnapshot()
     AssertReturn (!snapId.isNull(), (void) 0);
     CSnapshot snapshot = mMachine.FindSnapshot(snapId);
 
-    if (!msgCenter().askAboutSnapshotDeleting (snapshot.GetName()))
+    if (!msgCenter().confirmSnapshotRemoval(snapshot.GetName()))
         return;
 
     /** @todo check available space on the target filesystem etc etc. */
 #if 0
-    if (!msgCenter().askAboutSnapshotDeletingFreeSpace (snapshot.GetName(),
-                                                          "/home/juser/.VirtualBox/Machines/SampleVM/Snapshots/{01020304-0102-0102-0102-010203040506}.vdi",
-                                                          "59 GiB",
-                                                          "15 GiB"))
+    if (!msgCenter().warnAboutSnapshotRemovalFreeSpace(snapshot.GetName(),
+                                                       "/home/juser/.VirtualBox/Machines/SampleVM/Snapshots/{01020304-0102-0102-0102-010203040506}.vdi",
+                                                       "59 GiB",
+                                                       "15 GiB"))
         return;
 #endif
 
-
     /* Open a direct session (this call will handle all errors) */
     bool busy = mSessionState != KSessionState_Unlocked;
     CSession session;
@@ -654,19 +703,19 @@ void VBoxSnapshotsWgt::sltDeleteSnapshot()
     if (session.isNull())
         return;
 
-    CConsole console = session.GetConsole();
-    CProgress progress = console.DeleteSnapshot (snapId);
-    if (console.isOk())
+    /* Remove chosen snapshot: */
+    CMachine machine = session.GetMachine();
+    CProgress progress = machine.DeleteSnapshot(snapId);
+    if (machine.isOk())
     {
         /* Show the progress dialog */
-        msgCenter().showModalProgressDialog (progress, mMachine.GetName(), ":/progress_snapshot_discard_90px.png",
-                                               msgCenter().mainWindowShown(), true);
+        msgCenter().showModalProgressDialog(progress, mMachine.GetName(), ":/progress_snapshot_discard_90px.png");
 
         if (progress.GetResultCode() != 0)
-            msgCenter().cannotDeleteSnapshot (progress,  snapshot.GetName());
+            msgCenter().cannotRemoveSnapshot(progress,  snapshot.GetName(), mMachine.GetName());
     }
     else
-        msgCenter().cannotDeleteSnapshot (console,  snapshot.GetName());
+        msgCenter().cannotRemoveSnapshot(machine,  snapshot.GetName(), mMachine.GetName());
 
     session.UnlockMachine();
 }
@@ -796,102 +845,72 @@ bool VBoxSnapshotsWgt::takeSnapshot()
 
     if (fIsValid)
     {
-        /* Get corresponding console object also: */
-        CConsole console = session.GetConsole();
-        /* Remember runtime state: */
-        bool fAtRuntime = mMachine.GetState() == KMachineState_Running;
-        /* Remember paused state: */
-        bool fWasPaused = mMachine.GetState() == KMachineState_Paused ||
-                          mMachine.GetState() == KMachineState_TeleportingPausedVM;
+        /* Get corresponding machine object also: */
+        CMachine machine = session.GetMachine();
 
-        /* Pause VM if necessary: */
-        if (fIsValid && fAtRuntime && !fWasPaused)
+        /* Create take-snapshot dialog: */
+        QWidget *pDlgParent = windowManager().realParentWindow(this);
+        QPointer<VBoxTakeSnapshotDlg> pDlg = new VBoxTakeSnapshotDlg(pDlgParent, mMachine);
+        windowManager().registerNewParent(pDlg, pDlgParent);
+
+        /* Assign corresponding icon: */
+        pDlg->mLbIcon->setPixmap(vboxGlobal().vmGuestOSTypeIcon(mMachine.GetOSTypeId()));
+
+        /* Search for the max available snapshot index: */
+        int iMaxSnapShotIndex = 0;
+        QString snapShotName = tr("Snapshot %1");
+        QRegExp regExp(QString("^") + snapShotName.arg("([0-9]+)") + QString("$"));
+        QTreeWidgetItemIterator iterator(mTreeWidget);
+        while (*iterator)
         {
-            /* Pausing VM: */
-            console.Pause();
-            if (!console.isOk())
-            {
-                msgCenter().cannotPauseMachine(console);
-                fIsValid = false;
-            }
+            QString snapShot = static_cast<SnapshotWgtItem*>(*iterator)->text(0);
+            int pos = regExp.indexIn(snapShot);
+            if (pos != -1)
+                iMaxSnapShotIndex = regExp.cap(1).toInt() > iMaxSnapShotIndex ? regExp.cap(1).toInt() : iMaxSnapShotIndex;
+            ++iterator;
         }
+        pDlg->mLeName->setText(snapShotName.arg(iMaxSnapShotIndex + 1));
 
-        if (fIsValid)
+        /* Exec the dialog: */
+        bool fDialogAccepted = pDlg->exec() == QDialog::Accepted;
+
+        /* Is the dialog still valid? */
+        if (pDlg)
         {
-            /* Create take-snapshot dialog: */
-            QPointer<VBoxTakeSnapshotDlg> pDlg = new VBoxTakeSnapshotDlg(this, mMachine);
+            /* Acquire variables: */
+            QString strSnapshotName = pDlg->mLeName->text().trimmed();
+            QString strSnapshotDescription = pDlg->mTeDescription->toPlainText();
 
-            /* Assign corresponding icon: */
-            pDlg->mLbIcon->setPixmap(vboxGlobal().vmGuestOSTypeIcon(mMachine.GetOSTypeId()));
+            /* Destroy dialog early: */
+            delete pDlg;
 
-            /* Search for the max available snapshot index: */
-            int iMaxSnapShotIndex = 0;
-            QString snapShotName = tr("Snapshot %1");
-            QRegExp regExp(QString("^") + snapShotName.arg("([0-9]+)") + QString("$"));
-            QTreeWidgetItemIterator iterator(mTreeWidget);
-            while (*iterator)
+            /* Was the dialog accepted? */
+            if (fDialogAccepted)
             {
-                QString snapShot = static_cast<SnapshotWgtItem*>(*iterator)->text(0);
-                int pos = regExp.indexIn(snapShot);
-                if (pos != -1)
-                    iMaxSnapShotIndex = regExp.cap(1).toInt() > iMaxSnapShotIndex ? regExp.cap(1).toInt() : iMaxSnapShotIndex;
-                ++iterator;
-            }
-            pDlg->mLeName->setText(snapShotName.arg(iMaxSnapShotIndex + 1));
-
-            /* Exec the dialog: */
-            bool fDialogAccepted = pDlg->exec() == QDialog::Accepted;
-
-            /* Is the dialog still valid? */
-            if (pDlg)
-            {
-                /* Acquire variables: */
-                QString strSnapshotName = pDlg->mLeName->text().trimmed();
-                QString strSnapshotDescription = pDlg->mTeDescription->toPlainText();
-
-                /* Destroy dialog early: */
-                delete pDlg;
-
-                /* Was the dialog accepted? */
-                if (fDialogAccepted)
+                QString strSnapshotId;
+                /* Prepare the take-snapshot progress: */
+                CProgress progress = machine.TakeSnapshot(strSnapshotName, strSnapshotDescription, true, strSnapshotId);
+                if (machine.isOk())
                 {
-                    /* Prepare the take-snapshot progress: */
-                    CProgress progress = console.TakeSnapshot(strSnapshotName, strSnapshotDescription);
-                    if (console.isOk())
+                    /* Show the take-snapshot progress: */
+                    msgCenter().showModalProgressDialog(progress, mMachine.GetName(), ":/progress_snapshot_create_90px.png");
+                    if (!progress.isOk() || progress.GetResultCode() != 0)
                     {
-                        /* Show the take-snapshot progress: */
-                        msgCenter().showModalProgressDialog(progress, mMachine.GetName(), ":/progress_snapshot_create_90px.png",
-                                                            msgCenter().mainWindowShown(), true);
-                        if (progress.GetResultCode() != 0)
-                        {
-                            msgCenter().cannotTakeSnapshot(progress);
-                            fIsValid = false;
-                        }
-                    }
-                    else
-                    {
-                        msgCenter().cannotTakeSnapshot(console);
+                        msgCenter().cannotTakeSnapshot(progress, mMachine.GetName());
                         fIsValid = false;
                     }
                 }
                 else
+                {
+                    msgCenter().cannotTakeSnapshot(machine, mMachine.GetName());
                     fIsValid = false;
+                }
             }
             else
                 fIsValid = false;
         }
-
-        /* Resume VM if necessary: */
-        if (fIsValid && fAtRuntime && !fWasPaused)
-        {
-            /* Resuming VM: */
-            console.Resume();
-            if (!console.isOk())
-            {
-                msgCenter().cannotResumeMachine(console);
-                fIsValid = false;
-            }
-        }
+        else
+            fIsValid = false;
 
         /* Unlock machine finally: */
         session.UnlockMachine();
@@ -932,7 +951,7 @@ void VBoxSnapshotsWgt::refreshAll()
         Assert (mCurSnapshotItem);
 
         /* Add the "current state" item */
-        SnapshotWgtItem *csi = new SnapshotWgtItem (mCurSnapshotItem, mMachine);
+        SnapshotWgtItem *csi = new SnapshotWgtItem(this, mCurSnapshotItem, mMachine);
         csi->setBold (true);
         csi->recache();
 
@@ -951,7 +970,7 @@ void VBoxSnapshotsWgt::refreshAll()
         mCurSnapshotItem = 0;
 
         /* Add the "current state" item */
-        SnapshotWgtItem *csi = new SnapshotWgtItem (mTreeWidget, mMachine);
+        SnapshotWgtItem *csi = new SnapshotWgtItem(this, mTreeWidget, mMachine);
         csi->setBold (true);
         csi->recache();
 
@@ -989,8 +1008,8 @@ SnapshotWgtItem *VBoxSnapshotsWgt::curStateItem()
 
 void VBoxSnapshotsWgt::populateSnapshots (const CSnapshot &aSnapshot, QTreeWidgetItem *aItem)
 {
-    SnapshotWgtItem *item = aItem ? new SnapshotWgtItem (aItem, aSnapshot) :
-                                    new SnapshotWgtItem (mTreeWidget, aSnapshot);
+    SnapshotWgtItem *item = aItem ? new SnapshotWgtItem(this, aItem, aSnapshot) :
+                                    new SnapshotWgtItem(this, mTreeWidget, aSnapshot);
     item->recache();
 
     CSnapshot curSnapshot = mMachine.GetCurrentSnapshot();
