@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -146,7 +146,7 @@ static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClie
         {
             uint32_t len;
 
-            len = strlen(pFolderMapping->pszFolderName);
+            len = (uint32_t)strlen(pFolderMapping->pszFolderName);
             rc = SSMR3PutU32(pSSM, len);
             AssertRCReturn(rc, rc);
 
@@ -272,6 +272,11 @@ static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClie
 
             /* 'i' is the root handle of the saved mapping. */
             rc = vbsfMappingLoaded (&mapping, i);
+            if (RT_FAILURE(rc))
+            {
+                LogRel(("SharedFolders host service: %Rrc loading %d [%ls] -> [%s]\n",
+                        rc, i, pMapName->String.ucs2, pszFolderName));
+            }
 
             RTMemFree(pMapName);
             RTStrFree(pszFolderName);
@@ -384,7 +389,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 SHFLSTRING *pString    = (SHFLSTRING *)paParms[1].u.pointer.addr;
 
                 /* Verify parameters values. */
-                if (!ShflStringIsValid(pString, paParms[1].u.pointer.size))
+                if (!ShflStringIsValidOut(pString, paParms[1].u.pointer.size))
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
@@ -430,7 +435,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint32_t cbParms        = paParms[2].u.pointer.size;
 
                 /* Verify parameters values. */
-                if (   !ShflStringIsValid(pPath, cbPath)
+                if (   !ShflStringIsValidIn(pPath, cbPath, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8))
                     || (cbParms != sizeof (SHFLCREATEPARMS))
                    )
                 {
@@ -441,7 +446,6 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 else
                 {
                     /* Execute the function. */
-
                     rc = vbsfCreate (pClient, root, pPath, cbPath, pParms);
 
                     if (RT_SUCCESS(rc))
@@ -489,7 +493,6 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 else
                 {
                     /* Execute the function. */
-
                     rc = vbsfClose (pClient, root, Handle);
 
                     if (RT_SUCCESS(rc))
@@ -761,7 +764,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 /* Verify parameters values. */
                 if (   (length < sizeof (SHFLDIRINFO))
                     ||  length > paParms[5].u.pointer.size
-                    ||  !ShflStringIsValidOrNull(pPath, paParms[4].u.pointer.size)
+                    ||  !ShflStringIsValidOrNullIn(pPath, paParms[4].u.pointer.size, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8))
                    )
                 {
                     rc = VERR_INVALID_PARAMETER;
@@ -829,7 +832,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint32_t  cbBuffer = paParms[2].u.pointer.size;
 
                 /* Verify parameters values. */
-                if (!ShflStringIsValidOrNull(pPath, paParms[1].u.pointer.size))
+                if (!ShflStringIsValidOrNullIn(pPath, paParms[1].u.pointer.size, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8)))
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
@@ -874,7 +877,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 RTUTF16     delimiter  = (RTUTF16)paParms[2].u.uint32;
 
                 /* Verify parameters values. */
-                if (!ShflStringIsValid(pszMapName, paParms[0].u.pointer.size))
+                if (!ShflStringIsValidIn(pszMapName, paParms[0].u.pointer.size, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8)))
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
@@ -925,21 +928,36 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 bool        fCaseSensitive = !!paParms[3].u.uint32;
 
                 /* Verify parameters values. */
-                if (!ShflStringIsValid(pszMapName, paParms[0].u.pointer.size))
+                if (ShflStringIsValidIn(pszMapName, paParms[0].u.pointer.size, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8)))
                 {
-                    rc = VERR_INVALID_PARAMETER;
+                    rc = VINF_SUCCESS;
                 }
                 else
                 {
+                    rc = VERR_INVALID_PARAMETER;
 
-                    /* Execute the function. */
+                    /* Fudge for windows GAs getting the length wrong by one char. */
+                    if (   !(pClient->fu32Flags & SHFL_CF_UTF8)
+                        && paParms[0].u.pointer.size >= sizeof(SHFLSTRING)
+                        && pszMapName->u16Length >= 2
+                        && pszMapName->String.ucs2[pszMapName->u16Length / 2 - 1] == 0x0000)
+                    {
+                        pszMapName->u16Length -= 2;
+                        if (ShflStringIsValidIn(pszMapName, paParms[0].u.pointer.size, false /*fUtf8Not16*/))
+                            rc = VINF_SUCCESS;
+                        else
+                            pszMapName->u16Length += 2;
+                    }
+                }
+
+                /* Execute the function. */
+                if (RT_SUCCESS(rc))
                     rc = vbsfMapFolder (pClient, pszMapName, delimiter, fCaseSensitive, &root);
 
-                    if (RT_SUCCESS(rc))
-                    {
-                        /* Update parameters.*/
-                        paParms[1].u.uint32 = root;
-                    }
+                if (RT_SUCCESS(rc))
+                {
+                    /* Update parameters.*/
+                    paParms[1].u.uint32 = root;
                 }
             }
             Log(("SharedFolders host service: map operation result %Rrc\n", rc));
@@ -1064,7 +1082,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint32_t flags          = paParms[2].u.uint32;
 
                 /* Verify parameters values. */
-                if (!ShflStringIsValid(pPath, cbPath))
+                if (!ShflStringIsValidIn(pPath, cbPath, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8)))
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
@@ -1108,8 +1126,8 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint32_t    flags       = paParms[3].u.uint32;
 
                 /* Verify parameters values. */
-                if (    !ShflStringIsValid(pSrc, paParms[1].u.pointer.size)
-                    ||  !ShflStringIsValid(pDest, paParms[2].u.pointer.size)
+                if (    !ShflStringIsValidIn(pSrc, paParms[1].u.pointer.size, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8))
+                    ||  !ShflStringIsValidIn(pDest, paParms[2].u.pointer.size, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8))
                    )
                 {
                     rc = VERR_INVALID_PARAMETER;
@@ -1208,8 +1226,8 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint32_t     cbInfo   = paParms[3].u.pointer.size;
 
                 /* Verify parameters values. */
-                if (    !ShflStringIsValid(pNewPath, paParms[1].u.pointer.size)
-                    ||  !ShflStringIsValid(pOldPath, paParms[2].u.pointer.size)
+                if (    !ShflStringIsValidIn(pNewPath, paParms[1].u.pointer.size, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8))
+                    ||  !ShflStringIsValidIn(pOldPath, paParms[2].u.pointer.size, RT_BOOL(pClient->fu32Flags & SHFL_CF_UTF8))
                     ||  (cbInfo != sizeof(SHFLFSOBJINFO))
                    )
                 {
@@ -1283,7 +1301,7 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
     case SHFL_FN_ADD_MAPPING:
     {
         Log(("SharedFolders host service: svcCall: SHFL_FN_ADD_MAPPING\n"));
-        LogRel(("SharedFolders host service: adding host mapping\n"));
+        LogRel(("SharedFolders host service: Adding host mapping\n"));
         /* Verify parameter count and types. */
         if (   (cParms != SHFL_CPARMS_ADD_MAPPING)
            )
@@ -1305,8 +1323,8 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
             uint32_t fFlags         = paParms[2].u.uint32;
 
             /* Verify parameters values. */
-            if (    !ShflStringIsValid(pFolderName, paParms[0].u.pointer.size)
-                ||  !ShflStringIsValid(pMapName, paParms[1].u.pointer.size)
+            if (    !ShflStringIsValidIn(pFolderName, paParms[0].u.pointer.size, false /*fUtf8Not16*/)
+                ||  !ShflStringIsValidIn(pMapName, paParms[1].u.pointer.size, false /*fUtf8Not16*/)
                )
             {
                 rc = VERR_INVALID_PARAMETER;
@@ -1335,14 +1353,14 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
             }
         }
         if (RT_FAILURE(rc))
-            LogRel(("SharedFolders host service: adding host mapping failed with rc=%Rrc\n", rc));
+            LogRel(("SharedFolders host service: Adding host mapping failed with rc=%Rrc\n", rc));
         break;
     }
 
     case SHFL_FN_REMOVE_MAPPING:
     {
         Log(("SharedFolders host service: svcCall: SHFL_FN_REMOVE_MAPPING\n"));
-        LogRel(("SharedFolders host service: removing host mapping '%ls'\n",
+        LogRel(("SharedFolders host service: Removing host mapping '%ls'\n",
                 ((SHFLSTRING *)paParms[0].u.pointer.addr)->String.ucs2));
 
         /* Verify parameter count and types. */
@@ -1361,7 +1379,7 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
             SHFLSTRING *pString = (SHFLSTRING *)paParms[0].u.pointer.addr;
 
             /* Verify parameters values. */
-            if (!ShflStringIsValid(pString, paParms[0].u.pointer.size))
+            if (!ShflStringIsValidIn(pString, paParms[0].u.pointer.size, false /*fUtf8Not16*/))
             {
                 rc = VERR_INVALID_PARAMETER;
             }
@@ -1378,7 +1396,7 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
             }
         }
         if (RT_FAILURE(rc))
-            LogRel(("SharedFolders host service: removing host mapping failed with rc=%Rrc\n", rc));
+            LogRel(("SharedFolders host service: Removing host mapping failed with rc=%Rrc\n", rc));
         break;
     }
 
@@ -1435,7 +1453,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
 
     if (!VALID_PTR(ptable))
     {
-        LogRelFunc(("SharedFolders host service: bad value of ptable (%p)\n", ptable));
+        LogRelFunc(("SharedFolders host service: Bad value of ptable (%p)\n", ptable));
         rc = VERR_INVALID_PARAMETER;
     }
     else
@@ -1446,7 +1464,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
         if (    ptable->cbSize != sizeof (VBOXHGCMSVCFNTABLE)
             ||  ptable->u32Version != VBOX_HGCM_SVC_VERSION)
         {
-            LogRelFunc(("SharedFolders host service: version mismatch while loading: ptable->cbSize = %u (should be %u), ptable->u32Version = 0x%08X (should be 0x%08X)\n",
+            LogRelFunc(("SharedFolders host service: Version mismatch while loading: ptable->cbSize = %u (should be %u), ptable->u32Version = 0x%08X (should be 0x%08X)\n",
                         ptable->cbSize, sizeof (VBOXHGCMSVCFNTABLE), ptable->u32Version, VBOX_HGCM_SVC_VERSION));
             rc = VERR_VERSION_MISMATCH;
         }
