@@ -1,6 +1,5 @@
 /* $Id: tstSharedFolderService.cpp $ */
 /** @file
- *
  * Testcase for the shared folder service vbsf API.
  *
  * Note that this is still very threadbare (there is an awful lot which should
@@ -11,7 +10,7 @@
  */
 
 /*
- * Copyright (C) 2011 Oracle Corporation
+ * Copyright (C) 2011-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,9 +21,10 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/******************************************************************************
-*   Header Files                                                              *
-******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 
 #include "tstSharedFolderService.h"
 #include "vbsf.h"
@@ -39,20 +39,22 @@
 
 #include "teststubs.h"
 
-/******************************************************************************
-*   Global Variables                                                          *
-******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 static RTTEST g_hTest = NIL_RTTEST;
 
-/******************************************************************************
-*   Declarations                                                              *
-******************************************************************************/
 
+/*********************************************************************************************************************************
+*   Declarations                                                                                                                 *
+*********************************************************************************************************************************/
 extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *ptable);
 
-/******************************************************************************
-*   Helpers                                                                   *
-******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Helpers                                                                                                                      *
+*********************************************************************************************************************************/
 
 /** Simple call handle structure for the guest call completion callback */
 struct VBOXHGCMCALLHANDLE_TYPEDEF
@@ -62,7 +64,7 @@ struct VBOXHGCMCALLHANDLE_TYPEDEF
 };
 
 /** Call completion callback for guest calls. */
-static void callComplete(VBOXHGCMCALLHANDLE callHandle, int32_t rc)
+static DECLCALLBACK(void) callComplete(VBOXHGCMCALLHANDLE callHandle, int32_t rc)
 {
     callHandle->rc = rc;
 }
@@ -102,15 +104,16 @@ static void bufferFromPath(void *pvDest, size_t cb, const char *pcszSrc)
 }
 
 #define ARRAY_FROM_PATH(a, b) \
-do { \
-    Assert((a) == (a)); /* Constant parameter */ \
-    Assert(sizeof((a)) > 0); \
-    bufferFromPath(a, sizeof(a), b); \
-} while(0)
+    do { \
+        Assert((a) == (a)); /* Constant parameter */ \
+        Assert(sizeof((a)) > 0); \
+        bufferFromPath(a, sizeof(a), b); \
+    } while (0)
 
-/******************************************************************************
-*   Stub functions                                                            *
-******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Stub functions and data                                                                                                      *
+*********************************************************************************************************************************/
 
 static PRTDIR testRTDirClosepDir;
 
@@ -306,7 +309,7 @@ static uint64_t testRTFileSetFMode;
 extern int testRTFileSetMode(RTFILE File, RTFMODE fMode)
 {
  /* RTPrintf("%s: fMode=%llu\n", __PRETTY_FUNCTION__, LLUIFY(fMode)); */
-    testRTFileSetFMode = fMode; 
+    testRTFileSetFMode = fMode;
     return VINF_SUCCESS;
 }
 
@@ -369,7 +372,7 @@ extern int  testRTFileWrite(RTFILE File, const void *pvBuf, size_t cbToWrite,
         *pcbWritten = strlen(testRTFileWriteData) + 1;
     return VINF_SUCCESS;
 }
-                             
+
 extern int testRTFsQueryProperties(const char *pszFsPath,
                                       PRTFSPROPERTIES pProperties)
 {
@@ -383,7 +386,7 @@ extern int testRTFsQueryProperties(const char *pszFsPath,
 extern int testRTFsQuerySerial(const char *pszFsPath, uint32_t *pu32Serial)
 { RTPrintf("%s\n", __PRETTY_FUNCTION__); return 0; }
 extern int testRTFsQuerySizes(const char *pszFsPath, PRTFOFF pcbTotal,
-                                 RTFOFF *pcbFree, uint32_t *pcbBlock, 
+                                 RTFOFF *pcbFree, uint32_t *pcbBlock,
                                  uint32_t *pcbSector) { RTPrintf("%s\n", __PRETTY_FUNCTION__); return 0; }
 
 extern int testRTPathQueryInfoEx(const char *pszPath,
@@ -398,15 +401,16 @@ extern int testRTPathQueryInfoEx(const char *pszPath,
     return VINF_SUCCESS;
 }
 
-extern int testRTSymlinkDelete(const char *pszSymlink, uint32_t fDelete) 
+extern int testRTSymlinkDelete(const char *pszSymlink, uint32_t fDelete)
 { RTPrintf("%s\n", __PRETTY_FUNCTION__); return 0; }
 extern int testRTSymlinkRead(const char *pszSymlink, char *pszTarget,
                               size_t cbTarget, uint32_t fRead)
 { RTPrintf("%s\n", __PRETTY_FUNCTION__); return 0; }
 
-/******************************************************************************
-*   Tests                                                                     *
-******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Tests                                                                                                                        *
+*********************************************************************************************************************************/
 
 /* Sub-tests for testMappingsQuery(). */
 void testMappingsQuerySimple(RTTEST hTest) {}
@@ -484,13 +488,11 @@ struct TESTSHFLSTRING
 static void fillTestShflString(struct TESTSHFLSTRING *pDest,
                                const char *pcszSource)
 {
-    unsigned i;
-    
-    AssertRelease(  strlen(pcszSource) * 2 + 2 < sizeof(*pDest)
-                  - RT_UOFFSETOF(SHFLSTRING, String));
-    pDest->string.u16Size = (uint16_t)strlen(pcszSource) * 2 + 2;
-    pDest->string.u16Length = (uint16_t)strlen(pcszSource);
-    for (i = 0; i < strlen(pcszSource) + 1; ++i)
+    AssertRelease(  strlen(pcszSource) * 2 + 2
+                  < sizeof(*pDest) - RT_UOFFSETOF(SHFLSTRING, String));
+    pDest->string.u16Length = (uint16_t)(strlen(pcszSource) * sizeof(RTUTF16));
+    pDest->string.u16Size   = pDest->string.u16Length + sizeof(RTUTF16);
+    for (unsigned i = 0; i <= pDest->string.u16Length; ++i)
         pDest->string.String.ucs2[i] = (uint16_t)pcszSource[i];
 }
 
@@ -511,6 +513,7 @@ static SHFLROOT initWithWritableMapping(RTTEST hTest,
     AssertReleaseRC(VBoxHGCMSvcLoad(psvcTable));
     AssertRelease(  psvcTable->pvService
                   = RTTestGuardedAllocTail(hTest, psvcTable->cbClient));
+    RT_BZERO(psvcTable->pvService, psvcTable->cbClient);
     fillTestShflString(&FolderName, pcszFolderName);
     fillTestShflString(&Mapping, pcszMapping);
     aParms[0].setPointer(&FolderName,   RT_UOFFSETOF(SHFLSTRING, String)
@@ -1109,9 +1112,10 @@ void testLockFileSimple(RTTEST hTest)
                      (hTest, "File=%llu\n", LLUIFY(testRTFileCloseFile)));
 }
 
-/******************************************************************************
-*   Main code                                                                 *
-******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Main code                                                                                                                    *
+*********************************************************************************************************************************/
 
 static void testAPI(RTTEST hTest)
 {
