@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,12 +19,14 @@
 # include "testcase/tstSharedFolderService.h"
 #endif
 
+#include "vbsfpath.h"
 #include "mappings.h"
 #include "vbsf.h"
 #include "shflhandle.h"
 
 #include <iprt/alloc.h>
 #include <iprt/assert.h>
+#include <iprt/asm.h>
 #include <iprt/fs.h>
 #include <iprt/dir.h>
 #include <iprt/file.h>
@@ -106,473 +108,34 @@ void vbsfStripLastComponent(char *pszFullPath, uint32_t cbFullPathRoot)
     LogFlowFunc(("%s, %s, %s\n", pszFullPath, delimLast, delimSecondLast));
 }
 
-static int vbsfCorrectCasing(SHFLCLIENTDATA *pClient, char *pszFullPath, char *pszStartComponent)
-{
-    PRTDIRENTRYEX  pDirEntry = NULL;
-    uint32_t       cbDirEntry, cbComponent;
-    int            rc = VERR_FILE_NOT_FOUND;
-    PRTDIR         hSearch = 0;
-    char           szWildCard[4];
-
-    Log2(("vbsfCorrectCasing: %s %s\n", pszFullPath, pszStartComponent));
-
-    cbComponent = (uint32_t) strlen(pszStartComponent);
-
-    cbDirEntry = 4096;
-    pDirEntry  = (PRTDIRENTRYEX)RTMemAlloc(cbDirEntry);
-    if (pDirEntry == 0)
-    {
-        AssertFailed();
-        return VERR_NO_MEMORY;
-    }
-
-    /** @todo this is quite inefficient, especially for directories with many files */
-    Assert(pszFullPath < pszStartComponent-1);
-    Assert(*(pszStartComponent-1) == RTPATH_DELIMITER);
-    *(pszStartComponent-1) = 0;
-    strcpy(pDirEntry->szName, pszFullPath);
-    szWildCard[0] = RTPATH_DELIMITER;
-    szWildCard[1] = '*';
-    szWildCard[2] = 0;
-    strcat(pDirEntry->szName, szWildCard);
-
-    rc = RTDirOpenFiltered(&hSearch, pDirEntry->szName, RTDIRFILTER_WINNT, 0);
-    *(pszStartComponent-1) = RTPATH_DELIMITER;
-    if (RT_FAILURE(rc))
-        goto end;
-
-    for (;;)
-    {
-        size_t cbDirEntrySize = cbDirEntry;
-
-        rc = RTDirReadEx(hSearch, pDirEntry, &cbDirEntrySize, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
-        if (rc == VERR_NO_MORE_FILES)
-            break;
-
-        if (   rc != VINF_SUCCESS
-            && rc != VWRN_NO_DIRENT_INFO)
-        {
-            AssertFailed();
-            if (   rc == VERR_NO_TRANSLATION
-                || rc == VERR_INVALID_UTF8_ENCODING)
-                continue;
-            break;
-        }
-
-        Log2(("vbsfCorrectCasing: found %s\n", &pDirEntry->szName[0]));
-        if (    pDirEntry->cbName == cbComponent
-            &&  !RTStrICmp(pszStartComponent, &pDirEntry->szName[0]))
-        {
-            Log(("Found original name %s (%s)\n", &pDirEntry->szName[0], pszStartComponent));
-            strcpy(pszStartComponent, &pDirEntry->szName[0]);
-            rc = VINF_SUCCESS;
-            break;
-        }
-    }
-
-end:
-    if (RT_FAILURE(rc))
-        Log(("vbsfCorrectCasing %s failed with %d\n", pszStartComponent, rc));
-
-    if (pDirEntry)
-        RTMemFree(pDirEntry);
-
-    if (hSearch)
-        RTDirClose(hSearch);
-    return rc;
-}
-
-/**
- * Do a simple path check given by pUtf8Path. Verify that the path is within
- * the root directory of the mapping. Count '..' and other path components
- * and check that we do not go over the root.
- *
- * @remarks This function assumes that the path will be appended to the root
- * directory of the shared folder mapping. Keep that in mind when checking
- * absolute pathes!
- */
-static int vbsfPathCheck(const char *pUtf8Path, size_t cbPath)
-{
-    int rc = VINF_SUCCESS;
-
-    size_t i = 0;
-    int cComponents = 0; /* How many normal path components. */
-    int cParentDirs = 0; /* How many '..' components. */
-
-    for (;;)
-    {
-        /* Skip leading path delimiters. */
-        while (   i < cbPath
-               && (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/'))
-            i++;
-
-        if (i >= cbPath)
-            break;
-
-        /* Check if that is a dot component. */
-        int cDots = 0;
-        while (i < cbPath && pUtf8Path[i] == '.')
-        {
-            cDots++;
-            i++;
-        }
-
-        if (   cDots >= 2 /* Consider all multidots sequences as a 'parent dir'. */
-            && (i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/')))
-        {
-            cParentDirs++;
-        }
-        else if (   cDots == 1
-                 && (i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/')))
-        {
-            /* Single dot, nothing changes. */
-        }
-        else
-        {
-            /* Skip this component. */
-            while (   i < cbPath
-                   && (pUtf8Path[i] != '\\' && pUtf8Path[i] != '/'))
-                i++;
-
-            cComponents++;
-        }
-
-        Assert(i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/'));
-
-        /* Verify counters for every component. */
-        if (cParentDirs > cComponents)
-        {
-            rc = VERR_INVALID_NAME;
-            break;
-        }
-    }
-
-    return rc;
-}
-
 static int vbsfBuildFullPath(SHFLCLIENTDATA *pClient, SHFLROOT root, PSHFLSTRING pPath,
                              uint32_t cbPath, char **ppszFullPath, uint32_t *pcbFullPathRoot,
                              bool fWildCard = false, bool fPreserveLastComponent = false)
 {
-    int rc = VINF_SUCCESS;
-    char *pszFullPath = NULL;
-    size_t cbRoot;
-    const char *pszRoot = vbsfMappingsQueryHostRoot(root);
-
-    if (   !pszRoot
-        || !(cbRoot = strlen(pszRoot)))
-    {
-        Log(("vbsfBuildFullPath: invalid root!\n"));
-        return VERR_INVALID_PARAMETER;
-    }
-
+    char *pszHostPath = NULL;
+    uint32_t fu32PathFlags = 0;
+    int rc = vbsfPathGuestToHost(pClient, root, pPath, cbPath,
+                                 &pszHostPath, pcbFullPathRoot, fWildCard, fPreserveLastComponent, &fu32PathFlags);
     if (BIT_FLAG(pClient->fu32Flags, SHFL_CF_UTF8))
     {
-        /* Verify that the path is under the root directory. */
-        rc = vbsfPathCheck((const char *)&pPath->String.utf8[0], pPath->u16Length);
-        if (RT_SUCCESS(rc))
-        {
-            size_t cbUtf8FullPath = cbRoot + 1 + pPath->u16Length + 1;
-            char *utf8FullPath = (char *) RTMemAllocZ(cbUtf8FullPath);
-
-            if (!utf8FullPath)
-            {
-                rc = VERR_NO_MEMORY;
-                *ppszFullPath = NULL;
-                Log(("RTMemAllocZ %x failed!!\n", cbUtf8FullPath));
-            }
-            else
-            {
-                memcpy(utf8FullPath, pszRoot, cbRoot);
-                utf8FullPath[cbRoot] = '/';
-                memcpy(utf8FullPath + cbRoot + 1, &pPath->String.utf8[0], pPath->u16Length);
-                utf8FullPath[cbUtf8FullPath - 1] = 0;
-                pszFullPath = utf8FullPath;
-
-                if (pcbFullPathRoot)
-                    *pcbFullPathRoot = (uint32_t)cbRoot; /* Must index the path delimiter. */
-            }
-        }
-        else
-        {
-            Log(("vbsfBuildFullPath: RTUtf16ToUtf8 failed with %Rrc\n", rc));
-        }
+        LogRel2(("SharedFolders: GuestToHost 0x%RX32 [%.*s]->[%s] %Rrc\n", fu32PathFlags, pPath->u16Length, &pPath->String.utf8[0], pszHostPath, rc));
     }
     else
     {
-#ifdef RT_OS_DARWIN
-/** @todo This belongs in rtPathToNative or in the windows shared folder file system driver...
- * The question is simply whether the NFD normalization is actually applied on a (virtual) file
- * system level in darwin, or just by the user mode application libs. */
-        SHFLSTRING *pPathParameter = pPath;
-        size_t cbPathLength;
-        CFMutableStringRef inStr = ::CFStringCreateMutable(NULL, 0);
-        uint16_t ucs2Length;
-        CFRange rangeCharacters;
-
-        // Is 8 times length enough for decomposed in worst case...?
-        cbPathLength = sizeof(SHFLSTRING) + pPathParameter->u16Length * 8 + 2;
-        pPath = (SHFLSTRING *)RTMemAllocZ(cbPathLength);
-        if (!pPath)
-        {
-            rc = VERR_NO_MEMORY;
-            Log(("RTMemAllocZ %x failed!!\n", cbPathLength));
-            return rc;
-        }
-
-        ::CFStringAppendCharacters(inStr, (UniChar*)pPathParameter->String.ucs2,
-                                   pPathParameter->u16Length / sizeof(pPathParameter->String.ucs2[0]));
-        ::CFStringNormalize(inStr, kCFStringNormalizationFormD);
-        ucs2Length = ::CFStringGetLength(inStr);
-
-        rangeCharacters.location = 0;
-        rangeCharacters.length = ucs2Length;
-        ::CFStringGetCharacters(inStr, rangeCharacters, pPath->String.ucs2);
-        pPath->String.ucs2[ucs2Length] = 0x0000; // NULL terminated
-        pPath->u16Length = ucs2Length * sizeof(pPath->String.ucs2[0]);
-        pPath->u16Size = pPath->u16Length + sizeof(pPath->String.ucs2[0]);
-
-        CFRelease(inStr);
-#endif
-        /* Client sends us UCS2, so convert it to UTF8. */
-        Log(("Root %s path %.*ls\n", pszRoot, pPath->u16Length/sizeof(pPath->String.ucs2[0]), pPath->String.ucs2));
-
-        /* Allocate buffer that will be able to contain the root prefix and
-         * the pPath converted to UTF8. Expect a 2 bytes UCS2 to be converted
-         * to 8 bytes UTF8 in the worst case.
-         */
-        uint32_t cbFullPath = (cbRoot + ShflStringLength(pPath)) * 4;
-        pszFullPath = (char *)RTMemAllocZ(cbFullPath);
-        if (!pszFullPath)
-        {
-            rc = VERR_NO_MEMORY;
-        }
-        else
-        {
-            memcpy(pszFullPath, pszRoot, cbRoot + 1);
-            char *pszDst = pszFullPath;
-            size_t cbDst = strlen(pszDst);
-            size_t cb    = cbFullPath;
-            if (pszDst[cbDst - 1] != RTPATH_DELIMITER)
-            {
-                pszDst[cbDst] = RTPATH_DELIMITER;
-                cbDst++;
-            }
-
-            if (pcbFullPathRoot)
-                *pcbFullPathRoot = cbDst - 1; /* Must index the path delimiter.  */
-
-            pszDst += cbDst;
-            cb     -= cbDst;
-
-            if (pPath->u16Length)
-            {
-                /* Convert and copy components. */
-                PRTUTF16 pwszSrc = &pPath->String.ucs2[0];
-
-                /* Correct path delimiters */
-                if (pClient->PathDelimiter != RTPATH_DELIMITER)
-                {
-                    LogFlow(("Correct path delimiter in %ls\n", pwszSrc));
-                    while (*pwszSrc)
-                    {
-                        if (*pwszSrc == pClient->PathDelimiter)
-                            *pwszSrc = RTPATH_DELIMITER;
-                        pwszSrc++;
-                    }
-                    pwszSrc = &pPath->String.ucs2[0];
-                    LogFlow(("Corrected string %ls\n", pwszSrc));
-                }
-                if (*pwszSrc == RTPATH_DELIMITER)
-                    pwszSrc++;  /* we already appended a delimiter to the first part */
-
-                rc = RTUtf16ToUtf8Ex(pwszSrc, RTSTR_MAX, &pszDst, cb, NULL);
-                if (RT_FAILURE(rc))
-                {
-                    AssertFailed();
-#ifdef RT_OS_DARWIN
-                    RTMemFree(pPath);
-                    pPath = pPathParameter;
-#endif
-                    return rc;
-                }
-
-                cbDst = (uint32_t)strlen(pszDst);
-
-                /* Verify that the path is under the root directory. */
-                rc = vbsfPathCheck(pszDst, cbDst);
-                if (RT_FAILURE(rc))
-                {
-#ifdef RT_OS_DARWIN
-                    RTMemFree(pPath);
-                    pPath = pPathParameter;
-#endif
-                    return rc;
-                }
-
-                cb     -= cbDst;
-                pszDst += cbDst;
-
-                Assert(cb > 0);
-            }
-
-            /* Nul terminate the string */
-            *pszDst = 0;
-        }
-#ifdef RT_OS_DARWIN
-        RTMemFree(pPath);
-        pPath = pPathParameter;
-#endif
+        LogRel2(("SharedFolders: GuestToHost 0x%RX32 [%.*ls]->[%s] %Rrc\n", fu32PathFlags, pPath->u16Length / 2, &pPath->String.ucs2[0], pszHostPath, rc));
     }
 
     if (RT_SUCCESS(rc))
     {
-        /* When the host file system is case sensitive and the guest expects
-         * a case insensitive fs, then problems can occur */
-        if (     vbsfIsHostMappingCaseSensitive(root)
-            &&  !vbsfIsGuestMappingCaseSensitive(root))
-        {
-            RTFSOBJINFO info;
-            char *pszLastComponent = NULL;
-
-            if (fWildCard || fPreserveLastComponent)
-            {
-                /* strip off the last path component, that has to be preserved:
-                 * contains the wildcard(s) or a 'rename' target. */
-                size_t cb = strlen(pszFullPath);
-                char *pszSrc = pszFullPath + cb - 1;
-
-                while (pszSrc > pszFullPath)
-                {
-                    if (*pszSrc == RTPATH_DELIMITER)
-                        break;
-                    pszSrc--;
-                }
-                if (*pszSrc == RTPATH_DELIMITER)
-                {
-                    bool fHaveWildcards = false;
-                    char *psz = pszSrc;
-
-                    while (*psz)
-                    {
-                        char ch = *psz;
-                        if (ch == '*' || ch == '?' || ch == '>' || ch == '<' || ch == '"')
-                        {
-                            fHaveWildcards = true;
-                            break;
-                        }
-                        psz++;
-                    }
-
-                    if (fHaveWildcards || fPreserveLastComponent)
-                    {
-                        pszLastComponent = pszSrc;
-                        *pszLastComponent = 0;
-                    }
-                }
-            }
-
-            /** @todo don't check when creating files or directories; waste of time */
-            rc = RTPathQueryInfoEx(pszFullPath, &info, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
-            if (rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND)
-            {
-                size_t cb = strlen(pszFullPath);
-                char   *pszSrc = pszFullPath + cb - 1;
-
-                Log(("Handle case insensitive guest fs on top of host case sensitive fs for %s\n", pszFullPath));
-
-                /* Find partial path that's valid */
-                while (pszSrc > pszFullPath)
-                {
-                    if (*pszSrc == RTPATH_DELIMITER)
-                    {
-                        *pszSrc = 0;
-                        rc = RTPathQueryInfoEx(pszFullPath, &info, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
-                        *pszSrc = RTPATH_DELIMITER;
-                        if (RT_SUCCESS(rc))
-                        {
-#ifdef DEBUG
-                            *pszSrc = 0;
-                            Log(("Found valid partial path %s\n", pszFullPath));
-                            *pszSrc = RTPATH_DELIMITER;
-#endif
-                            break;
-                        }
-                    }
-
-                    pszSrc--;
-                }
-                Assert(*pszSrc == RTPATH_DELIMITER && RT_SUCCESS(rc));
-                if (    *pszSrc == RTPATH_DELIMITER
-                    &&  RT_SUCCESS(rc))
-                {
-                    pszSrc++;
-                    for (;;)
-                    {
-                        char *pszEnd = pszSrc;
-                        bool fEndOfString = true;
-
-                        while (*pszEnd)
-                        {
-                            if (*pszEnd == RTPATH_DELIMITER)
-                                break;
-                            pszEnd++;
-                        }
-
-                        if (*pszEnd == RTPATH_DELIMITER)
-                        {
-                            fEndOfString = false;
-                            *pszEnd = 0;
-                            rc = RTPathQueryInfoEx(pszSrc, &info, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
-                            Assert(rc == VINF_SUCCESS || rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND);
-                        }
-                        else if (pszEnd == pszSrc)
-                            rc = VINF_SUCCESS;  /* trailing delimiter */
-                        else
-                            rc = VERR_FILE_NOT_FOUND;
-
-                        if (rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND)
-                        {
-                            /* path component is invalid; try to correct the casing */
-                            rc = vbsfCorrectCasing(pClient, pszFullPath, pszSrc);
-                            if (RT_FAILURE(rc))
-                            {
-                                if (!fEndOfString)
-                                    *pszEnd = RTPATH_DELIMITER; /* restore the original full path */
-                                break;
-                            }
-                        }
-
-                        if (fEndOfString)
-                            break;
-
-                        *pszEnd = RTPATH_DELIMITER;
-                        pszSrc = pszEnd + 1;
-                    }
-                    if (RT_FAILURE(rc))
-                        Log(("Unable to find suitable component rc=%d\n", rc));
-                }
-                else
-                    rc = VERR_FILE_NOT_FOUND;
-
-            }
-            if (pszLastComponent)
-                *pszLastComponent = RTPATH_DELIMITER;
-
-            /* might be a new file so don't fail here! */
-            rc = VINF_SUCCESS;
-        }
-        *ppszFullPath = pszFullPath;
-
-        LogFlow(("vbsfBuildFullPath: %s rc=%Rrc\n", pszFullPath, rc));
+        if (ppszFullPath)
+            *ppszFullPath = pszHostPath;
     }
-
     return rc;
 }
 
 static void vbsfFreeFullPath(char *pszFullPath)
 {
-    RTMemFree(pszFullPath);
+    vbsfFreeHostPath(pszFullPath);
 }
 
 /**
@@ -1544,7 +1107,7 @@ int vbsfDirList(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle, SHFLS
 
             Assert(pHandle->dir.pLastValidEntry == 0);
 
-            rc = vbsfBuildFullPath(pClient, root, pPath, pPath->u16Size, &pszFullPath, NULL, true);
+            rc = vbsfBuildFullPath(pClient, root, pPath, pPath->u16Size + SHFLSTRING_HEADER_SIZE, &pszFullPath, NULL, true);
 
             if (RT_SUCCESS(rc))
             {
@@ -1732,6 +1295,17 @@ int vbsfReadLink(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pPath, uint
     if (RT_SUCCESS(rc))
     {
         rc = RTSymlinkRead(pszFullPath, (char *) pBuffer, cbBuffer, 0);
+        if (RT_SUCCESS(rc))
+        {
+            /* Convert the slashes in the link target to the guest path separator characters. */
+            char *psz = (char *)pBuffer;
+            while (*psz != '\0')
+            {
+                if (*psz == RTPATH_DELIMITER)
+                    *psz = pClient->PathDelimiter;
+                psz++;
+            }
+        }
 
         /* free the path string */
         vbsfFreeFullPath(pszFullPath);
@@ -1947,7 +1521,8 @@ int vbsfQueryVolumeInfo(SHFLCLIENTDATA *pClient, SHFLROOT root, uint32_t flags,
     pSFDEntry   = (PSHFLVOLINFO)pBuffer;
 
     ShflStringInitBuffer(&dummy, sizeof(dummy));
-    rc = vbsfBuildFullPath(pClient, root, &dummy, 0, &pszFullPath, NULL);
+    dummy.String.ucs2[0] = '\0';
+    rc = vbsfBuildFullPath(pClient, root, &dummy, sizeof(dummy), &pszFullPath, NULL);
 
     if (RT_SUCCESS(rc))
     {
@@ -2225,11 +1800,11 @@ int vbsfRename(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pSrc, SHFLSTR
     char *pszFullPathSrc = NULL;
     char *pszFullPathDest = NULL;
 
-    rc = vbsfBuildFullPath(pClient, root, pSrc, pSrc->u16Size, &pszFullPathSrc, NULL);
+    rc = vbsfBuildFullPath(pClient, root, pSrc, pSrc->u16Size + SHFLSTRING_HEADER_SIZE, &pszFullPathSrc, NULL);
     if (rc != VINF_SUCCESS)
         return rc;
 
-    rc = vbsfBuildFullPath(pClient, root, pDest, pDest->u16Size, &pszFullPathDest, NULL, false, true);
+    rc = vbsfBuildFullPath(pClient, root, pDest, pDest->u16Size + SHFLSTRING_HEADER_SIZE, &pszFullPathDest, NULL, false, true);
     if (RT_SUCCESS (rc))
     {
         Log(("Rename %s to %s\n", pszFullPathSrc, pszFullPathDest));
@@ -2281,6 +1856,7 @@ int vbsfSymlink(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pNewPath, SH
     int rc = VINF_SUCCESS;
 
     char *pszFullNewPath = NULL;
+    char *pszFullOldPath = NULL;
     const char *pszOldPath = (const char *)pOldPath->String.utf8;
 
     /* XXX: no support for UCS2 at the moment. */
@@ -2293,9 +1869,17 @@ int vbsfSymlink(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pNewPath, SH
     if (!fSymlinksCreate)
         return VERR_WRITE_PROTECT; /* XXX or VERR_TOO_MANY_SYMLINKS? */
 
-    rc = vbsfBuildFullPath(pClient, root, pNewPath, pNewPath->u16Size, &pszFullNewPath, NULL);
+    rc = vbsfBuildFullPath(pClient, root, pNewPath, pNewPath->u16Size + SHFLSTRING_HEADER_SIZE, &pszFullNewPath, NULL);
     AssertRCReturn(rc, rc);
 
+    /* Verify that the link target can be a valid host path, i.e. does not contain invalid characters. */
+    rc = vbsfBuildFullPath(pClient, root, pOldPath, pOldPath->u16Size + SHFLSTRING_HEADER_SIZE, &pszFullOldPath, NULL);
+    if (RT_FAILURE(rc))
+    {
+        vbsfFreeFullPath(pszFullNewPath);
+        return rc;
+    }
+
     rc = RTSymlinkCreate(pszFullNewPath, (const char *)pOldPath->String.utf8,
                          RTSYMLINKTYPE_UNKNOWN, 0);
     if (RT_SUCCESS(rc))
@@ -2306,6 +1890,7 @@ int vbsfSymlink(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pNewPath, SH
             vbfsCopyFsObjInfoFromIprt(pInfo, &info);
     }
 
+    vbsfFreeFullPath(pszFullOldPath);
     vbsfFreeFullPath(pszFullNewPath);
 
     return rc;
