@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2009-2012 Oracle Corporation
+ * Copyright (C) 2009-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -45,7 +45,10 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetActiveAttrib(GLuint program, GL
         crGetActive_t zero;
         zero.length = 0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
+    /* zero out just the header to ensure it initially contains zero size values */
+    memset(pLocal, 0, sizeof (*pLocal));
     cr_server.head_spu->dispatch_table.GetActiveAttrib(crStateGetProgramHWID(program), index, bufSize, &pLocal->length, &pLocal->size, &pLocal->type, (char*)&pLocal[1]);
     crServerReturnValue(pLocal, pLocal->length+1+sizeof(crGetActive_t));
     crFree(pLocal);
@@ -61,7 +64,10 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetActiveUniform(GLuint program, G
         crGetActive_t zero;
         zero.length = 0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
+    /* zero out just the header to ensure it initially contains zero size values */
+    memset(pLocal, 0, sizeof (*pLocal));
     cr_server.head_spu->dispatch_table.GetActiveUniform(crStateGetProgramHWID(program), index, bufSize, &pLocal->length, &pLocal->size, &pLocal->type, (char*)&pLocal[1]);
     crServerReturnValue(pLocal, pLocal->length+1+sizeof(crGetActive_t));
     crFree(pLocal);
@@ -76,33 +82,12 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetAttachedShaders(GLuint program,
     {
         GLsizei zero=0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
+    /* initial (fallback )value */
+    *pLocal = 0;
     cr_server.head_spu->dispatch_table.GetAttachedShaders(crStateGetProgramHWID(program), maxCount, pLocal, (GLuint*)&pLocal[1]);
 
-    {
-        GLsizei i;
-        GLuint *ids=(GLuint*)&pLocal[1];
-
-        for (i=0; i<*pLocal; ++i);
-          ids[i] = crStateGLSLShaderHWIDtoID(ids[i]);
-    }
-
-    crServerReturnValue(pLocal, (*pLocal)*sizeof(GLuint)+sizeof(GLsizei));
-    crFree(pLocal);
-}
-
-void SERVER_DISPATCH_APIENTRY crServerDispatchGetAttachedObjectsARB(GLhandleARB containerObj, GLsizei maxCount, GLsizei * count, GLhandleARB * obj)
-{
-    GLsizei *pLocal;
-
-    pLocal = (GLsizei*) crAlloc(maxCount*sizeof(GLhandleARB)+sizeof(GLsizei));
-    if (!pLocal)
-    {
-        GLsizei zero=0;
-        crServerReturnValue(&zero, sizeof(zero));
-    }
-    cr_server.head_spu->dispatch_table.GetAttachedObjectsARB(crStateGetProgramHWID(containerObj), maxCount, pLocal, (GLhandleARB*)&pLocal[1]);
-
     {
         GLsizei i;
         GLuint *ids=(GLuint*)&pLocal[1];
@@ -111,13 +96,40 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetAttachedObjectsARB(GLhandleARB
           ids[i] = crStateGLSLShaderHWIDtoID(ids[i]);
     }
 
-    crServerReturnValue(pLocal, (*pLocal)*sizeof(GLhandleARB)+sizeof(GLsizei));
+    crServerReturnValue(pLocal, (*pLocal)*sizeof(GLuint)+sizeof(GLsizei));
+    crFree(pLocal);
+}
+
+void SERVER_DISPATCH_APIENTRY crServerDispatchGetAttachedObjectsARB(VBoxGLhandleARB containerObj, GLsizei maxCount, GLsizei * count, VBoxGLhandleARB * obj)
+{
+    GLsizei *pLocal;
+
+    pLocal = (GLsizei*) crAlloc(maxCount*sizeof(VBoxGLhandleARB)+sizeof(GLsizei));
+    if (!pLocal)
+    {
+        GLsizei zero=0;
+        crServerReturnValue(&zero, sizeof(zero));
+        return;
+    }
+    /* initial (fallback )value */
+    *pLocal = 0;
+    cr_server.head_spu->dispatch_table.GetAttachedObjectsARB(crStateGetProgramHWID(containerObj), maxCount, pLocal, (VBoxGLhandleARB*)&pLocal[1]);
+
+    {
+        GLsizei i;
+        GLuint *ids=(GLuint*)&pLocal[1];
+
+        for (i=0; i<*pLocal; ++i)
+          ids[i] = crStateGLSLShaderHWIDtoID(ids[i]);
+    }
+
+    crServerReturnValue(pLocal, (*pLocal)*sizeof(VBoxGLhandleARB)+sizeof(GLsizei));
     crFree(pLocal);
 }
 
 AssertCompile(sizeof(GLsizei) == 4);
 
-void SERVER_DISPATCH_APIENTRY crServerDispatchGetInfoLogARB(GLhandleARB obj, GLsizei maxLength, GLsizei * length, GLcharARB * infoLog)
+void SERVER_DISPATCH_APIENTRY crServerDispatchGetInfoLogARB(VBoxGLhandleARB obj, GLsizei maxLength, GLsizei * length, GLcharARB * infoLog)
 {
     GLsizei *pLocal;
     GLuint hwid;
@@ -127,7 +139,10 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetInfoLogARB(GLhandleARB obj, GLs
     {
         GLsizei zero=0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
+    /* initial (fallback )value */
+    *pLocal = 0;
     /*@todo: recheck*/
     hwid = crStateGetProgramHWID(obj);
     if (!hwid) hwid = crStateGetShaderHWID(obj);
@@ -146,7 +161,10 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetShaderInfoLog(GLuint shader, GL
     {
         GLsizei zero=0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
+    /* initial (fallback )value */
+    *pLocal = 0;
     cr_server.head_spu->dispatch_table.GetShaderInfoLog(crStateGetShaderHWID(shader), bufSize, pLocal, (char*)&pLocal[1]);
     crServerReturnValue(pLocal, pLocal[0]+sizeof(GLsizei));
     crFree(pLocal);
@@ -161,7 +179,10 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetProgramInfoLog(GLuint program,
     {
         GLsizei zero=0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
+    /* initial (fallback )value */
+    *pLocal = 0;
     cr_server.head_spu->dispatch_table.GetProgramInfoLog(crStateGetProgramHWID(program), bufSize, pLocal, (char*)&pLocal[1]);
     CRASSERT(pLocal[0] <= bufSize);
     crServerReturnValue(pLocal, pLocal[0]+sizeof(GLsizei));
@@ -177,7 +198,10 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetShaderSource(GLuint shader, GLs
     {
         GLsizei zero=0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
+    /* initial (fallback )value */
+    *pLocal = 0;
     cr_server.head_spu->dispatch_table.GetShaderSource(crStateGetShaderHWID(shader), bufSize, pLocal, (char*)&pLocal[1]);
     CRASSERT(pLocal[0] <= bufSize);
     crServerReturnValue(pLocal, pLocal[0]+sizeof(GLsizei));
@@ -197,18 +221,45 @@ crServerDispatchGetUniformsLocations(GLuint program, GLsizei maxcbData, GLsizei
     {
         GLsizei zero=0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
     
+    /* initial (fallback )value */
+    *pLocal = 0;
     crStateGLSLProgramCacheUniforms(program, maxcbData, pLocal, (char*)&pLocal[1]);
 
     crServerReturnValue(pLocal, (*pLocal)+sizeof(GLsizei));
     crFree(pLocal);
 }
 
+void SERVER_DISPATCH_APIENTRY
+crServerDispatchGetAttribsLocations(GLuint program, GLsizei maxcbData, GLsizei * cbData, GLvoid * pData)
+{
+    GLsizei *pLocal;
+
+    (void) cbData;
+    (void) pData;
+
+    pLocal = (GLsizei*) crAlloc(maxcbData+sizeof(GLsizei));
+    if (!pLocal)
+    {
+        GLsizei zero=0;
+        crServerReturnValue(&zero, sizeof(zero));
+        return;
+    }
+
+    /* initial (fallback )value */
+    *pLocal = 0;
+    crStateGLSLProgramCacheAttribs(program, maxcbData, pLocal, (char*)&pLocal[1]);
+
+    crServerReturnValue(pLocal, (*pLocal)+sizeof(GLsizei));
+    crFree(pLocal);
+}
+
 static GLint __GetUniformSize(GLuint program, GLint location)
 {
-    GLint  size;
-    GLenum type;
+    GLint  size = 0;
+    GLenum type = 0;
 
     /*@todo: check if index and location is the same*/
     cr_server.head_spu->dispatch_table.GetActiveUniform(crStateGetProgramHWID(program), location, 0, NULL, &size, &type, NULL);
@@ -226,6 +277,7 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetUniformfv(GLuint program, GLint
     {
         GLsizei zero=0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
 
     cr_server.head_spu->dispatch_table.GetUniformfv(crStateGetProgramHWID(program), location, pLocal);
@@ -244,6 +296,7 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetUniformiv(GLuint program, GLint
     {
         GLsizei zero=0;
         crServerReturnValue(&zero, sizeof(zero));
+        return;
     }
 
     cr_server.head_spu->dispatch_table.GetUniformiv(crStateGetProgramHWID(program), location, pLocal);
@@ -286,7 +339,7 @@ GLboolean SERVER_DISPATCH_APIENTRY crServerDispatchIsProgram(GLuint program)
     return retval; /* ignored */
 }
 
-void SERVER_DISPATCH_APIENTRY crServerDispatchGetObjectParameterfvARB( GLhandleARB obj, GLenum pname, GLfloat * params )
+void SERVER_DISPATCH_APIENTRY crServerDispatchGetObjectParameterfvARB( VBoxGLhandleARB obj, GLenum pname, GLfloat * params )
 {
     GLfloat local_params[1];
     GLuint hwid = crStateGetProgramHWID(obj);
@@ -305,7 +358,7 @@ void SERVER_DISPATCH_APIENTRY crServerDispatchGetObjectParameterfvARB( GLhandleA
     crServerReturnValue( &(local_params[0]), 1*sizeof(GLfloat) );
 }
 
-void SERVER_DISPATCH_APIENTRY crServerDispatchGetObjectParameterivARB( GLhandleARB obj, GLenum pname, GLint * params )
+void SERVER_DISPATCH_APIENTRY crServerDispatchGetObjectParameterivARB( VBoxGLhandleARB obj, GLenum pname, GLint * params )
 {
     GLint local_params[1];
     GLuint hwid = crStateGetProgramHWID(obj);
