@@ -4,7 +4,7 @@
 # VirtualBox linux installation script
 
 #
-# Copyright (C) 2007-2012 Oracle Corporation
+# Copyright (C) 2007-2015 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -108,9 +108,12 @@ check_previous() {
     check_binary "/usr/bin/VBoxSDL" "$install_dir" &&
     check_binary "/usr/bin/VBoxVRDP" "$install_dir" &&
     check_binary "/usr/bin/VBoxHeadless" "$install_dir" &&
+    check_binary "/usr/bin/VBoxDTrace" "$install_dir" &&
     check_binary "/usr/bin/VBoxBalloonCtrl" "$install_dir" &&
     check_binary "/usr/bin/VBoxAutostart" "$install_dir" &&
-    check_binary "/usr/bin/vboxwebsrv" "$install_dir"
+    check_binary "/usr/bin/vboxwebsrv" "$install_dir" &&
+    check_binary "/usr/bin/vbox-img" "$install_dir" &&
+    check_binary "/sbin/rcvboxdrv" "$install_dir"
 }
 
 ##############################################################################
@@ -246,6 +249,7 @@ if [ "$ACTION" = "install" ]; then
     # Terminate Server and VBoxNetDHCP if running
     terminate_proc VBoxSVC
     terminate_proc VBoxNetDHCP
+    terminate_proc VBoxNetNAT
 
     # Remove previous installation
     if [ -n "$PREV_INSTALLATION" -a -z "$FORCE_UPGRADE" -a ! "$VERSION" = "$INSTALL_VER" ] &&
@@ -281,7 +285,7 @@ if [ "$ACTION" = "install" ]; then
         log "Removing previous installation of VirtualBox $INSTALL_VER$INSTALL_REV from $PREV_INSTALLATION"
         log ""
 
-        stop_init_script vboxnet
+        stop_init_script vboxnet >/dev/null 2>&1  # Do we need this?
         delrunlevel vboxnet > /dev/null 2>&1
         if [ "$BUILD_MODULE" = "true" ]; then
             stop_init_script vboxdrv
@@ -355,16 +359,14 @@ if [ "$ACTION" = "install" ]; then
     #                 create symlinks for working around unsupported $ORIGIN/.. in VBoxC.so (setuid),
     #                 and finally make sure the directory is only writable by the user (paranoid).
     if [ -n "$HARDENED" ]; then
-        test -e $INSTALLATION_DIR/VirtualBox    && chmod 4511 $INSTALLATION_DIR/VirtualBox
-        test -e $INSTALLATION_DIR/VBoxSDL       && chmod 4511 $INSTALLATION_DIR/VBoxSDL
-        test -e $INSTALLATION_DIR/VBoxHeadless  && chmod 4511 $INSTALLATION_DIR/VBoxHeadless
-        test -e $INSTALLATION_DIR/VBoxNetDHCP   && chmod 4511 $INSTALLATION_DIR/VBoxNetDHCP
+        test -e $INSTALLATION_DIR/VirtualBox     && chmod 4511 $INSTALLATION_DIR/VirtualBox
+        test -e $INSTALLATION_DIR/VBoxSDL        && chmod 4511 $INSTALLATION_DIR/VBoxSDL
+        test -e $INSTALLATION_DIR/VBoxHeadless   && chmod 4511 $INSTALLATION_DIR/VBoxHeadless
+        test -e $INSTALLATION_DIR/VBoxNetDHCP    && chmod 4511 $INSTALLATION_DIR/VBoxNetDHCP
+        test -e $INSTALLATION_DIR/VBoxNetNAT     && chmod 4511 $INSTALLATION_DIR/VBoxNetNAT
 
         ln -sf $INSTALLATION_DIR/VBoxVMM.so   $INSTALLATION_DIR/components/VBoxVMM.so
-        ln -sf $INSTALLATION_DIR/VBoxREM.so   $INSTALLATION_DIR/components/VBoxREM.so
         ln -sf $INSTALLATION_DIR/VBoxRT.so    $INSTALLATION_DIR/components/VBoxRT.so
-        ln -sf $INSTALLATION_DIR/VBoxDDU.so   $INSTALLATION_DIR/components/VBoxDDU.so
-        ln -sf $INSTALLATION_DIR/VBoxXPCOM.so $INSTALLATION_DIR/components/VBoxXPCOM.so
 
         chmod go-w $INSTALLATION_DIR
     fi
@@ -374,22 +376,29 @@ if [ "$ACTION" = "install" ]; then
     test -e $INSTALLATION_DIR/VBoxVolInfo && chmod 4511 $INSTALLATION_DIR/VBoxVolInfo
 
     # Install runlevel scripts
-    # Note: vboxdrv is also handled by setup_init_script. This function will
-    #       use chkconfig to adjust the sequence numbers, therefore vboxdrv
-    #       numbers here should match the numbers in the vboxdrv.sh check
-    #       header!
-    install_init_script vboxdrv.sh vboxdrv
-    install_init_script vboxballoonctrl-service.sh vboxballoonctrl-service
-    install_init_script vboxautostart-service.sh vboxautostart-service
-    install_init_script vboxweb-service.sh vboxweb-service
+    install_init_script $INSTALLATION_DIR/vboxdrv.sh vboxdrv 2>> $LOG
+    install_init_script $INSTALLATION_DIR/vboxballoonctrl-service.sh vboxballoonctrl-service 2>> $LOG
+    install_init_script $INSTALLATION_DIR/vboxautostart-service.sh vboxautostart-service 2>> $LOG
+    install_init_script $INSTALLATION_DIR/vboxweb-service.sh vboxweb-service 2>> $LOG
+
+    # Write the configuration. Do this before we call /etc/init.d/vboxdrv setup!
+    echo "# VirtualBox installation directory" > $CONFIG_DIR/$CONFIG
+    echo "INSTALL_DIR='$INSTALLATION_DIR'" >> $CONFIG_DIR/$CONFIG
+    echo "# VirtualBox version" >> $CONFIG_DIR/$CONFIG
+    echo "INSTALL_VER='$VERSION'" >> $CONFIG_DIR/$CONFIG
+    echo "INSTALL_REV='$SVNREV'" >> $CONFIG_DIR/$CONFIG
+    echo "# Build type and user name for logging purposes" >> $CONFIG_DIR/$CONFIG
+    echo "BUILD_TYPE='$BUILD_BUILDTYPE'" >> $CONFIG_DIR/$CONFIG
+    echo "USERNAME='$BUILD_USERNAME'" >> $CONFIG_DIR/$CONFIG
+
     delrunlevel vboxdrv > /dev/null 2>&1
-    addrunlevel vboxdrv 20 80 # This may produce useful output
+    addrunlevel vboxdrv 2>> $LOG # This may produce useful output
     delrunlevel vboxballoonctrl-service > /dev/null 2>&1
-    addrunlevel vboxballoonctrl-service 25 75 # This may produce useful output
+    addrunlevel vboxballoonctrl-service 2>> $LOG # This may produce useful output
     delrunlevel vboxautostart-service > /dev/null 2>&1
-    addrunlevel vboxautostart-service 25 75 # This may produce useful output
+    addrunlevel vboxautostart-service 2>> $LOG # This may produce useful output
     delrunlevel vboxweb-service > /dev/null 2>&1
-    addrunlevel vboxweb-service 25 75 # This may produce useful output
+    addrunlevel vboxweb-service 2>> $LOG # This may produce useful output
 
     # Create users group
     groupadd -r -f $GROUPNAME 2> /dev/null
@@ -403,7 +412,12 @@ if [ "$ACTION" = "install" ]; then
     ln -sf $INSTALLATION_DIR/VBox.sh /usr/bin/VBoxBalloonCtrl
     ln -sf $INSTALLATION_DIR/VBox.sh /usr/bin/VBoxAutostart
     ln -sf $INSTALLATION_DIR/VBox.sh /usr/bin/vboxwebsrv
+    ln -sf $INSTALLATION_DIR/vbox-img /usr/bin/vbox-img
     ln -sf $INSTALLATION_DIR/VBox.png /usr/share/pixmaps/VBox.png
+    if [ -f $INSTALLATION_DIR/VBoxDTrace ]; then
+        ln -sf $INSTALLATION_DIR/VBox.sh /usr/bin/VBoxDTrace
+    fi
+    ln -sf $INSTALLATION_DIR/vboxdrv.sh /sbin/rcvboxdrv
     # Unity and Nautilus seem to look here for their icons
     ln -sf $INSTALLATION_DIR/icons/128x128/virtualbox.png /usr/share/pixmaps/virtualbox.png
     ln -sf $INSTALLATION_DIR/virtualbox.desktop /usr/share/applications/virtualbox.desktop
@@ -416,6 +430,9 @@ if [ "$ACTION" = "install" ]; then
     ln -sf VBoxManage /usr/bin/vboxmanage > /dev/null 2>&1
     ln -sf VBoxSDL /usr/bin/vboxsdl > /dev/null 2>&1
     ln -sf VBoxHeadless /usr/bin/vboxheadless > /dev/null 2>&1
+    if [ -f $INSTALLATION_DIR/VBoxDTrace ]; then
+        ln -sf VBoxDTrace /usr/bin/vboxdtrace > /dev/null 2>&1
+    fi
 
     # Icons
     cur=`pwd`
@@ -424,7 +441,7 @@ if [ "$ACTION" = "install" ]; then
         cd $i
         if [ -d /usr/share/icons/hicolor/$i ]; then
             for j in *; do
-                if [ "$j" = "virtualbox.png" ]; then
+                if expr "$j" : "virtualbox\..*" > /dev/null; then
                     dst=apps
                 else
                     dst=mimetypes
@@ -452,16 +469,6 @@ if [ "$ACTION" = "install" ]; then
 
     install_device_node_setup "$VBOXDRV_GRP" "$VBOXDRV_MODE" "$INSTALLATION_DIR"
 
-    # Write the configuration. Do this before we call /etc/init.d/vboxdrv setup!
-    echo "# VirtualBox installation directory" > $CONFIG_DIR/$CONFIG
-    echo "INSTALL_DIR='$INSTALLATION_DIR'" >> $CONFIG_DIR/$CONFIG
-    echo "# VirtualBox version" >> $CONFIG_DIR/$CONFIG
-    echo "INSTALL_VER='$VERSION'" >> $CONFIG_DIR/$CONFIG
-    echo "INSTALL_REV='$SVNREV'" >> $CONFIG_DIR/$CONFIG
-    echo "# Build type and user name for logging purposes" >> $CONFIG_DIR/$CONFIG
-    echo "BUILD_TYPE='$BUILD_BUILDTYPE'" >> $CONFIG_DIR/$CONFIG
-    echo "USERNAME='$BUILD_USERNAME'" >> $CONFIG_DIR/$CONFIG
-
     # Make kernel module
     MODULE_FAILED="false"
     if [ "$BUILD_MODULE" = "true" ]
@@ -470,7 +477,7 @@ if [ "$ACTION" = "install" ]; then
         log "Output from the module build process (the Linux kernel build system) follows:"
         cur=`pwd`
         log ""
-        setup_init_script vboxdrv
+        ./vboxdrv.sh setup
         # Start VirtualBox kernel module
         if [ $RETVAL -eq 0 ] && ! start_init_script vboxdrv; then
             info "Failed to load the kernel module."
