@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2014 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,6 +18,7 @@
 #ifndef ____H_CONSOLEVRDPSERVER
 #define ____H_CONSOLEVRDPSERVER
 
+#include "VRDEServerInfoWrap.h"
 #include "RemoteUSBBackend.h"
 #include "HGCM.h"
 
@@ -29,6 +30,7 @@
 #include <VBox/RemoteDesktop/VRDETSMF.h>
 #define VRDE_VIDEOIN_WITH_VRDEINTERFACE /* Get the VRDE interface definitions. */
 #include <VBox/RemoteDesktop/VRDEVideoIn.h>
+#include <VBox/RemoteDesktop/VRDEInput.h>
 
 #include <VBox/HostServices/VBoxClipboardExt.h>
 #include <VBox/HostServices/VBoxHostChannel.h>
@@ -38,6 +40,8 @@
 // ConsoleVRDPServer
 ///////////////////////////////////////////////////////////////////////////////
 
+class EmWebcam;
+
 typedef struct _VRDPInputSynch
 {
     int cGuestNumLockAdaptions;
@@ -142,10 +146,6 @@ public:
                             uint32_t cBits);
 
     void SendAudioInputEnd(void *pvUserCtx);
-#ifdef VBOX_WITH_USB_VIDEO
-    int GetVideoFrameDimensions(uint16_t *pu16Heigh, uint16_t *pu16Width);
-    int SendVideoSreamOn(bool fFetch);
-#endif
 
     int SCardRequest(void *pvUser, uint32_t u32Function, const void *pvData, uint32_t cbData);
 
@@ -155,6 +155,11 @@ public:
     int VideoInControl(void *pvUser, const VRDEVIDEOINDEVICEHANDLE *pDeviceHandle,
                        const VRDEVIDEOINCTRLHDR *pReq, uint32_t cbReq);
 
+    Console *getConsole(void) { return mConsole; }
+
+    void onMousePointerShapeChange(BOOL visible, BOOL alpha, ULONG xHot, ULONG yHot,
+                                   ULONG width, ULONG height, ComSafeArrayIn(BYTE,shape));
+
 private:
     /* Note: This is not a ComObjPtr here, because the ConsoleVRDPServer object
      * is actually just a part of the Console.
@@ -191,11 +196,13 @@ private:
     static DECLCALLBACK(void) VRDPCallbackVideoModeHint     (void *pvCallback, unsigned cWidth, unsigned cHeight,  unsigned cBitsPerPixel, unsigned uScreenId);
     static DECLCALLBACK(void) VRDECallbackAudioIn           (void *pvCallback, void *pvCtx, uint32_t u32ClientId, uint32_t u32Event, const void *pvData, uint32_t cbData);
 
+    void fetchCurrentState(void);
+
     bool m_fGuestWantsAbsolute;
     int m_mousex;
     int m_mousey;
 
-    IFramebuffer *maFramebuffers[SchemaDefs::MaxGuestMonitors];
+    ComPtr<IDisplaySourceBitmap> maSourceBitmaps[SchemaDefs::MaxGuestMonitors];
 
     ComPtr<IEventListener> mConsoleListener;
 
@@ -247,6 +254,8 @@ private:
 
     uint32_t volatile mu32AudioInputClientId;
 
+    int32_t volatile mcClients;
+
     static DECLCALLBACK(void) H3DORBegin(const void *pvContext, void **ppvInstance,
                                          const char *pszFormat);
     static DECLCALLBACK(void) H3DORGeometry(void *pvInstance,
@@ -259,7 +268,7 @@ private:
     static DECLCALLBACK(int)  H3DORContextProperty(const void *pvContext, uint32_t index,
                                                    void *pvBuffer, uint32_t cbBuffer, uint32_t *pcbOut);
 
-    void remote3DRedirect(void);
+    void remote3DRedirect(bool fEnable);
 
     /*
      * VRDE server optional interfaces.
@@ -339,56 +348,62 @@ private:
                                                        void *pDeviceCtx,
                                                        const VRDEVIDEOINPAYLOADHDR *pFrame,
                                                        uint32_t cbFrame);
+    EmWebcam *mEmWebcam;
+
+    /* Input interface. */
+    VRDEINPUTINTERFACE m_interfaceInput;
+    VRDEINPUTCALLBACKS m_interfaceCallbacksInput;
+    static DECLCALLBACK(void) VRDECallbackInputSetup(void *pvCallback,
+                                                     int rcRequest,
+                                                     uint32_t u32Method,
+                                                     const void *pvResult,
+                                                     uint32_t cbResult);
+    static DECLCALLBACK(void) VRDECallbackInputEvent(void *pvCallback,
+                                                     uint32_t u32Method,
+                                                     const void *pvEvent,
+                                                     uint32_t cbEvent);
+    uint64_t mu64TouchInputTimestampMCS;
 };
 
 
 class Console;
 
 class ATL_NO_VTABLE VRDEServerInfo :
-    public VirtualBoxBase,
-    VBOX_SCRIPTABLE_IMPL(IVRDEServerInfo)
+    public VRDEServerInfoWrap
 {
 public:
-
-    VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT(VRDEServerInfo, IVRDEServerInfo)
-
     DECLARE_NOT_AGGREGATABLE(VRDEServerInfo)
 
-    DECLARE_PROTECT_FINAL_CONSTRUCT()
-
-    BEGIN_COM_MAP(VRDEServerInfo)
-        VBOX_DEFAULT_INTERFACE_ENTRIES(IVRDEServerInfo)
-    END_COM_MAP()
-
-    DECLARE_EMPTY_CTOR_DTOR (VRDEServerInfo)
+    DECLARE_EMPTY_CTOR_DTOR(VRDEServerInfo)
 
     HRESULT FinalConstruct();
     void FinalRelease();
 
     /* Public initializer/uninitializer for internal purposes only. */
-    HRESULT init (Console *aParent);
+    HRESULT init(Console *aParent);
     void uninit();
 
-    /* IVRDEServerInfo properties */
-    #define DECL_GETTER(_aType, _aName) STDMETHOD(COMGETTER(_aName)) (_aType *a##_aName)
-        DECL_GETTER (BOOL,    Active);
-        DECL_GETTER (LONG,    Port);
-        DECL_GETTER (ULONG,   NumberOfClients);
-        DECL_GETTER (LONG64,  BeginTime);
-        DECL_GETTER (LONG64,  EndTime);
-        DECL_GETTER (LONG64,  BytesSent);
-        DECL_GETTER (LONG64,  BytesSentTotal);
-        DECL_GETTER (LONG64,  BytesReceived);
-        DECL_GETTER (LONG64,  BytesReceivedTotal);
-        DECL_GETTER (BSTR,    User);
-        DECL_GETTER (BSTR,    Domain);
-        DECL_GETTER (BSTR,    ClientName);
-        DECL_GETTER (BSTR,    ClientIP);
-        DECL_GETTER (ULONG,   ClientVersion);
-        DECL_GETTER (ULONG,   EncryptionStyle);
-    #undef DECL_GETTER
-
 private:
+    // wrapped IVRDEServerInfo properties
+#define DECL_GETTER(_aType, _aName) virtual HRESULT get##_aName(_aType *a##_aName)
+#define DECL_GETTER_REF(_aType, _aName) virtual HRESULT get##_aName(_aType &a##_aName)
+    DECL_GETTER(BOOL, Active);
+    DECL_GETTER(LONG, Port);
+    DECL_GETTER(ULONG, NumberOfClients);
+    DECL_GETTER(LONG64, BeginTime);
+    DECL_GETTER(LONG64, EndTime);
+    DECL_GETTER(LONG64, BytesSent);
+    DECL_GETTER(LONG64, BytesSentTotal);
+    DECL_GETTER(LONG64, BytesReceived);
+    DECL_GETTER(LONG64, BytesReceivedTotal);
+    DECL_GETTER_REF(com::Utf8Str, User);
+    DECL_GETTER_REF(com::Utf8Str, Domain);
+    DECL_GETTER_REF(com::Utf8Str, ClientName);
+    DECL_GETTER_REF(com::Utf8Str, ClientIP);
+    DECL_GETTER(ULONG, ClientVersion);
+    DECL_GETTER(ULONG, EncryptionStyle);
+#undef DECL_GETTER_REF
+#undef DECL_GETTER
 
     Console * const         mParent;
 };
