@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2010 Oracle Corporation
+ * Copyright (C) 2008-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,11 +17,13 @@
 
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_MAIN
 
+#define NETIF_WITHOUT_NETCFG
+
 #include <iprt/asm.h>
 #include <iprt/err.h>
 #include <list>
@@ -319,7 +321,7 @@ static HRESULT netIfNetworkInterfaceHelperClient(SVCHlpClient *aClient,
                         rc = d->iface->init(Bstr(name), Bstr(name), guid, HostNetworkInterfaceType_HostOnly);
                         if (SUCCEEDED(rc))
                         {
-                            rc = d->iface->setVirtualBox(d->vBox);
+                            rc = d->iface->i_setVirtualBox(d->vBox);
                             if (SUCCEEDED(rc))
                             {
                                 rc = d->iface->updateConfig();
@@ -335,7 +337,8 @@ static HRESULT netIfNetworkInterfaceHelperClient(SVCHlpClient *aClient,
                         vrc = aClient->read(errMsg);
                         if (RT_FAILURE(vrc)) break;
 
-                        rc = E_FAIL;//TODO: setError(E_FAIL, errMsg);
+                        rc = E_FAIL;
+                        d->iface->setError(E_FAIL, errMsg.c_str());
                         endLoop = true;
                         break;
                     }
@@ -388,7 +391,8 @@ static HRESULT netIfNetworkInterfaceHelperClient(SVCHlpClient *aClient,
                         vrc = aClient->read(errMsg);
                         if (RT_FAILURE(vrc)) break;
 
-                        rc = E_FAIL; // TODO: setError(E_FAIL, errMsg);
+                        rc = E_FAIL;
+                        d->iface->setError(E_FAIL, errMsg.c_str());
                         endLoop = true;
                         break;
                     }
@@ -441,7 +445,8 @@ static HRESULT netIfNetworkInterfaceHelperClient(SVCHlpClient *aClient,
                         vrc = aClient->read(errMsg);
                         if (RT_FAILURE(vrc)) break;
 
-                        rc = E_FAIL; // TODO: setError(E_FAIL, errMsg);
+                        rc = E_FAIL;
+                        d->iface->setError(E_FAIL, errMsg.c_str());
                         endLoop = true;
                         break;
                     }
@@ -498,7 +503,8 @@ static HRESULT netIfNetworkInterfaceHelperClient(SVCHlpClient *aClient,
                         vrc = aClient->read(errMsg);
                         if (RT_FAILURE(vrc)) break;
 
-                        rc = E_FAIL; // TODO: setError(E_FAIL, errMsg);
+                        rc = E_FAIL;
+                        d->iface->setError(E_FAIL, errMsg.c_str());
                         endLoop = true;
                         break;
                     }
@@ -555,7 +561,8 @@ static HRESULT netIfNetworkInterfaceHelperClient(SVCHlpClient *aClient,
                         vrc = aClient->read(errMsg);
                         if (RT_FAILURE(vrc)) break;
 
-                        rc = E_FAIL; // TODO: setError(E_FAIL, errMsg);
+                        rc = E_FAIL;
+                        d->iface->setError(E_FAIL, errMsg.c_str());
                         endLoop = true;
                         break;
                     }
@@ -608,7 +615,8 @@ static HRESULT netIfNetworkInterfaceHelperClient(SVCHlpClient *aClient,
                         vrc = aClient->read(errMsg);
                         if (RT_FAILURE(vrc)) break;
 
-                        rc = E_FAIL; // TODO: setError(E_FAIL, errMsg);
+                        rc = E_FAIL;
+                        d->iface->setError(E_FAIL, errMsg.c_str());
                         endLoop = true;
                         break;
                     }
@@ -667,7 +675,8 @@ int netIfNetworkInterfaceHelperServer(SVCHlpClient *aClient,
             Bstr name;
             Bstr bstrErr;
 
-            hrc = VBoxNetCfgWinCreateHostOnlyNetworkInterface(NULL, false, guid.asOutParam(), name.asOutParam(), bstrErr.asOutParam());
+            hrc = VBoxNetCfgWinCreateHostOnlyNetworkInterface(NULL, false, guid.asOutParam(), name.asOutParam(),
+                                                              bstrErr.asOutParam());
 
             if (hrc == S_OK)
             {
@@ -954,32 +963,38 @@ static int vboxNetWinAddComponent(std::list<ComObjPtr<HostNetworkInterface> > *
         if (hr == S_OK)
         {
             NETIFINFO Info;
-            memset(&Info, 0, sizeof(Info));
+            RT_ZERO(Info);
             Info.Uuid = *(Guid(IfGuid).raw());
             rc = collectNetIfInfo(name, Guid(IfGuid), &Info, iDefaultInterface);
             if (RT_FAILURE(rc))
             {
-                Log(("vboxNetWinAddComponent: collectNetIfInfo() -> %Rrc\n", rc));
+                LogRel(("vboxNetWinAddComponent: collectNetIfInfo() -> %Rrc\n", rc));
             }
+            Log(("vboxNetWinAddComponent: adding %ls\n", lpszName));
             /* create a new object and add it to the list */
             ComObjPtr<HostNetworkInterface> iface;
             iface.createObject();
             /* remove the curly bracket at the end */
-            if (SUCCEEDED(iface->init(name, enmType, &Info)))
+            rc = iface->init(name, enmType, &Info);
+            if (SUCCEEDED(rc))
             {
                 if (Info.bIsDefault)
                     pPist->push_front(iface);
                 else
                     pPist->push_back(iface);
-                rc = VINF_SUCCESS;
             }
             else
             {
+                LogRel(("vboxNetWinAddComponent: HostNetworkInterface::init() -> %Rrc\n", rc));
                 Assert(0);
             }
         }
+        else
+            LogRel(("vboxNetWinAddComponent: failed to get device instance GUID (0x%x)\n", hr));
         CoTaskMemFree(lpszName);
     }
+    else
+        LogRel(("vboxNetWinAddComponent: failed to get device display name (0x%x)\n", hr));
 
     return rc;
 }
@@ -987,67 +1002,59 @@ static int vboxNetWinAddComponent(std::list<ComObjPtr<HostNetworkInterface> > *
 #endif /* VBOX_WITH_NETFLT */
 
 
-static int netIfListHostAdapters(std::list<ComObjPtr<HostNetworkInterface> > &list)
+static int netIfListHostAdapters(INetCfg *pNc, std::list<ComObjPtr<HostNetworkInterface> > &list)
 {
 #ifndef VBOX_WITH_NETFLT
     /* VBoxNetAdp is available only when VBOX_WITH_NETFLT is enabled */
     return VERR_NOT_IMPLEMENTED;
 #else /* #  if defined VBOX_WITH_NETFLT */
-    INetCfg              *pNc;
     INetCfgComponent     *pMpNcc;
-    LPWSTR               lpszApp = NULL;
     HRESULT              hr;
     IEnumNetCfgComponent  *pEnumComponent;
 
-    /* we are using the INetCfg API for getting the list of miniports */
-    hr = VBoxNetCfgWinQueryINetCfg(&pNc, FALSE,
-                       VBOX_APP_NAME,
-                       10000,
-                       &lpszApp);
-    Assert(hr == S_OK);
+    hr = pNc->EnumComponents(&GUID_DEVCLASS_NET, &pEnumComponent);
     if (hr == S_OK)
     {
-        hr = pNc->EnumComponents(&GUID_DEVCLASS_NET, &pEnumComponent);
-        if (hr == S_OK)
+        while ((hr = pEnumComponent->Next(1, &pMpNcc, NULL)) == S_OK)
         {
-            while ((hr = pEnumComponent->Next(1, &pMpNcc, NULL)) == S_OK)
+            LPWSTR pwszName;
+            ULONG uComponentStatus;
+            hr = pMpNcc->GetDisplayName(&pwszName);
+            if (hr == S_OK)
+                Log(("netIfListHostAdapters: %ls\n", pwszName));
+            else
+                LogRel(("netIfListHostAdapters: failed to get device display name (0x%x)\n", hr));
+            hr = pMpNcc->GetDeviceStatus(&uComponentStatus);
+            if (hr == S_OK)
             {
-                ULONG uComponentStatus;
-                hr = pMpNcc->GetDeviceStatus(&uComponentStatus);
-                if (hr == S_OK)
+                if (uComponentStatus == 0)
                 {
-                    if (uComponentStatus == 0)
+                    LPWSTR pId;
+                    hr = pMpNcc->GetId(&pId);
+                    Assert(hr == S_OK);
+                    if (hr == S_OK)
                     {
-                        LPWSTR pId;
-                        hr = pMpNcc->GetId(&pId);
-                        Assert(hr == S_OK);
-                        if (hr == S_OK)
+                        Log(("netIfListHostAdapters: id = %ls\n", pId));
+                        if (!_wcsnicmp(pId, L"sun_VBoxNetAdp", sizeof(L"sun_VBoxNetAdp")/2))
                         {
-                            if (!_wcsnicmp(pId, L"sun_VBoxNetAdp", sizeof(L"sun_VBoxNetAdp")/2))
-                            {
-                                vboxNetWinAddComponent(&list, pMpNcc, HostNetworkInterfaceType_HostOnly, -1);
-                            }
-                            CoTaskMemFree(pId);
+                            vboxNetWinAddComponent(&list, pMpNcc, HostNetworkInterfaceType_HostOnly, -1);
                         }
+                        CoTaskMemFree(pId);
                     }
+                    else
+                        LogRel(("netIfListHostAdapters: failed to get device id (0x%x)\n", hr));
                 }
-                pMpNcc->Release();
             }
-            Assert(hr == S_OK || hr == S_FALSE);
-
-            pEnumComponent->Release();
-        }
-        else
-        {
-            LogRel((__FUNCTION__": EnumComponents error (0x%x)", hr));
+            else
+                LogRel(("netIfListHostAdapters: failed to get device status (0x%x)\n", hr));
+            pMpNcc->Release();
         }
+        Assert(hr == S_OK || hr == S_FALSE);
 
-        VBoxNetCfgWinReleaseINetCfg(pNc, FALSE);
-    }
-    else if (lpszApp)
-    {
-        CoTaskMemFree(lpszApp);
+        pEnumComponent->Release();
     }
+    else
+        LogRel(("netIfListHostAdapters: EnumComponents error (0x%x)\n", hr));
 #endif /* #  if defined VBOX_WITH_NETFLT */
     return VINF_SUCCESS;
 }
@@ -1082,7 +1089,34 @@ int NetIfGetConfigByName(PNETIFINFO)
     return VERR_NOT_IMPLEMENTED;
 }
 
-int NetIfCreateHostOnlyNetworkInterface(VirtualBox *pVBox,
+/**
+ * Obtain the current state of the interface.
+ *
+ * @returns VBox status code.
+ *
+ * @param   pcszIfName  Interface name.
+ * @param   penmState   Where to store the retrieved state.
+ */
+int NetIfGetState(const char *pcszIfName, NETIFSTATUS *penmState)
+{
+    return VERR_NOT_IMPLEMENTED;
+}
+
+/**
+ * Retrieve the physical link speed in megabits per second. If the interface is
+ * not up or otherwise unavailable the zero speed is returned.
+ *
+ * @returns VBox status code.
+ *
+ * @param   pcszIfName  Interface name.
+ * @param   puMbits     Where to store the link speed.
+ */
+int NetIfGetLinkSpeed(const char * /*pcszIfName*/, uint32_t * /*puMbits*/)
+{
+    return VERR_NOT_IMPLEMENTED;
+}
+
+int NetIfCreateHostOnlyNetworkInterface(VirtualBox *pVirtualBox,
                                         IHostNetworkInterface **aHostNetworkInterface,
                                         IProgress **aProgress,
                                         const char *pcszName)
@@ -1095,10 +1129,10 @@ int NetIfCreateHostOnlyNetworkInterface(VirtualBox *pVBox,
     progress.createObject();
 
     ComPtr<IHost> host;
-    HRESULT rc = pVBox->COMGETTER(Host)(host.asOutParam());
+    HRESULT rc = pVirtualBox->COMGETTER(Host)(host.asOutParam());
     if (SUCCEEDED(rc))
     {
-        rc = progress->init(pVBox, host,
+        rc = progress->init(pVirtualBox, host,
                             Bstr(_T("Creating host only network interface")).raw(),
                             FALSE /* aCancelable */);
         if (SUCCEEDED(rc))
@@ -1119,13 +1153,12 @@ int NetIfCreateHostOnlyNetworkInterface(VirtualBox *pVBox,
             d->msgCode = SVCHlpMsg::CreateHostOnlyNetworkInterface;
 //            d->name = aName;
             d->iface = iface;
-            d->vBox = pVBox;
-
-            rc = pVBox->startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
-                                             netIfNetworkInterfaceHelperClient,
-                                             static_cast<void *>(d.get()),
-                                             progress);
+            d->vBox = pVirtualBox;
 
+            rc = pVirtualBox->i_startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
+                                                     netIfNetworkInterfaceHelperClient,
+                                                     static_cast<void *>(d.get()),
+                                                     progress);
             if (SUCCEEDED(rc))
             {
                 /* d is now owned by netIfNetworkInterfaceHelperClient(), so release it */
@@ -1138,7 +1171,7 @@ int NetIfCreateHostOnlyNetworkInterface(VirtualBox *pVBox,
 #endif
 }
 
-int NetIfRemoveHostOnlyNetworkInterface(VirtualBox *pVBox, IN_GUID aId,
+int NetIfRemoveHostOnlyNetworkInterface(VirtualBox *pVirtualBox, IN_GUID aId,
                                         IProgress **aProgress)
 {
 #ifndef VBOX_WITH_NETFLT
@@ -1148,10 +1181,10 @@ int NetIfRemoveHostOnlyNetworkInterface(VirtualBox *pVBox, IN_GUID aId,
     ComObjPtr<Progress> progress;
     progress.createObject();
     ComPtr<IHost> host;
-    HRESULT rc = pVBox->COMGETTER(Host)(host.asOutParam());
+    HRESULT rc = pVirtualBox->COMGETTER(Host)(host.asOutParam());
     if (SUCCEEDED(rc))
     {
-        rc = progress->init(pVBox, host,
+        rc = progress->init(pVirtualBox, host,
                            Bstr(_T("Removing host network interface")).raw(),
                            FALSE /* aCancelable */);
         if (SUCCEEDED(rc))
@@ -1167,10 +1200,10 @@ int NetIfRemoveHostOnlyNetworkInterface(VirtualBox *pVBox, IN_GUID aId,
             d->msgCode = SVCHlpMsg::RemoveHostOnlyNetworkInterface;
             d->guid = aId;
 
-            rc = pVBox->startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
-                                             netIfNetworkInterfaceHelperClient,
-                                             static_cast<void *>(d.get()),
-                                             progress);
+            rc = pVirtualBox->i_startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
+                                                     netIfNetworkInterfaceHelperClient,
+                                                     static_cast<void *>(d.get()),
+                                                     progress);
 
             if (SUCCEEDED(rc))
             {
@@ -1224,10 +1257,10 @@ int NetIfEnableStaticIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf, ULON
                     d->u.StaticIP.IPAddress = ip;
                     d->u.StaticIP.IPNetMask = mask;
 
-                    rc = vBox->startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
-                                                    netIfNetworkInterfaceHelperClient,
-                                                    static_cast<void *>(d.get()),
-                                                    progress);
+                    rc = vBox->i_startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
+                                                      netIfNetworkInterfaceHelperClient,
+                                                      static_cast<void *>(d.get()),
+                                                      progress);
 
                     if (SUCCEEDED(rc))
                     {
@@ -1245,7 +1278,8 @@ int NetIfEnableStaticIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf, ULON
 #endif
 }
 
-int NetIfEnableStaticIpConfigV6(VirtualBox *vBox, HostNetworkInterface * pIf, IN_BSTR aOldIPV6Address, IN_BSTR aIPV6Address, ULONG aIPV6MaskPrefixLength)
+int NetIfEnableStaticIpConfigV6(VirtualBox *vBox, HostNetworkInterface * pIf, IN_BSTR aOldIPV6Address,
+                                IN_BSTR aIPV6Address, ULONG aIPV6MaskPrefixLength)
 {
 #ifndef VBOX_WITH_NETFLT
     return VERR_NOT_IMPLEMENTED;
@@ -1285,10 +1319,10 @@ int NetIfEnableStaticIpConfigV6(VirtualBox *vBox, HostNetworkInterface * pIf, IN
                     d->u.StaticIPV6.IPV6Address = aIPV6Address;
                     d->u.StaticIPV6.IPV6NetMaskLength = aIPV6MaskPrefixLength;
 
-                    rc = vBox->startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
-                                                    netIfNetworkInterfaceHelperClient,
-                                                    static_cast<void *>(d.get()),
-                                                    progress);
+                    rc = vBox->i_startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
+                                                      netIfNetworkInterfaceHelperClient,
+                                                      static_cast<void *>(d.get()),
+                                                      progress);
 
                     if (SUCCEEDED(rc))
                     {
@@ -1344,10 +1378,10 @@ int NetIfEnableDynamicIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf)
                     d->guid = guid;
                     d->iface = pIf;
 
-                    rc = vBox->startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
-                                                    netIfNetworkInterfaceHelperClient,
-                                                    static_cast<void *>(d.get()),
-                                                    progress);
+                    rc = vBox->i_startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
+                                                      netIfNetworkInterfaceHelperClient,
+                                                      static_cast<void *>(d.get()),
+                                                      progress);
 
                     if (SUCCEEDED(rc))
                     {
@@ -1403,10 +1437,10 @@ int NetIfDhcpRediscover(VirtualBox *vBox, HostNetworkInterface * pIf)
                     d->guid = guid;
                     d->iface = pIf;
 
-                    rc = vBox->startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
-                                                    netIfNetworkInterfaceHelperClient,
-                                                    static_cast<void *>(d.get()),
-                                                    progress);
+                    rc = vBox->i_startSVCHelperClient(IsUACEnabled() == TRUE /* aPrivileged */,
+                                                      netIfNetworkInterfaceHelperClient,
+                                                      static_cast<void *>(d.get()),
+                                                      progress);
 
                     if (SUCCEEDED(rc))
                     {
@@ -1424,12 +1458,13 @@ int NetIfDhcpRediscover(VirtualBox *vBox, HostNetworkInterface * pIf)
 #endif
 }
 
+#ifndef NETIF_WITHOUT_NETCFG
 int NetIfList(std::list<ComObjPtr<HostNetworkInterface> > &list)
 {
 #ifndef VBOX_WITH_NETFLT
     return VERR_NOT_IMPLEMENTED;
 #else /* #  if defined VBOX_WITH_NETFLT */
-    INetCfg              *pNc;
+    INetCfg              *pNc = NULL;
     INetCfgComponent     *pMpNcc;
     INetCfgComponent     *pTcpIpNcc;
     LPWSTR               lpszApp;
@@ -1440,20 +1475,49 @@ int NetIfList(std::list<ComObjPtr<HostNetworkInterface> > &list)
     INetCfgBindingInterface *pBi;
     int                  iDefault = getDefaultInterfaceIndex();
 
+    Log(("NetIfList: building the list of interfaces\n"));
     /* we are using the INetCfg API for getting the list of miniports */
     hr = VBoxNetCfgWinQueryINetCfg(&pNc, FALSE,
                        VBOX_APP_NAME,
                        10000,
                        &lpszApp);
     Assert(hr == S_OK);
+    if (hr != S_OK)
+    {
+        if (pNc)
+            pNc->Release();
+        pNc = NULL;
+        LogRel(("NetIfList: failed to acquire INetCfg interface (0x%x), trying CoCreateInstance...\n", hr));
+        hr = CoCreateInstance(CLSID_CNetCfg, NULL, CLSCTX_INPROC_SERVER, IID_INetCfg, (PVOID*)&pNc);
+        if (SUCCEEDED(hr))
+        {
+            hr = pNc->Initialize(NULL);
+            if (FAILED(hr))
+            {
+                LogRel(("NetIfList: INetCfg::Initialize failed with 0x%x\n", hr));
+                if (pNc)
+                    pNc->Release();
+                pNc = NULL;
+            }
+        }
+        else
+            LogRel(("NetIfList: CoCreateInstance failed with 0x%x\n", hr));
+    }
+
     if (hr == S_OK)
     {
 # ifdef VBOX_NETFLT_ONDEMAND_BIND
         /* for the protocol-based approach for now we just get all miniports the MS_TCPIP protocol binds to */
         hr = pNc->FindComponent(L"MS_TCPIP", &pTcpIpNcc);
 # else
-        /* for the filter-based approach we get all miniports our filter (sun_VBoxNetFlt)is bound to */
-        hr = pNc->FindComponent(L"sun_VBoxNetFlt", &pTcpIpNcc);
+        /* for the filter-based approach we get all miniports our filter (oracle_VBoxNetLwf)is bound to */
+        hr = pNc->FindComponent(L"oracle_VBoxNetLwf", &pTcpIpNcc);
+        if (hr != S_OK)
+        {
+            LogRel(("NetIfList: could not find VBoxNetLwf component (error 0x%x), trying VBoxNetFlt instead\n", hr));
+            /* fall back to NDIS5 miniport lookup (sun_VBoxNetFlt) */
+            hr = pNc->FindComponent(L"sun_VBoxNetFlt", &pTcpIpNcc);
+        }
 #  ifndef VBOX_WITH_HARDENING
         if (hr != S_OK)
         {
@@ -1480,12 +1544,13 @@ int NetIfList(std::list<ComObjPtr<HostNetworkInterface> > &list)
                     {
                         while ((hr = pEnumBp->Next(1, &pBp, NULL)) == S_OK)
                         {
+                            Log(("NetIfList: fetched INetCfgBindingPath interface\n"));
                             /* S_OK == enabled, S_FALSE == disabled */
                             if (pBp->IsEnabled() == S_OK)
                             {
                                 hr = pBp->EnumBindingInterfaces(&pEnumBi);
                                 Assert(hr == S_OK);
-                                if ( hr == S_OK )
+                                if (hr == S_OK)
                                 {
                                     hr = pEnumBi->Reset();
                                     Assert(hr == S_OK);
@@ -1493,48 +1558,259 @@ int NetIfList(std::list<ComObjPtr<HostNetworkInterface> > &list)
                                     {
                                         while ((hr = pEnumBi->Next(1, &pBi, NULL)) == S_OK)
                                         {
-                                            hr = pBi->GetLowerComponent( &pMpNcc );
+                                            Log(("NetIfList: fetched INetCfgBindingInterface interface\n"));
+                                            hr = pBi->GetLowerComponent(&pMpNcc);
                                             Assert(hr == S_OK);
                                             if (hr == S_OK)
                                             {
+                                                LPWSTR pwszName;
                                                 ULONG uComponentStatus;
+                                                hr = pMpNcc->GetDisplayName(&pwszName);
+                                                if (hr == S_OK)
+                                                    Log(("NetIfList: got %ls\n", pwszName));
+                                                else
+                                                    LogRel(("NetIfList: failed to get device display name (0x%x)\n", hr));
                                                 hr = pMpNcc->GetDeviceStatus(&uComponentStatus);
                                                 if (hr == S_OK)
                                                 {
                                                     if (uComponentStatus == 0)
                                                     {
-                                                        vboxNetWinAddComponent(&list, pMpNcc, HostNetworkInterfaceType_Bridged, iDefault);
+                                                        LPWSTR pId;
+                                                        hr = pMpNcc->GetId(&pId);
+                                                        Assert(hr == S_OK);
+                                                        if (hr == S_OK)
+                                                        {
+                                                            Log(("NetIfList: fetched network adapter id: %.80ls\n", pId));
+                                                            /*
+                                                             * Host-only interfaces are ignored here and included into the list
+                                                             * later in netIfListHostAdapters()
+                                                             */
+                                                            if (_wcsnicmp(pId, L"sun_VBoxNetAdp", sizeof(L"sun_VBoxNetAdp")/2))
+                                                            {
+                                                                vboxNetWinAddComponent(&list, pMpNcc, HostNetworkInterfaceType_Bridged,
+                                                                                       iDefault);
+                                                            }
+                                                            CoTaskMemFree(pId);
+                                                        }
+                                                        else
+                                                            LogRel(("NetIfList: failed to get device id (0x%x)\n", hr));
                                                     }
+                                                    else
+                                                        LogRel(("NetIfList: wrong device status (0x%x)\n", uComponentStatus));
                                                 }
+                                                else
+                                                    LogRel(("NetIfList: failed to get device status (0x%x)\n", hr));
                                                 pMpNcc->Release();
                                             }
+                                            else
+                                                LogRel(("NetIfList: failed to get lower component (0x%x)\n", hr));
                                             pBi->Release();
                                         }
                                         Assert(hr == S_OK || hr == S_FALSE);
                                     }
+                                    else
+                                        LogRel(("NetIfList: IEnumNetCfgBindingInterface::Reset failed (0x%x)\n", hr));
                                     pEnumBi->Release();
                                 }
+                                else
+                                    LogRel(("NetIfList: failed to enumerate binding interfaces (0x%x)\n", hr));
                             }
+                            else
+                                LogRel(("NetIfList: INetCfgBindingPath is disabled\n"));
                             pBp->Release();
                         }
                         Assert(hr == S_OK || hr == S_FALSE);
                     }
+                    else
+                        LogRel(("NetIfList: IEnumNetCfgBindingPath::Reset failed (0x%x)\n", hr));
                     pEnumBp->Release();
                 }
+                else
+                    LogRel(("NetIfList: EnumBindingPaths failed (0x%x)\n", hr));
                 pBindings->Release();
             }
+            else
+                LogRel(("NetIfList: failed to acquire INetCfgComponentBindings interface\n"));
             pTcpIpNcc->Release();
         }
         else
         {
-            LogRel(("failed to get the sun_VBoxNetFlt component, error (0x%x)", hr));
+            LogRel(("failed to get the oracle_VBoxNetLwf(sun_VBoxNetFlt) component, error (0x%x)\n", hr));
         }
 
+        /* Add host-only adapters to the list */
+        netIfListHostAdapters(pNc, list);
+
         VBoxNetCfgWinReleaseINetCfg(pNc, FALSE);
     }
 
-    netIfListHostAdapters(list);
-
     return VINF_SUCCESS;
 #endif /* #  if defined VBOX_WITH_NETFLT */
 }
+
+#else /* !NETIF_WITHOUT_NETCFG */
+int NetIfList(std::list<ComObjPtr<HostNetworkInterface> > &list)
+{
+    DWORD dwRc;
+    int rc = VINF_SUCCESS;
+    int iDefault = getDefaultInterfaceIndex();
+    /*
+     * Most of the hosts probably have less than 10 adapters,
+     * so we'll mostly succeed from the first attempt.
+     */
+    ULONG uBufLen = sizeof(IP_ADAPTER_ADDRESSES) * 10;
+    PIP_ADAPTER_ADDRESSES pAddresses = (PIP_ADAPTER_ADDRESSES)RTMemAlloc(uBufLen);
+    if (!pAddresses)
+        return VERR_NO_MEMORY;
+    dwRc = GetAdaptersAddresses(AF_UNSPEC, GAA_FLAG_INCLUDE_PREFIX, NULL, pAddresses, &uBufLen);
+    if (dwRc == ERROR_BUFFER_OVERFLOW)
+    {
+        /* Impressive! More than 10 adapters! Get more memory and try again. */
+        RTMemFree(pAddresses);
+        pAddresses = (PIP_ADAPTER_ADDRESSES)RTMemAlloc(uBufLen);
+        if (!pAddresses)
+            return VERR_NO_MEMORY;
+        dwRc = GetAdaptersAddresses(AF_UNSPEC, GAA_FLAG_INCLUDE_PREFIX, NULL, pAddresses, &uBufLen);
+    }
+    if (dwRc == NO_ERROR)
+    {
+        PIP_ADAPTER_ADDRESSES pAdapter;
+        for (pAdapter = pAddresses; pAdapter; pAdapter = pAdapter->Next)
+        {
+            Log(("Enumerating %s\n", pAdapter->AdapterName));
+            /* Vista+ systems introduced separate type for wireless adapters */
+            if (pAdapter->IfType != IF_TYPE_ETHERNET_CSMACD && pAdapter->IfType != IF_TYPE_IEEE80211)
+            {
+                LogRel(("Skipped non-Ethernet '%ls'\n", pAdapter->FriendlyName));
+                continue;
+            }
+
+            NETIFINFO Info;
+            RT_ZERO(Info);
+
+            if (pAdapter->AdapterName[0] == '{')
+            {
+                char *pszUuid = pAdapter->AdapterName + 1;
+                size_t len = strlen(pszUuid) - 1;
+                if (pszUuid[len] != '}')
+                {
+                    LogRel(("%s is not a valid UUID!\n", pAdapter->AdapterName));
+                    continue;
+                }
+                pszUuid[len] = 0;
+                rc = RTUuidFromStr(&Info.Uuid, pszUuid);
+                if (RT_FAILURE(rc))
+                {
+                    LogRel(("NetIfList: Failed to convert %s to UUID (%Rrc)\n", pszUuid, rc));
+                    continue;
+                }
+                bool fIPv4Found, fIPv6Found;
+                PIP_ADAPTER_UNICAST_ADDRESS pAddr;
+                fIPv4Found = fIPv6Found = false;
+                for (pAddr = pAdapter->FirstUnicastAddress;
+                     pAddr && !(fIPv4Found && fIPv6Found);
+                     pAddr = pAddr->Next)
+                {
+                    switch (pAddr->Address.lpSockaddr->sa_family)
+                    {
+                        case AF_INET:
+                            if (!fIPv4Found)
+                            {
+                                fIPv4Found = true;
+                                memcpy(&Info.IPAddress,
+                                       &((struct sockaddr_in *)pAddr->Address.lpSockaddr)->sin_addr.s_addr,
+                                       sizeof(Info.IPAddress));
+                            }
+                            break;
+                        case AF_INET6:
+                            if (!fIPv6Found)
+                            {
+                                fIPv6Found = true;
+                                memcpy(&Info.IPv6Address,
+                                       ((struct sockaddr_in6 *)pAddr->Address.lpSockaddr)->sin6_addr.s6_addr,
+                                       sizeof(Info.IPv6Address));
+                            }
+                            break;
+                    }
+                }
+                PIP_ADAPTER_PREFIX pPrefix;
+                fIPv4Found = fIPv6Found = false;
+                for (pPrefix = pAdapter->FirstPrefix;
+                     pPrefix && !(fIPv4Found && fIPv6Found);
+                     pPrefix = pPrefix->Next)
+                {
+                    switch (pPrefix->Address.lpSockaddr->sa_family)
+                    {
+                        case AF_INET:
+                            if (!fIPv4Found)
+                            {
+                                if (pPrefix->PrefixLength <= sizeof(Info.IPNetMask) * 8)
+                                {
+                                    fIPv4Found = true;
+                                    ASMBitSetRange(&Info.IPNetMask, 0, pPrefix->PrefixLength);
+                                }
+                                else
+                                    LogRel(("NetIfList: Unexpected IPv4 prefix length of %d\n",
+                                            pPrefix->PrefixLength));
+                            }
+                            break;
+                        case AF_INET6:
+                            if (!fIPv6Found)
+                            {
+                                if (pPrefix->PrefixLength <= sizeof(Info.IPv6NetMask) * 8)
+                                {
+                                    fIPv6Found = true;
+                                    ASMBitSetRange(&Info.IPv6NetMask, 0, pPrefix->PrefixLength);
+                                }
+                                else
+                                    LogRel(("NetIfList: Unexpected IPv6 prefix length of %d\n",
+                                            pPrefix->PrefixLength));
+                            }
+                            break;
+                    }
+                }
+                if (sizeof(Info.MACAddress) != pAdapter->PhysicalAddressLength)
+                    LogRel(("NetIfList: Unexpected physical address length: %u\n",
+                            pAdapter->PhysicalAddressLength));
+                else
+                    memcpy(Info.MACAddress.au8, pAdapter->PhysicalAddress, sizeof(Info.MACAddress));
+                Info.enmMediumType = NETIF_T_ETHERNET;
+                Info.enmStatus = pAdapter->OperStatus == IfOperStatusUp ? NETIF_S_UP : NETIF_S_DOWN;
+                Info.bDhcpEnabled = !!(pAdapter->Flags & IP_ADAPTER_DHCP_ENABLED);
+                Info.bIsDefault = (pAdapter->IfIndex == iDefault);
+                HostNetworkInterfaceType enmType;
+                /*
+                 * For some reason, I would not even want to speculate what it is, the users see
+                 * adapter's description as its name in its property dialog box.
+                 */
+                enmType = wcsncmp(pAdapter->Description, L"VirtualBox", 10) == 0
+                    ? HostNetworkInterfaceType_HostOnly
+                    : HostNetworkInterfaceType_Bridged;
+                Log(("Adding %ls as %s\n", pAdapter->Description,
+                     enmType == HostNetworkInterfaceType_Bridged ? "bridged" : "host-only"));
+                /* create a new object and add it to the list */
+                ComObjPtr<HostNetworkInterface> iface;
+                iface.createObject();
+                /* remove the curly bracket at the end */
+                rc = iface->init(pAdapter->Description, enmType, &Info);
+                if (SUCCEEDED(rc))
+                {
+                    if (Info.bIsDefault)
+                        list.push_front(iface);
+                    else
+                        list.push_back(iface);
+                }
+                else
+                {
+                    LogRel(("NetIfList: HostNetworkInterface::init() -> %Rrc\n", rc));
+                    Assert(0);
+                }
+            }
+        }
+    }
+
+    RTMemFree(pAddresses);
+
+    return VINF_SUCCESS;
+}
+#endif /* !NETIF_WITHOUT_NETCFG */
