@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,33 +25,50 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP RTLOGGROUP_LDR
 #include <iprt/ldr.h>
 #include "internal/iprt.h"
 
-#include <iprt/alloc.h>
 #include <iprt/assert.h>
-#include <iprt/log.h>
-#include <iprt/string.h>
 #include <iprt/err.h>
+#include <iprt/log.h>
+#include <iprt/md5.h>
+#include <iprt/mem.h>
+#include <iprt/sha.h>
+#include <iprt/string.h>
 #include "internal/ldr.h"
 #include "internal/ldrMZ.h"
 
+#ifdef LDR_ONLY_PE
+# undef LDR_WITH_PE
+# undef LDR_WITH_KLDR
+# undef LDR_WITH_ELF
+# undef LDR_WITH_LX
+# undef LDR_WITH_LE
+# undef LDR_WITH_NE
+# undef LDR_WITH_MZ
+# undef LDR_WITH_AOUT
+# define LDR_WITH_PE
+#endif
 
-/**
- * Open part with reader.
- *
- * @returns iprt status code.
- * @param   pReader     The loader reader instance which will provide the raw image bits.
- * @param   fFlags      Reserved, MBZ.
- * @param   enmArch     Architecture specifier.
- * @param   phMod       Where to store the handle.
- */
-int rtldrOpenWithReader(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phMod)
+
+RTDECL(int) RTLdrOpenWithReader(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phMod, PRTERRINFO pErrInfo)
 {
+    /*
+     * Resolve RTLDRARCH_HOST.
+     */
+    if (enmArch == RTLDRARCH_HOST)
+#if   defined(RT_ARCH_AMD64)
+        enmArch = RTLDRARCH_AMD64;
+#elif defined(RT_ARCH_X86)
+        enmArch = RTLDRARCH_X86_32;
+#else
+        enmArch = RTLDRARCH_WHATEVER;
+#endif
+
     /*
      * Read and verify the file signature.
      */
@@ -104,37 +121,37 @@ int rtldrOpenWithReader(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch
      */
     if (uSign.u32 == IMAGE_NT_SIGNATURE)
 #ifdef LDR_WITH_PE
-        rc = rtldrPEOpen(pReader, fFlags, enmArch, offHdr, phMod);
+        rc = rtldrPEOpen(pReader, fFlags, enmArch, offHdr, phMod, pErrInfo);
 #else
         rc = VERR_PE_EXE_NOT_SUPPORTED;
 #endif
     else if (uSign.u32 == IMAGE_ELF_SIGNATURE)
 #if defined(LDR_WITH_ELF)
-        rc = rtldrELFOpen(pReader, fFlags, enmArch, phMod);
+        rc = rtldrELFOpen(pReader, fFlags, enmArch, phMod, pErrInfo);
 #else
         rc = VERR_ELF_EXE_NOT_SUPPORTED;
 #endif
     else if (uSign.au16[0] == IMAGE_LX_SIGNATURE)
 #ifdef LDR_WITH_LX
-        rc = rtldrLXOpen(pReader, fFlags, enmArch, offHdr, phMod);
+        rc = rtldrLXOpen(pReader, fFlags, enmArch, offHdr, phMod, pErrInfo);
 #else
         rc = VERR_LX_EXE_NOT_SUPPORTED;
 #endif
     else if (uSign.au16[0] == IMAGE_LE_SIGNATURE)
 #ifdef LDR_WITH_LE
-        rc = rtldrLEOpen(pReader, fFlags, enmArch, phMod);
+        rc = rtldrLEOpen(pReader, fFlags, enmArch, phMod, pErrInfo);
 #else
         rc = VERR_LE_EXE_NOT_SUPPORTED;
 #endif
     else if (uSign.au16[0] == IMAGE_NE_SIGNATURE)
 #ifdef LDR_WITH_NE
-        rc = rtldrNEOpen(pReader, fFlags, enmArch, phMod);
+        rc = rtldrNEOpen(pReader, fFlags, enmArch, phMod, pErrInfo);
 #else
         rc = VERR_NE_EXE_NOT_SUPPORTED;
 #endif
     else if (uSign.au16[0] == IMAGE_DOS_SIGNATURE)
 #ifdef LDR_WITH_MZ
-        rc = rtldrMZOpen(pReader, fFlags, enmArch, phMod);
+        rc = rtldrMZOpen(pReader, fFlags, enmArch, phMod, pErrInfo);
 #else
         rc = VERR_MZ_EXE_NOT_SUPPORTED;
 #endif
@@ -142,7 +159,7 @@ int rtldrOpenWithReader(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch
              || uSign.u32 == IMAGE_AOUT_Z_SIGNATURE*/ /** @todo find the aout magics in emx or binutils. */
              0)
 #ifdef LDR_WITH_AOUT
-        rc = rtldrAOUTOpen(pReader, fFlags, enmArch, phMod);
+        rc = rtldrAOUTOpen(pReader, fFlags, enmArch, phMod, pErrInfo);
 #else
         rc = VERR_AOUT_EXE_NOT_SUPPORTED;
 #endif
@@ -157,7 +174,14 @@ int rtldrOpenWithReader(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch
 #ifdef LDR_WITH_KLDR
     /* Try kLdr if it's a format we don't recognize. */
     if (rc <= VERR_INVALID_EXE_SIGNATURE && rc > VERR_BAD_EXE_FORMAT)
-        rc = rtldrkLdrOpen(pReader, fFlags, enmArch, phMod);
+    {
+        int rc2 = rtldrkLdrOpen(pReader, fFlags, enmArch, phMod, pErrInfo);
+        if (   RT_SUCCESS(rc2)
+            || (rc == VERR_INVALID_EXE_SIGNATURE && rc2 != VERR_MZ_EXE_NOT_SUPPORTED /* Quick fix for bad return code. */)
+            || rc2 >  VERR_INVALID_EXE_SIGNATURE
+            || rc2 <= VERR_BAD_EXE_FORMAT)
+            rc = rc2;
+    }
 #endif
 
     LogFlow(("rtldrOpenWithReader: %s: returns %Rrc *phMod=%p\n", pReader->pfnLogName(pReader), rc, *phMod));
@@ -165,15 +189,6 @@ int rtldrOpenWithReader(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch
 }
 
 
-/**
- * Gets the size of the loaded image.
- * This is only supported for modules which has been opened using RTLdrOpen() and RTLdrOpenBits().
- *
- * @returns image size (in bytes).
- * @returns ~(size_t)0 on if not opened by RTLdrOpen().
- * @param   hLdrMod     Handle to the loader module.
- * @remark  Not supported for RTLdrLoad() images.
- */
 RTDECL(size_t) RTLdrSize(RTLDRMOD hLdrMod)
 {
     LogFlow(("RTLdrSize: hLdrMod=%RTldrm\n", hLdrMod));
@@ -205,6 +220,7 @@ RT_EXPORT_SYMBOL(RTLdrSize);
  *                          Must be as large as RTLdrSize() suggests.
  * @param   BaseAddress     The base address.
  * @param   pfnGetImport    Callback function for resolving imports one by one.
+ *                          If this is NULL, imports will not be resolved.
  * @param   pvUser          User argument for the callback.
  * @remark  Not supported for RTLdrLoad() images.
  */
@@ -217,8 +233,8 @@ RTDECL(int) RTLdrGetBits(RTLDRMOD hLdrMod, void *pvBits, RTLDRADDR BaseAddress,
      * Validate input.
      */
     AssertMsgReturn(rtldrIsValid(hLdrMod), ("hLdrMod=%p\n", hLdrMod), VERR_INVALID_HANDLE);
-    AssertMsgReturn(VALID_PTR(pvBits), ("pvBits=%p\n", pvBits), VERR_INVALID_PARAMETER);
-    AssertMsgReturn(VALID_PTR(pfnGetImport), ("pfnGetImport=%p\n", pfnGetImport), VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pvBits, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pfnGetImport, VERR_INVALID_POINTER);
     PRTLDRMODINTERNAL pMod = (PRTLDRMODINTERNAL)hLdrMod;
     AssertMsgReturn(pMod->eState == LDR_STATE_OPENED, ("eState=%d\n", pMod->eState), VERR_WRONG_ORDER);
 
@@ -271,46 +287,29 @@ RTDECL(int) RTLdrRelocate(RTLDRMOD hLdrMod, void *pvBits, RTLDRADDR NewBaseAddre
 RT_EXPORT_SYMBOL(RTLdrRelocate);
 
 
-/**
- * Gets the address of a named exported symbol.
- *
- * This function differs from the plain one in that it can deal with
- * both GC and HC address sizes, and that it can calculate the symbol
- * value relative to any given base address.
- *
- * @returns iprt status code.
- * @param   hLdrMod         The loader module handle.
- * @param   pvBits          Optional pointer to the loaded image.
- *                          Set this to NULL if no RTLdrGetBits() processed image bits are available.
- *                          Not supported for RTLdrLoad() images and must be NULL.
- * @param   BaseAddress     Image load address.
- *                          Not supported for RTLdrLoad() images and must be 0.
- * @param   pszSymbol       Symbol name.
- * @param   pValue          Where to store the symbol value.
- */
-RTDECL(int) RTLdrGetSymbolEx(RTLDRMOD hLdrMod, const void *pvBits, RTLDRADDR BaseAddress, const char *pszSymbol,
-                             PRTLDRADDR pValue)
+RTDECL(int) RTLdrGetSymbolEx(RTLDRMOD hLdrMod, const void *pvBits, RTLDRADDR BaseAddress,
+                             uint32_t iOrdinal, const char *pszSymbol, PRTLDRADDR pValue)
 {
-    LogFlow(("RTLdrGetSymbolEx: hLdrMod=%RTldrm pvBits=%p BaseAddress=%RTptr pszSymbol=%p:{%s} pValue\n",
-             hLdrMod, pvBits, BaseAddress, pszSymbol, pszSymbol, pValue));
+    LogFlow(("RTLdrGetSymbolEx: hLdrMod=%RTldrm pvBits=%p BaseAddress=%RTptr iOrdinal=%#x pszSymbol=%p:{%s} pValue=%p\n",
+             hLdrMod, pvBits, BaseAddress, iOrdinal, pszSymbol, pszSymbol, pValue));
 
     /*
      * Validate input.
      */
     AssertMsgReturn(rtldrIsValid(hLdrMod), ("hLdrMod=%p\n", hLdrMod), VERR_INVALID_HANDLE);
-    AssertMsgReturn(!pvBits || VALID_PTR(pvBits), ("pvBits=%p\n", pvBits), VERR_INVALID_PARAMETER);
-    AssertMsgReturn(pszSymbol, ("pszSymbol=%p\n", pszSymbol), VERR_INVALID_PARAMETER);
-    AssertMsgReturn(VALID_PTR(pValue), ("pValue=%p\n", pvBits), VERR_INVALID_PARAMETER);
+    AssertPtrNullReturn(pvBits, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pszSymbol, VERR_INVALID_POINTER);
+    AssertReturn(pszSymbol || iOrdinal != UINT32_MAX, VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pValue, VERR_INVALID_POINTER);
     PRTLDRMODINTERNAL pMod = (PRTLDRMODINTERNAL)hLdrMod;
-    //AssertMsgReturn(pMod->eState == LDR_STATE_OPENED, ("eState=%d\n", pMod->eState), VERR_WRONG_ORDER);
 
     /*
      * Do it.
      */
     int rc;
     if (pMod->pOps->pfnGetSymbolEx)
-        rc = pMod->pOps->pfnGetSymbolEx(pMod, pvBits, BaseAddress, pszSymbol, pValue);
-    else if (!BaseAddress && !pvBits)
+        rc = pMod->pOps->pfnGetSymbolEx(pMod, pvBits, BaseAddress, iOrdinal, pszSymbol, pValue);
+    else if (!BaseAddress && !pvBits && iOrdinal == UINT32_MAX)
     {
         void *pvValue;
         rc = pMod->pOps->pfnGetSymbol(pMod, pszSymbol, &pvValue);
@@ -325,6 +324,46 @@ RTDECL(int) RTLdrGetSymbolEx(RTLDRMOD hLdrMod, const void *pvBits, RTLDRADDR Bas
 RT_EXPORT_SYMBOL(RTLdrGetSymbolEx);
 
 
+RTDECL(int) RTLdrQueryForwarderInfo(RTLDRMOD hLdrMod, const void *pvBits, uint32_t iOrdinal, const char *pszSymbol,
+                                    PRTLDRIMPORTINFO pInfo, size_t cbInfo)
+{
+    LogFlow(("RTLdrQueryForwarderInfo: hLdrMod=%RTldrm pvBits=%p iOrdinal=%#x pszSymbol=%p:{%s} pInfo=%p cbInfo=%zu\n",
+             hLdrMod, pvBits, iOrdinal, pszSymbol, pszSymbol, pInfo, cbInfo));
+
+    /*
+     * Validate input.
+     */
+    AssertMsgReturn(rtldrIsValid(hLdrMod), ("hLdrMod=%p\n", hLdrMod), VERR_INVALID_HANDLE);
+    AssertPtrNullReturn(pvBits, VERR_INVALID_POINTER);
+    AssertMsgReturn(pszSymbol, ("pszSymbol=%p\n", pszSymbol), VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pInfo, VERR_INVALID_PARAMETER);
+    AssertReturn(cbInfo >= sizeof(*pInfo), VERR_INVALID_PARAMETER);
+    PRTLDRMODINTERNAL pMod = (PRTLDRMODINTERNAL)hLdrMod;
+
+    /*
+     * Do it.
+     */
+    int rc;
+    if (pMod->pOps->pfnQueryForwarderInfo)
+    {
+        rc = pMod->pOps->pfnQueryForwarderInfo(pMod, pvBits, iOrdinal, pszSymbol, pInfo, cbInfo);
+        if (RT_SUCCESS(rc))
+            LogFlow(("RTLdrQueryForwarderInfo: returns %Rrc pInfo={%#x,%#x,%s,%s}\n", rc,
+                     pInfo->iSelfOrdinal, pInfo->iOrdinal, pInfo->pszSymbol, pInfo->szModule));
+        else
+            LogFlow(("RTLdrQueryForwarderInfo: returns %Rrc\n", rc));
+    }
+    else
+    {
+        LogFlow(("RTLdrQueryForwarderInfo: returns VERR_NOT_SUPPORTED\n"));
+        rc = VERR_NOT_SUPPORTED;
+    }
+    return rc;
+
+}
+RT_EXPORT_SYMBOL(RTLdrQueryForwarderInfo);
+
+
 /**
  * Enumerates all symbols in a module.
  *
@@ -487,8 +526,7 @@ RT_EXPORT_SYMBOL(RTLdrLinkAddressToRva);
 
 RTDECL(int) RTLdrSegOffsetToRva(RTLDRMOD hLdrMod, uint32_t iSeg, RTLDRADDR offSeg, PRTLDRADDR pRva)
 {
-    LogFlow(("RTLdrSegOffsetToRva: hLdrMod=%RTldrm LinkAddress=%RTptr iSeg=%#x offSeg=%RTptr pRva=%p\n",
-             hLdrMod, iSeg, offSeg, pRva));
+    LogFlow(("RTLdrSegOffsetToRva: hLdrMod=%RTldrm iSeg=%#x offSeg=%RTptr pRva=%p\n", hLdrMod, iSeg, offSeg, pRva));
 
     /*
      * Validate input.
@@ -547,3 +585,147 @@ RTDECL(int) RTLdrRvaToSegOffset(RTLDRMOD hLdrMod, RTLDRADDR Rva, uint32_t *piSeg
 }
 RT_EXPORT_SYMBOL(RTLdrRvaToSegOffset);
 
+
+RTDECL(int) RTLdrQueryProp(RTLDRMOD hLdrMod, RTLDRPROP enmProp, void *pvBuf, size_t cbBuf)
+{
+    return RTLdrQueryPropEx(hLdrMod, enmProp, NULL /*pvBits*/, pvBuf, cbBuf, NULL);
+}
+RT_EXPORT_SYMBOL(RTLdrQueryProp);
+
+
+RTDECL(int) RTLdrQueryPropEx(RTLDRMOD hLdrMod, RTLDRPROP enmProp, void *pvBits, void *pvBuf, size_t cbBuf, size_t *pcbRet)
+{
+    AssertMsgReturn(rtldrIsValid(hLdrMod), ("hLdrMod=%p\n", hLdrMod), RTLDRENDIAN_INVALID);
+    PRTLDRMODINTERNAL pMod = (PRTLDRMODINTERNAL)hLdrMod;
+
+    AssertPtrNullReturn(pcbRet, VERR_INVALID_POINTER);
+    size_t cbRet;
+    if (!pcbRet)
+        pcbRet = &cbRet;
+
+    /*
+     * Do some pre screening of the input
+     */
+    switch (enmProp)
+    {
+        case RTLDRPROP_UUID:
+            *pcbRet = sizeof(RTUUID);
+            AssertReturn(cbBuf == sizeof(RTUUID), VERR_INVALID_PARAMETER);
+            break;
+        case RTLDRPROP_TIMESTAMP_SECONDS:
+            *pcbRet = sizeof(int64_t);
+            AssertReturn(cbBuf == sizeof(int32_t) || cbBuf == sizeof(int64_t), VERR_INVALID_PARAMETER);
+            break;
+        case RTLDRPROP_IS_SIGNED:
+            *pcbRet = sizeof(bool);
+            AssertReturn(cbBuf == sizeof(bool), VERR_INVALID_PARAMETER);
+            break;
+        case RTLDRPROP_PKCS7_SIGNED_DATA:
+            *pcbRet = 0;
+            break;
+        case RTLDRPROP_SIGNATURE_CHECKS_ENFORCED:
+            *pcbRet = sizeof(bool);
+            AssertReturn(cbBuf == sizeof(bool), VERR_INVALID_PARAMETER);
+            break;
+        case RTLDRPROP_IMPORT_COUNT:
+            *pcbRet = sizeof(uint32_t);
+            AssertReturn(cbBuf == sizeof(uint32_t), VERR_INVALID_PARAMETER);
+            break;
+        case RTLDRPROP_IMPORT_MODULE:
+            *pcbRet = sizeof(uint32_t);
+            AssertReturn(cbBuf >= sizeof(uint32_t), VERR_INVALID_PARAMETER);
+            break;
+
+        default:
+            AssertFailedReturn(VERR_INVALID_FUNCTION);
+    }
+    AssertPtrReturn(pvBuf, VERR_INVALID_POINTER);
+
+    /*
+     * Call the image specific worker, if there is one.
+     */
+    if (!pMod->pOps->pfnQueryProp)
+        return VERR_NOT_SUPPORTED;
+    return pMod->pOps->pfnQueryProp(pMod, enmProp, pvBits, pvBuf, cbBuf, pcbRet);
+}
+RT_EXPORT_SYMBOL(RTLdrQueryPropEx);
+
+
+RTDECL(int) RTLdrVerifySignature(RTLDRMOD hLdrMod, PFNRTLDRVALIDATESIGNEDDATA pfnCallback, void *pvUser, PRTERRINFO pErrInfo)
+{
+    AssertMsgReturn(rtldrIsValid(hLdrMod), ("hLdrMod=%p\n", hLdrMod), VERR_INVALID_HANDLE);
+    PRTLDRMODINTERNAL pMod = (PRTLDRMODINTERNAL)hLdrMod;
+    AssertPtrReturn(pfnCallback, VERR_INVALID_POINTER);
+
+    /*
+     * Call the image specific worker, if there is one.
+     */
+    if (!pMod->pOps->pfnVerifySignature)
+        return VERR_NOT_SUPPORTED;
+    return pMod->pOps->pfnVerifySignature(pMod, pfnCallback, pvUser, pErrInfo);
+}
+RT_EXPORT_SYMBOL(RTLdrVerifySignature);
+
+
+RTDECL(int) RTLdrHashImage(RTLDRMOD hLdrMod, RTDIGESTTYPE enmDigest, char *pszDigest, size_t cbDigest)
+{
+    AssertMsgReturn(rtldrIsValid(hLdrMod), ("hLdrMod=%p\n", hLdrMod), VERR_INVALID_HANDLE);
+    PRTLDRMODINTERNAL pMod = (PRTLDRMODINTERNAL)hLdrMod;
+
+    /*
+     * Make sure there is sufficient space for the wanted digest and that
+     * it's supported.
+     */
+    switch (enmDigest)
+    {
+        case RTDIGESTTYPE_MD5:      AssertReturn(cbDigest >= RTMD5_DIGEST_LEN    + 1, VERR_BUFFER_OVERFLOW); break;
+        case RTDIGESTTYPE_SHA1:     AssertReturn(cbDigest >= RTSHA1_DIGEST_LEN   + 1, VERR_BUFFER_OVERFLOW); break;
+        case RTDIGESTTYPE_SHA256:   AssertReturn(cbDigest >= RTSHA256_DIGEST_LEN + 1, VERR_BUFFER_OVERFLOW); break;
+        case RTDIGESTTYPE_SHA512:   AssertReturn(cbDigest >= RTSHA512_DIGEST_LEN + 1, VERR_BUFFER_OVERFLOW); break;
+        default:
+            if (enmDigest > RTDIGESTTYPE_INVALID && enmDigest < RTDIGESTTYPE_END)
+                return VERR_NOT_SUPPORTED;
+            AssertFailedReturn(VERR_INVALID_PARAMETER);
+    }
+    AssertPtrReturn(pszDigest, VERR_INVALID_POINTER);
+
+    /*
+     * Call the image specific worker, if there is one.
+     */
+    if (!pMod->pOps->pfnHashImage)
+        return VERR_NOT_SUPPORTED;
+    return pMod->pOps->pfnHashImage(pMod, enmDigest, pszDigest, cbDigest);
+}
+RT_EXPORT_SYMBOL(RTLdrHashImage);
+
+
+/**
+ * Internal method used by the IPRT debug bits.
+ *
+ * @returns IPRT status code.
+ * @param   hLdrMod             The loader handle which executable we wish to
+ *                              read from.
+ * @param   pvBuf               The output buffer.
+ * @param   iDbgInfo            The debug info ordinal number if the request
+ *                              corresponds exactly to a debug info part from
+ *                              pfnEnumDbgInfo.  Otherwise, pass UINT32_MAX.
+ * @param   off                 Where in the executable file to start reading.
+ * @param   cb                  The number of bytes to read.
+ *
+ * @remarks Fixups will only be applied if @a iDbgInfo is specified.
+ */
+DECLHIDDEN(int) rtLdrReadAt(RTLDRMOD hLdrMod, void *pvBuf, uint32_t iDbgInfo, RTFOFF off, size_t cb)
+{
+    AssertMsgReturn(rtldrIsValid(hLdrMod), ("hLdrMod=%p\n", hLdrMod), VERR_INVALID_HANDLE);
+    PRTLDRMODINTERNAL pMod = (PRTLDRMODINTERNAL)hLdrMod;
+
+    if (iDbgInfo != UINT32_MAX)
+    {
+        AssertReturn(pMod->pOps->pfnReadDbgInfo, VERR_NOT_SUPPORTED);
+        return pMod->pOps->pfnReadDbgInfo(pMod, iDbgInfo, off, cb, pvBuf);
+    }
+
+    AssertReturn(pMod->pReader, VERR_NOT_SUPPORTED);
+    return pMod->pReader->pfnRead(pMod->pReader, pvBuf, cb, off);
+}
+
