@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #ifdef RT_OS_WINDOWS
 # include <winsock2.h>
 #else
@@ -62,9 +62,9 @@
 #include "internal/socket.h"
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /* non-standard linux stuff (it seems). */
 #ifndef MSG_NOSIGNAL
 # define MSG_NOSIGNAL           0
@@ -93,9 +93,9 @@
 #define RTTCP_SERVER_BACKLOG    10
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * TCP Server state.
  */
@@ -142,9 +142,9 @@ typedef struct RTTCPSERVER
 } RTTCPSERVER;
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static DECLCALLBACK(int)  rtTcpServerThread(RTTHREAD ThreadSelf, void *pvServer);
 static int  rtTcpServerListen(PRTTCPSERVER pServer);
 static int  rtTcpServerListenCleanup(PRTTCPSERVER pServer);
@@ -804,12 +804,20 @@ RTR3DECL(int) RTTcpServerDestroy(PRTTCPSERVER pServer)
 
 
 RTR3DECL(int) RTTcpClientConnect(const char *pszAddress, uint32_t uPort, PRTSOCKET pSock)
+{
+    return RTTcpClientConnectEx(pszAddress, uPort, pSock, RT_SOCKETCONNECT_DEFAULT_WAIT, NULL);
+}
+
+
+RTR3DECL(int) RTTcpClientConnectEx(const char *pszAddress, uint32_t uPort, PRTSOCKET pSock,
+                                   RTMSINTERVAL cMillies, PRTTCPCLIENTCONNECTCANCEL volatile *ppCancelCookie)
 {
     /*
      * Validate input.
      */
     AssertReturn(uPort > 0, VERR_INVALID_PARAMETER);
     AssertPtrReturn(pszAddress, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(ppCancelCookie, VERR_INVALID_POINTER);
 
     /*
      * Resolve the address.
@@ -828,7 +836,25 @@ RTR3DECL(int) RTTcpClientConnect(const char *pszAddress, uint32_t uPort, PRTSOCK
     {
         RTSocketSetInheritance(Sock, false /*fInheritable*/);
 
-        rc = rtSocketConnect(Sock, &Addr);
+        if (!ppCancelCookie)
+            rc = rtSocketConnect(Sock, &Addr, cMillies);
+        else
+        {
+            RTSocketRetain(Sock);
+            if (ASMAtomicCmpXchgPtr(ppCancelCookie, (PRTTCPCLIENTCONNECTCANCEL)Sock, NULL))
+            {
+                rc = rtSocketConnect(Sock, &Addr, cMillies);
+                if (ASMAtomicCmpXchgPtr(ppCancelCookie, NULL, (PRTTCPCLIENTCONNECTCANCEL)Sock))
+                    RTSocketRelease(Sock);
+                else
+                    rc = VERR_CANCELLED;
+            }
+            else
+            {
+                RTSocketRelease(Sock);
+                rc = VERR_CANCELLED;
+            }
+        }
         if (RT_SUCCESS(rc))
         {
             *pSock = Sock;
@@ -837,10 +863,28 @@ RTR3DECL(int) RTTcpClientConnect(const char *pszAddress, uint32_t uPort, PRTSOCK
 
         rtTcpClose(Sock, "RTTcpClientConnect", false /*fTryGracefulShutdown*/);
     }
+    if (ppCancelCookie)
+        *ppCancelCookie = NULL;
     return rc;
 }
 
 
+RTR3DECL(int) RTTcpClientCancelConnect(PRTTCPCLIENTCONNECTCANCEL volatile *ppCancelCookie)
+{
+    AssertPtrReturn(ppCancelCookie, VERR_INVALID_POINTER);
+
+    AssertCompile(NIL_RTSOCKET == NULL);
+    RTSOCKET hSock = (RTSOCKET)ASMAtomicXchgPtr((void * volatile *)ppCancelCookie, (void *)(uintptr_t)0xdead9999);
+    if (hSock != NIL_RTSOCKET)
+    {
+        int rc = rtTcpClose(hSock, "RTTcpClientCancelConnect", false /*fTryGracefulShutdown*/);
+        AssertRCReturn(rc, rc);
+    }
+
+    return VINF_SUCCESS;
+}
+
+
 RTR3DECL(int) RTTcpClientClose(RTSOCKET Sock)
 {
     return rtTcpClose(Sock, "RTTcpClientClose", true /*fTryGracefulShutdown*/);
